@@ -12,9 +12,6 @@
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
-#include <wx/file.h>
-#include <wx/filename.h>
-
 #ifndef __WXMSW__
 	#define mutex mutex_override // Sadly on some platforms system headers include conflicting names
 	#include <netdb.h>
@@ -368,10 +365,10 @@ int CControlSocket::CheckOverwriteFile()
 
 	CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
 
-	if (pData->download)
-	{
-		if (!wxFile::Exists(pData->localFile))
+	if (pData->download) {
+		if (fz::local_filesys::get_file_type(fz::to_native(pData->localFile), true) != fz::local_filesys::file) {
 			return FZ_REPLY_OK;
+		}
 	}
 
 	CDirentry entry;
@@ -389,7 +386,7 @@ int CControlSocket::CheckOverwriteFile()
 		found = false;
 
 	if (!pData->download) {
-		if (!found && pData->remoteFileSize < 0 && !pData->fileTime.empty())
+		if (!found && pData->remoteFileSize < 0 && pData->fileTime.empty())
 			return FZ_REPLY_OK;
 	}
 
@@ -401,6 +398,7 @@ int CControlSocket::CheckOverwriteFile()
 	pNotification->remotePath = pData->remotePath;
 	pNotification->localSize = pData->localFileSize;
 	pNotification->remoteSize = pData->remoteFileSize;
+	pNotification->remoteTime = pData->fileTime;
 	pNotification->ascii = !pData->transferSettings.binary;
 
 	if (pData->download && pNotification->localSize >= 0)
@@ -412,15 +410,10 @@ int CControlSocket::CheckOverwriteFile()
 
 	pNotification->localTime = fz::local_filesys::get_modification_time(fz::to_native(pData->localFile));
 
-	if (pData->fileTime.empty())
-		pNotification->remoteTime = pData->fileTime;
-
 	if (found) {
-		if (!pData->fileTime.empty()) {
-			if (entry.has_date()) {
-				pNotification->remoteTime = entry.time;
-				pData->fileTime = entry.time;
-			}
+		if (pNotification->remoteTime.empty() && entry.has_date()) {
+			pNotification->remoteTime = entry.time;
+			pData->fileTime = entry.time;
 		}
 	}
 
@@ -440,15 +433,17 @@ CFileTransferOpData::~CFileTransferOpData()
 {
 }
 
-wxString CControlSocket::ConvToLocal(const char* buffer, size_t len)
+std::wstring CControlSocket::ConvToLocal(const char* buffer, size_t len)
 {
+	std::wstring ret;
+
 	size_t outLen{};
 	if (m_useUTF8) {
 		wxChar* out = ConvToLocalBuffer(buffer, wxConvUTF8, len, outLen);
 		if (out) {
-			wxString str(out, outLen - 1);
+			ret.assign(out, outLen - 1);
 			delete [] out;
-			return str;
+			return ret;
 		}
 
 		// Fall back to local charset on error
@@ -459,23 +454,24 @@ wxString CControlSocket::ConvToLocal(const char* buffer, size_t len)
 	}
 
 	if (m_pCSConv) {
-		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv, len, outLen);
+		wchar_t* out = ConvToLocalBuffer(buffer, *m_pCSConv, len, outLen);
 		if (out) {
-			wxString str(out, outLen - 1);
+			ret.assign(out, outLen - 1);
 			delete [] out;
-			return str;
+			return ret;
 		}
 	}
 
 	wxCSConv conv(_T("ISO-8859-1"));
-	wxString str = conv.cMB2WX(buffer);
-	if (str.empty())
-		str = wxConvCurrent->cMB2WX(buffer);
+	ret = conv.cMB2WX(buffer);
+	if (ret.empty()) {
+		ret = wxConvCurrent->cMB2WX(buffer);
+	}
 
-	return str;
+	return ret;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
+wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
 {
 	wxASSERT(buffer && len > 0 && !buffer[len - 1]);
 	outlen = conv.ToWChar(0, 0, buffer, len);
@@ -487,7 +483,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, si
 	return unicode;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
+wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
 {
 	if (m_useUTF8) {
 #ifdef __WXMSW__
@@ -496,13 +492,13 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t
 		// This helps when processing large directory listings.
 		int outlen2 = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, 0, 0);
 		if (outlen2 > 0) {
-			wxChar* out = new wxChar[outlen2];
+			wchar_t* out = new wchar_t[outlen2];
 			MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, out, outlen2);
 			outlen = static_cast<size_t>(outlen2);
 			return out;
 		}
 #else
-		wxChar* res = ConvToLocalBuffer(buffer, wxConvUTF8, len, outlen);
+		wchar_t* res = ConvToLocalBuffer(buffer, wxConvUTF8, len, outlen);
 		if (res && *res)
 			return res;
 #endif
@@ -515,18 +511,18 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t
 	}
 
 	if (m_pCSConv) {
-		wxChar* res = ConvToLocalBuffer(buffer, *m_pCSConv, len, outlen);
+		wchar_t* res = ConvToLocalBuffer(buffer, *m_pCSConv, len, outlen);
 		if (res && *res)
 			return res;
 	}
 
 	// Fallback: Conversion using current locale
-	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent, len, outlen);
+	wchar_t* res = ConvToLocalBuffer(buffer, *wxConvCurrent, len, outlen);
 
 	return res;
 }
 
-wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /*=false*/)
+wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8)
 {
 	if (m_useUTF8 || force_utf8) {
 		wxCharBuffer const buffer = str.utf8_str();
@@ -617,7 +613,7 @@ const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockSta
 	return iter;
 }
 
-bool CControlSocket::TryLockCache(enum locking_reason reason, const CServerPath& directory)
+bool CControlSocket::TryLockCache(locking_reason reason, const CServerPath& directory)
 {
 	wxASSERT(m_pCurrentServer);
 	wxASSERT(m_pCurOpData);
@@ -672,7 +668,7 @@ bool CControlSocket::TryLockCache(enum locking_reason reason, const CServerPath&
 	return true;
 }
 
-bool CControlSocket::IsLocked(enum locking_reason reason, const CServerPath& directory)
+bool CControlSocket::IsLocked(locking_reason reason, const CServerPath& directory)
 {
 	wxASSERT(m_pCurrentServer);
 
@@ -717,7 +713,7 @@ void CControlSocket::UnlockCache()
 	}
 
 	CServerPath directory = iter->directory;
-	enum locking_reason reason = iter->reason;
+	locking_reason reason = iter->reason;
 
 	m_lockInfoList.erase(iter);
 
@@ -747,7 +743,7 @@ void CControlSocket::UnlockCache()
 	}
 }
 
-enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
+CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 {
 	if (!m_pCurOpData)
 		return lock_unknown;
@@ -951,7 +947,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 	}
 }
 
-void CRealControlSocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+void CRealControlSocket::OnHostAddress(CSocketEventSource*, std::string const& address)
 {
 	if (!m_pBackend)
 		return;
@@ -1012,12 +1008,13 @@ void CRealControlSocket::OnClose(int error)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
 
-	if (GetCurrentCommandId() != Command::connect)
-	{
+	auto cmd = GetCurrentCommandId();
+	if (cmd != Command::connect) {
+		auto messageType = (cmd == Command::none) ? MessageType::Status : MessageType::Error;
 		if (!error)
-			LogMessage(MessageType::Error, _("Connection closed by server"));
+			LogMessage(messageType, _("Connection closed by server"));
 		else
-			LogMessage(MessageType::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
+			LogMessage(messageType, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
 	}
 	DoClose();
 }
@@ -1048,7 +1045,7 @@ int CRealControlSocket::ContinueConnect()
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
-		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->FormatHost(), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
+		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->Format(ServerFormat::with_optional_port), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
 		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
 		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
@@ -1056,7 +1053,7 @@ int CRealControlSocket::ContinueConnect()
 		delete m_pBackend;
 		m_pProxyBackend = new CProxySocket(this, m_pSocket, this);
 		m_pBackend = m_pProxyBackend;
-		int res = m_pProxyBackend->Handshake((enum CProxySocket::ProxyType)proxy_type,
+		int res = m_pProxyBackend->Handshake(static_cast<CProxySocket::ProxyType>(proxy_type),
 											m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
 											engine_.GetOptions().GetOption(OPTION_PROXY_USER),
 											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
@@ -1082,7 +1079,7 @@ int CRealControlSocket::ContinueConnect()
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
 	}
 
-	int res = m_pSocket->Connect(host, port);
+	int res = m_pSocket->Connect(fz::to_native(host), port);
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
 	if (res && res != EINPROGRESS) {
@@ -1126,8 +1123,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 {
 	wxASSERT(pFileExistsNotification);
 
-	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer)
-	{
+	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
 		return false;
 	}
@@ -1140,7 +1136,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::overwriteNewer:
-		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
+		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
 		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
@@ -1177,7 +1173,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::overwriteSizeOrNewer:
-		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
+		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
@@ -1210,22 +1206,42 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		break;
 	case CFileExistsNotification::rename:
 		if (pData->download) {
-			wxFileName fn = pData->localFile;
-			fn.SetFullName(pFileExistsNotification->newName);
-			pData->localFile = fn.GetFullPath();
+			{
+				wxString tmp;
+				CLocalPath l(pData->localFile, &tmp);
+				if (l.empty() || tmp.empty()) {
+					ResetOperation(FZ_REPLY_INTERNALERROR);
+					return false;
+				}
+				if (!l.ChangePath(pFileExistsNotification->newName)) {
+					ResetOperation(FZ_REPLY_INTERNALERROR);
+					return false;
+				}
+				if (!l.HasParent() || !l.MakeParent(&tmp)) {
+					ResetOperation(FZ_REPLY_INTERNALERROR);
+					return false;
+				}
+				
+				pData->localFile = l.GetPath() + tmp;
+			}
 
 			int64_t size;
 			bool isLink;
-			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file)
+			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file) {
 				pData->localFileSize = size;
-			else
+			}
+			else {
 				pData->localFileSize = -1;
+			}
 
-			if (CheckOverwriteFile() == FZ_REPLY_OK)
+			if (CheckOverwriteFile() == FZ_REPLY_OK) {
 				SendNextCommand();
+			}
 		}
 		else {
 			pData->remoteFile = pFileExistsNotification->newName;
+			pData->fileTime = fz::datetime();
+			pData->remoteFileSize = -1;
 
 			CDirentry entry;
 			bool dir_did_exist;
@@ -1234,15 +1250,13 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 				matched_case)
 			{
 				pData->remoteFileSize = entry.size;
-				if (entry.has_date())
+				if (entry.has_date()) {
 					pData->fileTime = entry.time;
+				}
 
-				if (CheckOverwriteFile() != FZ_REPLY_OK)
+				if (CheckOverwriteFile() != FZ_REPLY_OK) {
 					break;
-			}
-			else {
-				pData->fileTime = fz::datetime();
-				pData->remoteFileSize = -1;
+				}
 			}
 
 			SendNextCommand();
@@ -1276,7 +1290,7 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	if (local_path.empty() || !local_path.HasParent())
 		return;
 
-	// Only go back as far as needed. On comparison, wxWidgets'
+	// Only go back as far as needed. By comparison, wxWidgets'
 	// wxFileName::Mkdir always starts at the root.
 	std::vector<wxString> segments;
 	while (!local_path.Exists() && local_path.HasParent()) {
@@ -1289,16 +1303,21 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	for (auto iter = segments.rbegin(); iter != segments.rend(); ++iter) {
 		local_path.AddSegment(*iter);
 
-#ifdef __WXMSW__
-		BOOL res = CreateDirectory(local_path.GetPath().wc_str(), 0);
-		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
+#ifdef FZ_WINDOWS
+		BOOL res = CreateDirectory(local_path.GetPath().c_str(), 0);
+		if (!res && GetLastError() != ERROR_ALREADY_EXISTS) {
 			break;
+		}
 #else
-		const wxCharBuffer s = local_path.GetPath().fn_str();
-
-		int res = mkdir(s, 0777);
-		if (res && errno != EEXIST)
+		fz::native_string s = fz::to_native(local_path.GetPath());
+		if (s.empty()) {
 			break;
+		}
+
+		int res = mkdir(s.c_str(), 0777);
+		if (res && errno != EEXIST) {
+			break;
+		}
 #endif
 		last_successful = local_path;
 	}
@@ -1329,7 +1348,7 @@ int CControlSocket::RawCommand(const wxString&)
 	return FZ_REPLY_NOTSUPPORTED;
 }
 
-int CControlSocket::Delete(const CServerPath&, std::deque<wxString>&&)
+int CControlSocket::Delete(const CServerPath&, std::deque<std::wstring>&&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
