@@ -1,16 +1,16 @@
 #include <filezilla.h>
 
-wxString CDirentry::dump() const
+std::wstring CDirentry::dump() const
 {
-	wxString str = wxString::Format(_T("name=%s\nsize=%lld\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n"),
-				name, static_cast<long long>(size), *permissions, *ownerGroup, flags & flag_dir, flags & flag_link,
+	std::wstring str = fz::sprintf(L"name=%s\nsize=%d\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n",
+				name, size, *permissions, *ownerGroup, flags & flag_dir, flags & flag_link,
 				target ? *target : std::wstring(), flags & flag_unsure);
 
-	if( has_date() ) {
-		str += _T("date=") + time.format(_T("%Y-%m-%d"), fz::datetime::local) + _T("\n");
+	if (has_date()) {
+		str += L"date=" + time.format(L"%Y-%m-%d", fz::datetime::local) + L"\n";
 	}
-	if( has_time() ) {
-		str += _T("time=") + time.format(_T("%H-%M-%S"), fz::datetime::local) + _T("\n");
+	if (has_time()) {
+		str += L"time=" + time.format(L"%H-%M-%S", fz::datetime::local) + L"\n";
 	}
 	return str;
 }
@@ -32,38 +32,15 @@ bool CDirentry::operator==(const CDirentry &op) const
 	if (flags != op.flags)
 		return false;
 
-	if( has_date() ) {
-		if (time != op.time)
+	if (has_date()) {
+		if (time != op.time) {
 			return false;
+		}
 	}
 
 	return true;
 }
 
-void CDirectoryListing::SetCount(unsigned int count)
-{
-	if (count == m_entryCount)
-		return;
-
-	const unsigned int old_count = m_entryCount;
-
-	if (!count)
-	{
-		m_entryCount = 0;
-		return;
-	}
-
-	if (count < old_count)
-	{
-		m_searchmap_case.clear();
-		m_searchmap_nocase.clear();
-	}
-
-	m_entries.Get().resize(count);
-
-	m_entryCount = count;
-}
-
 const CDirentry& CDirectoryListing::operator[](unsigned int index) const
 {
 	// Commented out, too heavy speed penalty
@@ -75,16 +52,14 @@ CDirentry& CDirectoryListing::operator[](unsigned int index)
 {
 	// Commented out, too heavy speed penalty
 	// wxASSERT(index < m_entryCount);
-	return m_entries.Get()[index].Get();
+	return m_entries.get()[index].get();
 }
 
-void CDirectoryListing::Assign(std::deque<CRefcountObject<CDirentry>> &entries)
+void CDirectoryListing::Assign(std::deque<fz::shared_value<CDirentry>> &entries)
 {
-	m_entryCount = entries.size();
-
-	std::vector<CRefcountObject<CDirentry> >& own_entries = m_entries.Get();
+	std::vector<fz::shared_value<CDirentry> >& own_entries = m_entries.get();
 	own_entries.clear();
-	own_entries.reserve(m_entryCount);
+	own_entries.reserve(entries.size());
 
 	m_flags &= ~(listing_has_dirs | listing_has_perms | listing_has_usergroup);
 
@@ -110,16 +85,14 @@ bool CDirectoryListing::RemoveEntry(unsigned int index)
 	m_searchmap_case.clear();
 	m_searchmap_nocase.clear();
 
-	std::vector<CRefcountObject<CDirentry> >& entries = m_entries.Get();
-	std::vector<CRefcountObject<CDirentry> >::iterator iter = entries.begin() + index;
+	std::vector<fz::shared_value<CDirentry> >& entries = m_entries.get();
+	std::vector<fz::shared_value<CDirentry> >::iterator iter = entries.begin() + index;
 	if ((*iter)->is_dir())
 		m_flags |= CDirectoryListing::unsure_dir_removed;
 	else
 		m_flags |= CDirectoryListing::unsure_file_removed;
 	entries.erase(iter);
 
-	--m_entryCount;
-
 	return true;
 }
 
@@ -132,11 +105,13 @@ void CDirectoryListing::GetFilenames(std::vector<std::wstring> &names) const
 
 int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 {
-	if (!m_entryCount)
+	if (!m_entries || m_entries->empty()) {
 		return -1;
+	}
 
-	if (!m_searchmap_case)
-		m_searchmap_case.Get();
+	if (!m_searchmap_case) {
+		m_searchmap_case.get();
+	}
 
 	// Search map
 	auto iter = m_searchmap_case->find(to_wstring(name));
@@ -144,13 +119,14 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 		return iter->second;
 
 	unsigned int i = m_searchmap_case->size();
-	if (i == m_entryCount)
+	if (i == m_entries->size()) {
 		return -1;
+	}
 
-	auto & searchmap_case = m_searchmap_case.Get();
+	auto & searchmap_case = m_searchmap_case.get();
 
 	// Build map if not yet complete
-	std::vector<CRefcountObject<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
+	std::vector<fz::shared_value<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
 	for (; entry_iter != m_entries->end(); ++entry_iter, ++i) {
 		std::wstring const& entry_name = (*entry_iter)->name;
 		searchmap_case.insert(std::pair<std::wstring const, unsigned int>(entry_name, i));
@@ -165,11 +141,11 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 
 int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 {
-	if (!m_entryCount)
+	if (!m_entries || m_entries->empty())
 		return -1;
 
 	if (!m_searchmap_nocase)
-		m_searchmap_nocase.Get();
+		m_searchmap_nocase.get();
 
 	name.MakeLower();
 
@@ -179,13 +155,14 @@ int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 		return iter->second;
 
 	unsigned int i = m_searchmap_nocase->size();
-	if (i == m_entryCount)
+	if (i == m_entries->size()) {
 		return -1;
+	}
 
-	auto& searchmap_nocase = m_searchmap_nocase.Get();
+	auto& searchmap_nocase = m_searchmap_nocase.get();
 
 	// Build map if not yet complete
-	std::vector<CRefcountObject<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
+	std::vector<fz::shared_value<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
 	for (; entry_iter != m_entries->end(); ++entry_iter, ++i) {
 		wxString entry_name = (*entry_iter)->name;
 		entry_name.MakeLower();
@@ -207,3 +184,8 @@ void CDirectoryListing::ClearFindMap()
 	m_searchmap_case.clear();
 	m_searchmap_nocase.clear();
 }
+
+void CDirectoryListing::Append(CDirentry&& entry)
+{
+	m_entries.get().emplace_back(entry);
+}
\ No newline at end of file
