@@ -18,9 +18,10 @@ std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
 std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
-CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent)
+CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler)
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
+	, notification_handler_(notificationHandler)
 	, m_options(context.GetOptions())
 	, m_rateLimiter(context.GetRateLimiter())
 	, directory_cache_(context.GetDirectoryCache())
@@ -129,19 +130,21 @@ Command CFileZillaEnginePrivate::GetCurrentCommandId() const
 		return GetCurrentCommand()->GetId();
 }
 
+void CFileZillaEnginePrivate::AddNotification(fz::scoped_lock& lock, CNotification *pNotification)
+{
+	m_NotificationList.push_back(pNotification);
+
+	if (m_maySendNotificationEvent) {
+		m_maySendNotificationEvent = false;
+		lock.unlock();
+		notification_handler_.OnEngineEvent(&parent_);
+	}
+}
+
 void CFileZillaEnginePrivate::AddNotification(CNotification *pNotification)
 {
-	{
-		fz::scoped_lock lock(notification_mutex_);
-		m_NotificationList.push_back(pNotification);
-
-		if (!m_maySendNotificationEvent || !m_pEventHandler) {
-			return;
-		}
-		m_maySendNotificationEvent = false;
-	}
-
-	m_pEventHandler->QueueEvent(new wxFzEvent(&parent_));
+	fz::scoped_lock lock(notification_mutex_);
+	AddNotification(lock, pNotification);
 }
 
 void CFileZillaEnginePrivate::AddLogNotification(CLogmsgNotification *pNotification)
@@ -150,15 +153,17 @@ void CFileZillaEnginePrivate::AddLogNotification(CLogmsgNotification *pNotificat
 
 	if (pNotification->msgType == MessageType::Error) {
 		queue_logs_ = false;
-		SendQueuedLogs();
-		AddNotification(pNotification);
+
+		m_NotificationList.insert(m_NotificationList.end(), queued_logs_.begin(), queued_logs_.end());
+		queued_logs_.clear();
+		AddNotification(lock, pNotification);
 	}
 	else if (pNotification->msgType == MessageType::Status) {
-		ClearQueuedLogs(false);
-		AddNotification(pNotification);
+		ClearQueuedLogs(lock, false);
+		AddNotification(lock, pNotification);
 	}
 	else if (!queue_logs_) {
-		AddNotification(pNotification);
+		AddNotification(lock, pNotification);
 	}
 	else {
 		queued_logs_.push_back(pNotification);
@@ -176,19 +181,17 @@ void CFileZillaEnginePrivate::SendQueuedLogs(bool reset_flag)
 			queue_logs_ = ShouldQueueLogsFromOptions();
 		}
 
-		if (!m_maySendNotificationEvent || !m_pEventHandler || m_NotificationList.empty()) {
+		if (!m_maySendNotificationEvent || m_NotificationList.empty()) {
 			return;
 		}
 		m_maySendNotificationEvent = false;
 	}
 
-	m_pEventHandler->QueueEvent(new wxFzEvent(&parent_));
+	notification_handler_.OnEngineEvent(&parent_);
 }
 
-void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
+void CFileZillaEnginePrivate::ClearQueuedLogs(fz::scoped_lock& lock, bool reset_flag)
 {
-	fz::scoped_lock lock(notification_mutex_);
-
 	for (auto msg : queued_logs_) {
 		delete msg;
 	}
@@ -199,6 +202,12 @@ void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 	}
 }
 
+void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
+{
+	fz::scoped_lock lock(notification_mutex_);
+	ClearQueuedLogs(lock, reset_flag);
+}
+
 int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 {
 	fz::scoped_lock lock(mutex_);
@@ -561,7 +570,7 @@ void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& pa
 	wxASSERT(pOwnServer);
 
 	for (auto & engine : m_engineList) {
-		if (engine == this)
+		if (!engine || engine == this)
 			continue;
 
 		if (!engine->m_pControlSocket)
@@ -721,13 +730,6 @@ void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsync
 	m_pControlSocket->SetAsyncRequestReply(reply.get());
 }
 
-int CFileZillaEnginePrivate::Init(wxEvtHandler *pEventHandler)
-{
-	fz::scoped_lock lock(mutex_);
-	m_pEventHandler = pEventHandler;
-	return FZ_REPLY_OK;
-}
-
 int CFileZillaEnginePrivate::Execute(const CCommand &command)
 {
 	fz::scoped_lock lock(mutex_);
@@ -836,11 +838,12 @@ int CFileZillaEnginePrivate::Cancel()
 void CFileZillaEnginePrivate::OnOptionsChanged(changed_options_t const&)
 {
 	bool queue_logs = ShouldQueueLogsFromOptions();
-	fz::scoped_lock lock(notification_mutex_);
-	queue_logs_ = queue_logs;
-
-	if (!queue_logs_) {
-		SendQueuedLogs();
+	if (queue_logs) {
+		fz::scoped_lock lock(notification_mutex_);
+		queue_logs_ = true;
+	}
+	else {
+		SendQueuedLogs(true);
 	}
 }
 
@@ -861,7 +864,7 @@ void CTransferStatusManager::Reset()
 	engine_.AddNotification(new CTransferStatusNotification());
 }
 
-void CTransferStatusManager::Init(wxFileOffset totalSize, wxFileOffset startOffset, bool list)
+void CTransferStatusManager::Init(int64_t totalSize, int64_t startOffset, bool list)
 {
 	fz::scoped_lock lock(mutex_);
 	if (startOffset < 0)
@@ -889,7 +892,7 @@ void CTransferStatusManager::SetMadeProgress()
 	status_.madeProgress = true;
 }
 
-void CTransferStatusManager::Update(wxFileOffset transferredBytes)
+void CTransferStatusManager::Update(int64_t transferredBytes)
 {
 	CNotification* notification = 0;
 
