@@ -5,11 +5,10 @@
 #include "httpcontrolsocket.h"
 #include "tlssocket.h"
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
-#include <wx/file.h>
-
 #define FZ_REPLY_REDIRECTED FZ_REPLY_ALREADYCONNECTED
 
 // Connect is special for HTTP: It is done on a per-command basis, so we need
@@ -72,15 +71,9 @@ public:
 	CHttpFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path)
 		: CFileTransferOpData(is_download, local_file, remote_file, remote_path), CHttpOpData(this)
 	{
-		pFile = 0;
 	}
 
-	virtual ~CHttpFileTransferOpData()
-	{
-		delete pFile;
-	}
-
-	wxFile* pFile;
+	fz::file file;
 };
 
 CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
@@ -199,7 +192,7 @@ int CHttpControlSocket::DoReceive()
 {
 	do
 	{
-		const enum CSocket::SocketState state = m_pSocket->GetState();
+		const CSocket::SocketState state = m_pSocket->GetState();
 		if (state != CSocket::connected && state != CSocket::closing)
 			return 0;
 
@@ -343,7 +336,7 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	m_pCurOpData = pData;
 	m_pHttpOpData = pData;
 
-	m_current_uri = wxURI(m_pCurrentServer->FormatServer() + pData->remotePath.FormatFilename(pData->remoteFile));
+	m_current_uri = wxURI(m_pCurrentServer->Format(ServerFormat::with_user_and_optional_port) + pData->remotePath.FormatFilename(pData->remoteFile));
 
 	if (!localFile.empty()) {
 		pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
@@ -393,14 +386,13 @@ int CHttpControlSocket::FileTransferSend()
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSend()"));
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if( !m_current_uri.HasScheme() || !m_current_uri.HasServer() || !m_current_uri.HasPath() ) {
+	if (!m_current_uri.HasScheme() || !m_current_uri.HasServer() || !m_current_uri.HasPath()) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Invalid URI: %s"), m_current_uri.BuildURI());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -408,39 +400,40 @@ int CHttpControlSocket::FileTransferSend()
 
 	CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
 
-	if (pData->opState == filetransfer_waitfileexists)
-	{
+	if (pData->opState == filetransfer_waitfileexists) {
 		pData->opState = filetransfer_transfer;
 
 		int res = OpenFile(pData);
-		if( res != FZ_REPLY_OK)
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 
 		res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
-		if (res != FZ_REPLY_OK)
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 	}
 
-	wxString location = m_current_uri.GetPath();
-	if( m_current_uri.HasQuery() ) {
-		location += _T("?") + m_current_uri.GetQuery();
+	std::wstring location = m_current_uri.GetPath().ToStdWstring();
+	if (m_current_uri.HasQuery()) {
+		location += L"?" + m_current_uri.GetQuery().ToStdWstring();
 	}
-	wxString action = wxString::Format(_T("GET %s HTTP/1.1"), location );
+	std::wstring action = fz::sprintf(L"GET %s HTTP/1.1", location);
 	LogMessageRaw(MessageType::Command, action);
 
-	wxString hostWithPort = m_current_uri.GetServer();
-	if( m_current_uri.HasPort() ) {
-		hostWithPort += _T(":") + m_current_uri.GetPort();
+	std::string hostWithPort = fz::to_utf8(m_current_uri.GetServer());
+	if (m_current_uri.HasPort()) {
+		hostWithPort += ":" + fz::to_utf8(m_current_uri.GetPort());
 	}
-	wxString command = wxString::Format(_T("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n"), action, hostWithPort, wxString(PACKAGE_STRING, wxConvLocal));
-	if( pData->resume ) {
-		command += wxString::Format(_T("Range: bytes=%") + wxString(wxFileOffsetFmtSpec) + _T("d-\r\n"), pData->localFileSize);
+	std::string command = fz::sprintf("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n", fz::to_utf8(action), hostWithPort, PACKAGE_STRING);
+	if (pData->resume) {
+		command += fz::sprintf("Range: bytes=%d-\r\n", pData->localFileSize);
 	}
-	command += _T("\r\n");
+	command += "\r\n";
 
-	const wxWX2MBbuf str = command.mb_str();
-	if (!Send(str, strlen(str)))
+	if (!Send(command.c_str(), command.size())) {
 		return FZ_REPLY_ERROR;
+	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -479,7 +472,7 @@ int CHttpControlSocket::DoInternalConnect()
 	delete m_pBackend;
 	m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 
-	int res = m_pSocket->Connect(pData->host, pData->port);
+	int res = m_pSocket->Connect(fz::to_native(pData->host), pData->port);
 	if (!res)
 		return FZ_REPLY_OK;
 
@@ -517,9 +510,10 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 		engine_.AddNotification(new CDataNotification(q, len));
 	}
 	else {
-		wxASSERT(pData->pFile);
+		wxASSERT(pData->file.opened());
 
-		if (pData->pFile->Write(p, len) != len) {
+		auto write = static_cast<int64_t>(len);
+		if (pData->file.write(p, write) != write) {
 			LogMessage(MessageType::Error, _("Failed to write to file %s"), pData->localFile);
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
@@ -647,7 +641,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 						return FZ_REPLY_ERROR;
 					}
 
-					enum ServerProtocol protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.GetScheme());
+					ServerProtocol protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.GetScheme());
 					if( protocol != HTTP && protocol != HTTPS ) {
 						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
 						ResetOperation(FZ_REPLY_ERROR);
@@ -867,21 +861,19 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 
 int CHttpControlSocket::ResetOperation(int nErrorCode)
 {
-	if (m_pCurOpData && m_pCurOpData->opId == Command::transfer)
-	{
+	if (m_pCurOpData && m_pCurOpData->opId == Command::transfer) {
 		CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
-		delete pData->pFile;
-		pData->pFile = 0;
+		pData->file.close();
 	}
 
-	if (!m_pCurOpData || !m_pCurOpData->pNextOpData)
-	{
-		if (m_pBackend)
-		{
-			if (nErrorCode == FZ_REPLY_OK)
+	if (!m_pCurOpData || !m_pCurOpData->pNextOpData) {
+		if (m_pBackend) {
+			if (nErrorCode == FZ_REPLY_OK) {
 				LogMessage(MessageType::Status, _("Disconnected from server"));
-			else
+			}
+			else {
 				LogMessage(MessageType::Error, _("Disconnected from server"));
+			}
 		}
 		ResetSocket();
 		m_pHttpOpData = 0;
@@ -1000,16 +992,16 @@ int CHttpControlSocket::Disconnect()
 
 int CHttpControlSocket::OpenFile(CHttpFileTransferOpData* pData)
 {
-	delete pData->pFile;
-	pData->pFile = new wxFile();
+	pData->file.close();
+
 	CreateLocalDir(pData->localFile);
 
-	if (!pData->pFile->Open(pData->localFile, pData->resume ? wxFile::write_append : wxFile::write)) {
+	if (!pData->file.open(fz::to_native(pData->localFile), fz::file::writing, pData->resume ? fz::file::existing : fz::file::empty)) {
 		LogMessage(MessageType::Error, _("Failed to open \"%s\" for writing"), pData->localFile);
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
-	wxFileOffset end = pData->pFile->SeekEnd();
+	int64_t end = pData->file.seek(0, fz::file::end);
 	if (!end) {
 		pData->resume = false;
 	}
