@@ -1,63 +1,65 @@
 #include <filezilla.h>
 #include "local_path.h"
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 #include <errno.h>
 #endif
 
 #include <deque>
 
-#ifdef __WXMSW__
-const wxChar CLocalPath::path_separator = '\\';
+#ifdef FZ_WINDOWS
+wchar_t const CLocalPath::path_separator = '\\';
 #else
-const wxChar CLocalPath::path_separator = '/';
+wchar_t const CLocalPath::path_separator = '/';
 #endif
 
 CLocalPath::CLocalPath(const wxString& path, wxString* file /*=0*/)
 {
-	SetPath(path, file);
+	SetPath(path.ToStdWstring(), file);
 }
 
-bool CLocalPath::SetPath(const wxString& path, wxString* file /*=0*/)
+bool CLocalPath::SetPath(std::wstring const& path, wxString* file /*=0*/)
 {
 	// This function ensures that the path is in canonical form on success.
-
 	if (path.empty()) {
 		m_path.clear();
 		return false;
 	}
 
-	std::deque<wxChar*> segments; // List to store the beginnings of segments
+#ifdef FZ_WINDOWS
+	if (path == _T("\\")) {
+		m_path.get() = path;
+		if (file)
+			file->clear();
+		return true;
+	}
+#endif
 
-	const wxChar* in = path.c_str();
+	std::deque<wchar_t*> segments; // List to store the beginnings of segments
+
+	wchar_t const* in = path.c_str();
 
 	{
-		wxStringBuffer start(m_path.Get(), path.Len() + 2);
-		wxChar* out = start;
+		std::wstring & path_out = m_path.get();
+		path_out.resize(path.size() + 1);
+		wchar_t * const start = &(path_out[0]);
+		wchar_t * out = start;
 
-#ifdef __WXMSW__
-		if (path == _T("\\"))
-		{
-			*out++ = '\\';
-			*out++ = 0;
-			if (file)
-				file->clear();
-			return true;
-		}
+#ifdef FZ_WINDOWS
 
 		if (*in == '\\') {
 			// possibly UNC
-
 			in++;
 			if (*in++ != '\\') {
-				*start = 0;
+				path_out.clear();
 				return false;
 			}
 
 			if (*in == '?') {
 				// Could be \\?\c:\foo\bar
-				// or \\?\UNC\server\sharee
+				// or \\?\UNC\server\share
 				// or something else we do not support.
 				if (*(++in) != '\\') {
+					path_out.clear();
 					return false;
 				}
 				in++;
@@ -65,17 +67,22 @@ bool CLocalPath::SetPath(const wxString& path, wxString* file /*=0*/)
 					// It's \\?\c:\foo\bar
 					goto parse_regular;
 				}
-				if (fz::strlen(in) < 5 || wxStrnicmp(in, _T("UNC\\"), 4)) {
+				wchar_t const* in_end = in + path.size();
+				if (in_end - in < 5 || wxStrnicmp(in, _T("UNC\\"), 4)) {
+					path_out.clear();
 					return false;
 				}
 				in += 4;
+
+				// It's \\?\UNC\server\share
+				//              ^we are here
 			}
+
 			*out++ = '\\';
 			*out++ = '\\';
 
 			// UNC path
-			while (*in)
-			{
+			while (*in) {
 				if (*in == '/' || *in == '\\')
 					break;
 				*out++ = *in++;
@@ -84,39 +91,37 @@ bool CLocalPath::SetPath(const wxString& path, wxString* file /*=0*/)
 
 			if (out - start <= 3) {
 				// not a valid UNC path
-				*start = 0;
+				path_out.clear();
 				return false;
 			}
 
 			segments.push_back(out);
 		}
-		else if ((*in >= 'a' && *in <= 'z') || (*in >= 'A' || *in <= 'Z'))
-		{
+		else if ((*in >= 'a' && *in <= 'z') || (*in >= 'A' || *in <= 'Z')) {
 parse_regular:
 			// Regular path
 			*out++ = *in++;
 
 			if (*in++ != ':') {
-				*start = 0;
+				path_out.clear();
 				return false;
 			}
 			*out++ = ':';
 			if (*in != '/' && *in != '\\' && *in) {
-				*start = 0;
+				path_out.clear();
 				return false;
 			}
 			*out++ = path_separator;
 			segments.push_back(out);
 		}
 		else {
-			*start = 0;
+			path_out.clear();
 			return false;
 		}
 #else
-		if (*in++ != '/')
-		{
+		if (*in++ != '/') {
 			// SetPath only accepts absolute paths
-			*start = 0;
+			path_out.clear();
 			return false;
 		}
 
@@ -131,37 +136,33 @@ parse_regular:
 			dotdot,
 			segment
 		};
-		enum _last last = separator;
+		_last last = separator;
 
-		while (*in)
-		{
+		while (*in) {
 			if (*in == '/'
-	#ifdef __WXMSW__
+	#ifdef FZ_WINDOWS
 				|| *in == '\\'
 	#endif
 				)
 			{
 				in++;
-				if (last == separator)
-				{
+				if (last == separator) {
 					// /foo//bar is equal to /foo/bar
 					continue;
 				}
-				else if (last == dot)
-				{
+				else if (last == dot) {
 					// /foo/./bar is equal to /foo/bar
 					last = separator;
 					out = segments.back();
 					continue;
 				}
-				else if (last == dotdot)
-				{
+				else if (last == dotdot) {
 					last = separator;
 
 					// Go two segments back if possible
-					if (segments.size() > 1)
+					if (segments.size() > 1) {
 						segments.pop_back();
-					wxASSERT(!segments.empty());
+					}
 					out = segments.back();
 					continue;
 				}
@@ -172,41 +173,42 @@ parse_regular:
 				last = separator;
 				continue;
 			}
-			else if (*in == '.')
-			{
-				if (last == separator)
+			else if (*in == '.') {
+				if (last == separator) {
 					last = dot;
-				else if (last == dot)
+				}
+				else if (last == dot) {
 					last = dotdot;
-				else if (last == dotdot)
+				}
+				else if (last == dotdot) {
 					last = segment;
+				}
 			}
-			else
+			else {
 				last = segment;
+			}
 
 			*out++ = *in++;
 		}
-		if (last == dot)
-			out = segments.back();
-		else if (last == dotdot)
-		{
-			if (segments.size() > 1)
-				segments.pop_back();
+		if (last == dot) {
 			out = segments.back();
 		}
-		else if (last == segment)
-		{
-			if (file)
-			{
-				*out = 0;
+		else if (last == dotdot) {
+			if (segments.size() > 1) {
+				segments.pop_back();
+			}
+			out = segments.back();
+		}
+		else if (last == segment) {
+			if (file) {
+				*file = wxString(segments.back(), out);
 				out = segments.back();
-				*file = out;
 			}
 			else
 				*out++ = path_separator;
 		}
 
-		*out = 0;
+		path_out.resize(out - start);
 	}
 
 	return true;
@@ -227,14 +229,14 @@ bool CLocalPath::IsWriteable() const
 	if (m_path->empty())
 		return false;
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	if (m_path == _T("\\"))
 		// List of drives not writeable
 		return false;
 
-	if (m_path->Left(2) == _T("\\\\")) {
-		int pos = m_path->Mid(2).Find('\\');
-		if (pos == -1 || pos + 3 == (int)m_path->Len())
+	if (m_path->substr(0, 2) == _T("\\\\")) {
+		auto pos = m_path->find('\\', 2);
+		if (pos == std::wstring::npos || pos + 3 >= m_path->size())
 			// List of shares on a computer not writeable
 			return false;
 	}
@@ -245,7 +247,7 @@ bool CLocalPath::IsWriteable() const
 
 bool CLocalPath::HasParent() const
 {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	// C:\f\ has parent
 	// C:\ does not
 	// \\x\y\ shortest UNC
@@ -254,7 +256,7 @@ bool CLocalPath::HasParent() const
 #else
 	const int min = 0;
 #endif
-	for (int i = (int)m_path->Len() - 2; i >= min; --i) {
+	for (int i = static_cast<int>(m_path->size()) - 2; i >= min; --i) {
 		if ((*m_path)[i] == path_separator)
 			return true;
 	}
@@ -264,22 +266,23 @@ bool CLocalPath::HasParent() const
 
 bool CLocalPath::HasLogicalParent() const
 {
-#ifdef __WXMSW__
-	if (m_path->Len() == 3 && (*m_path)[0] != '\\') // Drive root
+#ifdef FZ_WINDOWS
+	if (m_path->size() == 3 && (*m_path)[0] != '\\') // Drive root
 		return true;
 #endif
 	return HasParent();
 }
 
-CLocalPath CLocalPath::GetParent(wxString* last_segment /*=0*/) const
+CLocalPath CLocalPath::GetParent(wxString* last_segment) const
 {
 	CLocalPath parent;
 
-#ifdef __WXMSW__
-	if (m_path->Len() == 3 && (*m_path)[0] != '\\') // Drive root
-	{
-		if (last_segment)
+#ifdef FZ_WINDOWS
+	if (m_path->size() == 3 && (*m_path)[0] != '\\') {
+		// Drive root
+		if (last_segment) {
 			last_segment->clear();
+		}
 		return CLocalPath(_T("\\"));
 	}
 
@@ -291,13 +294,12 @@ CLocalPath CLocalPath::GetParent(wxString* last_segment /*=0*/) const
 #else
 	const int min = 0;
 #endif
-	for (int i = (int)m_path->Len() - 2; i >= min; --i) {
+	for (int i = (int)m_path->size() - 2; i >= min; --i) {
 		if ((*m_path)[i] == path_separator) {
 			if (last_segment) {
-				*last_segment = m_path->Mid(i + 1);
-				last_segment->RemoveLast();
+				*last_segment = m_path->substr(i + 1, m_path->size() - i - 2);
 			}
-			return CLocalPath(m_path->Left(i + 1));
+			return CLocalPath(m_path->substr(0, i + 1));
 		}
 	}
 
@@ -306,11 +308,11 @@ CLocalPath CLocalPath::GetParent(wxString* last_segment /*=0*/) const
 
 bool CLocalPath::MakeParent(wxString* last_segment /*=0*/)
 {
-	wxString& path = m_path.Get();
+	std::wstring& path = m_path.get();
 
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path[0] != '\\') // Drive root
-	{
+#ifdef FZ_WINDOWS
+	if (path.size() == 3 && path[0] != '\\') {
+		// Drive root
 		path = _T("\\");
 		return true;
 	}
@@ -323,13 +325,12 @@ bool CLocalPath::MakeParent(wxString* last_segment /*=0*/)
 #else
 	const int min = 0;
 #endif
-	for (int i = (int)path.Len() - 2; i >= min; --i) {
+	for (int i = (int)path.size() - 2; i >= min; --i) {
 		if (path[i] == path_separator) {
 			if (last_segment) {
-				*last_segment = path.Mid(i + 1);
-				last_segment->RemoveLast();
+				*last_segment = path.substr(i + 1, path.size() - i - 2);
 			}
-			path = path.Left(i + 1);
+			path = path.substr(0, i + 1);
 			return true;
 		}
 	}
@@ -339,12 +340,12 @@ bool CLocalPath::MakeParent(wxString* last_segment /*=0*/)
 
 void CLocalPath::AddSegment(const wxString& segment)
 {
-	wxString& path = m_path.Get();
+	std::wstring& path = m_path.get();
 
 	wxASSERT(!path.empty());
-	wxASSERT(segment.Find(_T("/")) == -1);
-#ifdef __WXMSW__
-	wxASSERT(segment.Find(_T("\\")) == -1);
+	wxASSERT(segment.find(_T("/")) == std::wstring::npos);
+#ifdef FZ_WINDOWS
+	wxASSERT(segment.find(_T("\\")) == std::wstring::npos);
 #endif
 
 	if (!segment.empty()) {
@@ -355,51 +356,48 @@ void CLocalPath::AddSegment(const wxString& segment)
 
 bool CLocalPath::ChangePath(const wxString& new_path)
 {
-	if (new_path.empty())
+	if (new_path.empty()) {
 		return false;
+	}
 
-	wxString& path = m_path.Get();
-
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	if (new_path == _T("\\") || new_path == _T("/")) {
-		path = _T("\\");
+		m_path.get() = _T("\\");
 		return true;
 	}
 
-	if (new_path.Len() >= 2 && new_path[0] == '\\' && new_path[1] == '\\') {
+	if (new_path.size() >= 2 && new_path[0] == '\\' && new_path[1] == '\\') {
 		// Absolute UNC
-		return SetPath(new_path);
+		return SetPath(new_path.ToStdWstring());
 	}
-	if (new_path.Len() >= 2 && new_path[0] && new_path[1] == ':') {
+	if (new_path.size() >= 2 && new_path[0] && new_path[1] == ':') {
 		// Absolute new_path
-		return SetPath(new_path);
+		return SetPath(new_path.ToStdWstring());
 	}
 
 	// Relative new_path
-	if (path.empty())
+	if (m_path->empty())
 		return false;
 
-	if (new_path.Len() >= 2 && (new_path[0] == '\\' || new_path[0] == '/') && path[1] == ':') {
+	if (new_path.size() >= 2 && (new_path[0] == '\\' || new_path[0] == '/') && m_path->size() > 2 && (*m_path)[1] == ':') {
 		// Relative to drive root
-		return SetPath(path.Left(2) + new_path);
+		return SetPath(m_path->substr(0, 2) + new_path.ToStdWstring());
 	}
 	else {
 		// Relative to current directory
-		return SetPath(path + new_path);
+		return SetPath(*m_path + new_path.ToStdWstring());
 	}
 #else
 	if (!new_path.empty() && new_path[0] == path_separator) {
 		// Absolute new_path
-		return SetPath(new_path);
+		return SetPath(new_path.ToStdWstring());
 	}
-	else
-	{
+	else {
 		// Relative new_path
-
-		if (path.empty())
+		if (m_path->empty())
 			return false;
 
-		return SetPath(path + new_path);
+		return SetPath(*m_path + new_path.ToStdWstring());
 	}
 #endif
 }
@@ -410,9 +408,8 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 	if (m_path->empty())
 		return false;
 
-#ifdef __WXMSW__
-	if (m_path == _T("\\"))
-	{
+#ifdef FZ_WINDOWS
+	if (m_path == _T("\\")) {
 		// List of drives always exists
 		return true;
 	}
@@ -423,20 +420,23 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 		size_t pos;
 
 		// Search for backslash separating server from share
-		for (pos = 3; pos < m_path->Len(); pos++)
-			if ((*m_path)[pos] == '\\')
+		for (pos = 3; pos < m_path->size(); ++pos) {
+			if ((*m_path)[pos] == '\\') {
 				break;
-		pos++;
-		if (pos >= m_path->Len()) {
+			}
+		}
+		++pos;
+		if (pos >= m_path->size()) {
 			// Partial UNC path
 			return true;
 		}
 	}
 
-	wxString path = *m_path;
-	if (path.Len() > 3)
-		path.RemoveLast();
-	DWORD ret = ::GetFileAttributes(path.wc_str());
+	std::wstring path = *m_path;
+	if (path.size() > 3) {
+		path.resize(path.size() - 1);
+	}
+	DWORD ret = ::GetFileAttributes(path.c_str());
 	if (ret == INVALID_FILE_ATTRIBUTES) {
 		if (!error)
 			return false;
@@ -449,7 +449,7 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 		// Check for removable drive, display a more specific error message in that case
 		if (::GetLastError() != ERROR_NOT_READY)
 			return false;
-		int type = GetDriveType(m_path->Left(3).wc_str());
+		int type = GetDriveType(m_path->substr(0, 3).c_str());
 		if (type == DRIVE_REMOVABLE || type == DRIVE_CDROM)
 			error->Printf(_("Cannot access '%s', no media inserted or drive not ready."), path);
 		return false;
@@ -463,7 +463,7 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 	return true;
 #else
 	wxString path = *m_path;
-	if (path.Len() > 1)
+	if (path.size() > 1)
 		path.RemoveLast();
 
 	const wxCharBuffer s = path.fn_str();
@@ -495,8 +495,8 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 
 bool CLocalPath::operator==(const CLocalPath& op) const
 {
-#ifdef __WXMSW__
-	return m_path->CmpNoCase(*op.m_path) == 0;
+#ifdef FZ_WINDOWS
+	return m_path.is_same(op.m_path) || fz::stricmp(*m_path, *op.m_path) == 0;
 #else
 	return m_path == op.m_path;
 #endif
@@ -504,26 +504,37 @@ bool CLocalPath::operator==(const CLocalPath& op) const
 
 bool CLocalPath::operator!=(const CLocalPath& op) const
 {
-#ifdef __WXMSW__
-	return m_path->CmpNoCase(*op.m_path) != 0;
+#ifdef FZ_WINDOWS
+	return !m_path.is_same(op.m_path) && fz::stricmp(*m_path, *op.m_path) != 0;
 #else
 	return m_path != op.m_path;
 #endif
 }
 
+bool CLocalPath::operator<(CLocalPath const& op) const
+{
+#ifdef FZ_WINDOWS
+	return !m_path.is_same(op.m_path) && fz::stricmp(*m_path, *op.m_path) < 0;
+#else
+	return m_path < op.m_path;
+#endif
+}
+
 bool CLocalPath::IsParentOf(const CLocalPath &path) const
 {
-	if (empty() || path.empty())
+	if (empty() || path.empty()) {
 		return false;
+	}
 
-	if (path.m_path->Len() < m_path->Len())
+	if (path.m_path->size() < m_path->size()) {
 		return false;
+	}
 
-#ifdef __WXMSW__
-	if (m_path->CmpNoCase(path.m_path->Left(m_path->Len())))
+#ifdef FZ_WINDOWS
+	if (fz::stricmp(*m_path, path.m_path->substr(0, m_path->size())))
 		return false;
 #else
-	if (*m_path != path.m_path->Left(m_path->Len()))
+	if (*m_path != path.m_path->substr(0, m_path->size()))
 		return false;
 #endif
 
@@ -535,14 +546,14 @@ bool CLocalPath::IsSubdirOf(const CLocalPath &path) const
 	if (empty() || path.empty())
 		return false;
 
-	if (path.m_path->Len() > m_path->Len())
+	if (path.m_path->size() > m_path->size())
 		return false;
 
-#ifdef __WXMSW__
-	if (path.m_path->CmpNoCase(m_path->Left(path.m_path->Len())))
+#ifdef FZ_WINDOWS
+	if (fz::stricmp(*path.m_path, m_path->substr(0, path.m_path->size())))
 		return false;
 #else
-	if (*path.m_path != m_path->Left(path.m_path->Len()))
+	if (*path.m_path != m_path->substr(0, path.m_path->size()))
 		return false;
 #endif
 
@@ -553,7 +564,7 @@ wxString CLocalPath::GetLastSegment() const
 {
 	wxASSERT(HasParent());
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	// C:\f\ has parent
 	// C:\ does not
 	// \\x\y\ shortest UNC
@@ -562,10 +573,9 @@ wxString CLocalPath::GetLastSegment() const
 #else
 	const int min = 0;
 #endif
-	for (int i = (int)m_path->Len() - 2; i >= min; i--) {
+	for (int i = (int)m_path->size() - 2; i >= min; i--) {
 		if ((*m_path)[i] == path_separator) {
-			wxString last = m_path->Mid(i + 1);
-			last.RemoveLast();
+			wxString last = m_path->substr(i + 1, m_path->size() - i - 2);
 			return last;
 		}
 	}
