@@ -1,19 +1,21 @@
 #include <filezilla.h>
+#include "tlssocket.h"
+
+#include <libfilezilla/time.hpp>
+
+#include <wx/utils.h>
+
 #include <gnutls/gnutls.h>
 #include <sqlite3.h>
 #include <random>
 #include <cstdint>
 
-#include <libfilezilla/time.hpp>
-
-#include "tlssocket.h"
-
-wxString GetDependencyVersion(dependency::type d)
+wxString GetDependencyVersion(lib_dependency d)
 {
 	switch (d) {
-	case dependency::wxwidgets:
+	case lib_dependency::wxwidgets:
 		return wxVERSION_NUM_DOT_STRING_T;
-	case dependency::gnutls:
+	case lib_dependency::gnutls:
 		{
 			const char* v = gnutls_check_version(0);
 			if (!v || !*v)
@@ -21,21 +23,21 @@ wxString GetDependencyVersion(dependency::type d)
 
 			return wxString(v, wxConvLibc);
 		}
-	case dependency::sqlite:
+	case lib_dependency::sqlite:
 		return wxString::FromUTF8(sqlite3_libversion());
 	default:
 		return wxString();
 	}
 }
 
-wxString GetDependencyName(dependency::type d)
+wxString GetDependencyName(lib_dependency d)
 {
 	switch (d) {
-	case dependency::wxwidgets:
+	case lib_dependency::wxwidgets:
 		return _T("wxWidgets");
-	case dependency::gnutls:
+	case lib_dependency::gnutls:
 		return _T("GnuTLS");
-	case dependency::sqlite:
+	case lib_dependency::sqlite:
 		return _T("SQLite");
 	default:
 		return wxString();
@@ -83,3 +85,34 @@ bool GetRealOsVersion(int& major, int& minor)
 	return true;
 #endif
 }
+
+std::wstring url_encode(std::wstring const& s, bool keep_slashes)
+{
+	std::wstring ret;
+
+	std::string utf8 = fz::to_utf8(s);
+	ret.reserve(utf8.size());
+
+	for (auto const& c : utf8) {
+		if (!c) {
+			break;
+		}
+		else if ((c >= '0' && c <= '9') ||
+				(c >= 'a' && c <= 'z') ||
+				(c >= 'A' && c <= 'Z') ||
+				c == '-' || c == '.' || c == '_' || c == '~')
+		{
+			ret += c;
+		}
+		else if (c == '/' && keep_slashes) {
+			ret += c;
+		}
+		else {
+			ret += '%';
+			ret += fz::int_to_hex_char(static_cast<unsigned char>(c) >> 4);
+			ret += fz::int_to_hex_char(c & 0xf);
+		}
+	}
+
+	return ret;
+}
