@@ -1,36 +1,11 @@
 #include <filezilla.h>
 
-wxDEFINE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
-
-wxFzEvent::wxFzEvent()
-	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
-	, engine_()
-{
-}
-
-wxFzEvent::wxFzEvent(CFileZillaEngine const* engine)
-	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
-	, engine_(engine)
-{
-}
-
-wxFzEvent::wxFzEvent(int id)
-	: wxEvent(id, fzEVT_NOTIFICATION)
-	, engine_()
-{
-}
-
-wxEvent *wxFzEvent::Clone() const
-{
-	return new wxFzEvent(*this);
-}
-
 CDirectoryListingNotification::CDirectoryListingNotification(const CServerPath& path, const bool modified /*=false*/, const bool failed /*=false*/)
 	: m_modified(modified), m_failed(failed), m_path(path)
 {
 }
 
-enum RequestId CFileExistsNotification::GetRequestID() const
+RequestId CFileExistsNotification::GetRequestID() const
 {
 	return reqId_fileexists;
 }
@@ -42,7 +17,7 @@ CInteractiveLoginNotification::CInteractiveLoginNotification(type t, const wxStr
 {
 }
 
-enum RequestId CInteractiveLoginNotification::GetRequestID() const
+RequestId CInteractiveLoginNotification::GetRequestID() const
 {
 	return reqId_interactiveLogin;
 }
@@ -67,7 +42,7 @@ CHostKeyNotification::CHostKeyNotification(wxString host, int port, wxString fin
 {
 }
 
-enum RequestId CHostKeyNotification::GetRequestID() const
+RequestId CHostKeyNotification::GetRequestID() const
 {
 	return m_changed ? reqId_hostkeyChanged : reqId_hostkey;
 }
@@ -217,16 +192,15 @@ CCertificateNotification::CCertificateNotification(const wxString& host, unsigne
 		const wxString& sessionCipher,
 		const wxString& sessionMac,
 		int algorithmWarnings,
-		const std::vector<CCertificate> &certificates)
+		std::vector<CCertificate> && certificates)
 	: m_protocol(protocol)
 	, m_keyExchange(keyExchange)
 	, m_algorithmWarnings(algorithmWarnings)
+	, m_certificates(certificates)
 {
 	m_host = host;
 	m_port = port;
 
 	m_sessionCipher = sessionCipher;
 	m_sessionMac = sessionMac;
-
-	m_certificates = certificates;
 }
