@@ -6,6 +6,7 @@
   #include <ws2tcpip.h>
 #else
   #include <sys/socket.h>
+  #include <netdb.h>
 #endif
 #include <filezilla.h>
 #include "engineprivate.h"
@@ -14,9 +15,7 @@
 
 #include <libfilezilla/iputils.hpp>
 
-#ifndef FZ_WINDOWS
-#include <netdb.h>
-#endif
+#include <algorithm>
 
 enum handshake_state
 {
@@ -263,7 +262,7 @@ void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int err
 	}
 }
 
-void CProxySocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+void CProxySocket::OnHostAddress(CSocketEventSource*, std::string const& address)
 {
 	m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
 }
@@ -600,7 +599,7 @@ void CProxySocket::OnReceive()
 				{
 					const wxWX2MBbuf host = m_host.mb_str(wxConvUTF8);
 					const int hostlen = strlen(host);
-					int addrlen = wxMax(hostlen, 16);
+					int addrlen = std::max(hostlen, 16);
 
 					m_pSendBuffer = new char[7 + addrlen];
 					m_pSendBuffer[0] = 5;
