@@ -2,7 +2,7 @@
 
 struct t_protocolInfo
 {
-	const enum ServerProtocol protocol;
+	const ServerProtocol protocol;
 	const wxString prefix;
 	bool alwaysShowPrefix;
 	unsigned int defaultPort;
@@ -16,8 +16,8 @@ static const t_protocolInfo protocolInfos[] = {
 	{ SFTP,         _T("sftp"),   true,  22,  false, _T("SFTP - SSH File Transfer Protocol"),                              false },
 	{ HTTP,         _T("http"),   true,  80,  false, _T("HTTP - Hypertext Transfer Protocol"),                             false  },
 	{ HTTPS,        _T("https"),  true, 443,  true,  TRANSLATE_T("HTTPS - HTTP over TLS"),                                 false  },
-	{ FTPS,         _T("ftps"),   true, 990,  true,  TRANSLATE_T("FTPS - FTP over implicit TLS/SSL"),                      true  },
-	{ FTPES,        _T("ftpes"),  true,  21,  true,  TRANSLATE_T("FTPES - FTP over explicit TLS/SSL"),                     true  },
+	{ FTPS,         _T("ftps"),   true, 990,  true,  TRANSLATE_T("FTPS - FTP over implicit TLS"),                      true  },
+	{ FTPES,        _T("ftpes"),  true,  21,  true,  TRANSLATE_T("FTPES - FTP over explicit TLS"),                     true  },
 	{ INSECURE_FTP, _T("ftp"),    false, 21,  true,  TRANSLATE_T("FTP - Insecure File Transfer Protocol"), true  },
 	{ UNKNOWN,      _T(""),       false, 21,  false, _T(""), false }
 };
@@ -26,13 +26,14 @@ static const wxString typeNames[SERVERTYPE_MAX] = {
 	TRANSLATE_T("Default (Autodetect)"),
 	_T("Unix"),
 	_T("VMS"),
-	_T("DOS"),
+	_T("DOS with backslash separators"),
 	_T("MVS, OS/390, z/OS"),
 	_T("VxWorks"),
 	_T("z/VM"),
 	_T("HP NonStop"),
 	TRANSLATE_T("DOS-like with virtual paths"),
-	_T("Cygwin")
+	_T("Cygwin"),
+	_T("DOS with forward-slash separators"),
 };
 
 static const t_protocolInfo& GetProtocolInfo(ServerProtocol protocol)
@@ -88,7 +89,7 @@ bool CServer::ParseUrl(wxString host, unsigned int port, wxString user, wxString
 		if (m_protocol == UNKNOWN)
 		{
 			// TODO: http:// once WebDAV is officially supported
-			error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
+			error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
 			return false;
 		}
 	}
@@ -273,6 +274,14 @@ wxString CServer::GetAccount() const
 	return m_account;
 }
 
+wxString CServer::GetKeyFile() const
+{
+	if (m_logonType != KEY)
+		return wxString();
+
+	return m_keyFile;
+}
+
 CServer& CServer::operator=(const CServer &op)
 {
 	m_protocol = op.m_protocol;
@@ -283,6 +292,7 @@ CServer& CServer::operator=(const CServer &op)
 	m_user = op.m_user;
 	m_pass = op.m_pass;
 	m_account = op.m_account;
+	m_keyFile = op.m_keyFile;
 	m_timezoneOffset = op.m_timezoneOffset;
 	m_pasvMode = op.m_pasvMode;
 	m_maximumMultipleConnections = op.m_maximumMultipleConnections;
@@ -324,6 +334,11 @@ bool CServer::operator==(const CServer &op) const
 			if (m_account != op.m_account)
 				return false;
 		}
+		else if (m_logonType == KEY)
+		{
+			if (m_keyFile != op.m_keyFile)
+				return false;
+		}
 	}
 	if (m_timezoneOffset != op.m_timezoneOffset)
 		return false;
@@ -484,7 +499,7 @@ bool CServer::EqualsNoPass(const CServer &op) const
 	return true;
 }
 
-CServer::CServer(enum ServerProtocol protocol, enum ServerType type, wxString host, unsigned int port, wxString user, wxString pass, wxString account)
+CServer::CServer(ServerProtocol protocol, ServerType type, wxString host, unsigned int port, wxString user, wxString pass, wxString account)
 {
 	Initialize();
 	m_protocol = protocol;
@@ -497,7 +512,7 @@ CServer::CServer(enum ServerProtocol protocol, enum ServerType type, wxString ho
 	m_account = account;
 }
 
-CServer::CServer(enum ServerProtocol protocol, enum ServerType type, wxString host, unsigned int port)
+CServer::CServer(ServerProtocol protocol, ServerType type, wxString host, unsigned int port)
 {
 	Initialize();
 	m_protocol = protocol;
@@ -506,23 +521,23 @@ CServer::CServer(enum ServerProtocol protocol, enum ServerType type, wxString ho
 	m_port = port;
 }
 
-void CServer::SetType(enum ServerType type)
+void CServer::SetType(ServerType type)
 {
 	m_type = type;
 }
 
-enum LogonType CServer::GetLogonType() const
+LogonType CServer::GetLogonType() const
 {
 	return m_logonType;
 }
 
-void CServer::SetLogonType(enum LogonType logonType)
+void CServer::SetLogonType(LogonType logonType)
 {
 	wxASSERT(logonType != LOGONTYPE_MAX);
 	m_logonType = logonType;
 }
 
-void CServer::SetProtocol(enum ServerProtocol serverProtocol)
+void CServer::SetProtocol(ServerProtocol serverProtocol)
 {
 	wxASSERT(serverProtocol != UNKNOWN);
 
@@ -577,6 +592,16 @@ bool CServer::SetAccount(const wxString& account)
 	return true;
 }
 
+bool CServer::SetKeyFile(const wxString& keyFile)
+{
+	if (m_logonType != KEY)
+		return false;
+
+	m_keyFile = keyFile;
+
+	return true;
+}
+
 bool CServer::SetTimezoneOffset(int minutes)
 {
 	if (minutes > (60 * 24) || minutes < (-60 * 24))
@@ -592,12 +617,12 @@ int CServer::GetTimezoneOffset() const
 	return m_timezoneOffset;
 }
 
-enum PasvMode CServer::GetPasvMode() const
+PasvMode CServer::GetPasvMode() const
 {
 	return m_pasvMode;
 }
 
-void CServer::SetPasvMode(enum PasvMode pasvMode)
+void CServer::SetPasvMode(PasvMode pasvMode)
 {
 	m_pasvMode = pasvMode;
 }
@@ -612,36 +637,58 @@ int CServer::MaximumMultipleConnections() const
 	return m_maximumMultipleConnections;
 }
 
-wxString CServer::FormatHost(bool always_omit_port /*=false*/) const
+wxString CServer::Format(ServerFormat formatType) const
 {
-	wxString host = m_host;
+	t_protocolInfo const& info = GetProtocolInfo(m_protocol);
+	wxString server = m_host;
 
-	if (host.Find(':') != -1)
-		host = _T("[") + host + _T("]");
+	if (server.Find(':') != -1)
+		server = _T("[") + server + _T("]");
 
-	if (!always_omit_port)
-	{
-		if (m_port != GetDefaultPort(m_protocol))
-			host += wxString::Format(_T(":%d"), m_port);
+	if (formatType == ServerFormat::host_only) {
+		return server;
+	}
+	
+	if (m_port != GetDefaultPort(m_protocol)) {
+		server += wxString::Format(_T(":%d"), m_port);
 	}
 
-	return host;
-}
-
-wxString CServer::FormatServer(const bool always_include_prefix /*=false*/) const
-{
-	wxString server = FormatHost();
-
-	if (m_logonType != ANONYMOUS)
-		server = GetUser() + _T("@") + server;
-
-	const t_protocolInfo& info = GetProtocolInfo(m_protocol);
-	if (!info.prefix.empty())
-	{
-		if (always_include_prefix || info.alwaysShowPrefix)
-			server = info.prefix + _T("://") + server;
-		else if (m_port != info.defaultPort)
-			server = info.prefix + _T("://") + server;
+	if (formatType == ServerFormat::with_optional_port) {
+		return server;
+	}
+		
+	if (m_logonType != ANONYMOUS) {
+		auto user = GetUser().ToStdWstring();
+		// For now, only escape if formatting for URL.
+		// Open question: Do we need some form of escapement for presentation within the GUI,
+		// that deals e.g. with whitespace but does not touch Unicode characters?
+		if (formatType == ServerFormat::url || formatType == ServerFormat::url_with_password) {
+			user = url_encode(user);
+		}
+		if (!user.empty()) {
+			if (formatType == ServerFormat::url_with_password) {
+				auto pass = GetPass().ToStdWstring();
+				if (!pass.empty()) {
+					if (formatType == ServerFormat::url || formatType == ServerFormat::url_with_password) {
+						pass = url_encode(pass);
+					}
+					server = user + _T(":") + pass + _T("@") + server;
+				}
+			}
+			else {
+				server = url_encode(user) + _T("@") + server;
+			}
+		}
+	}
+	
+	if (formatType == ServerFormat::with_user_and_optional_port) {
+		if (!info.alwaysShowPrefix && m_port == info.defaultPort) {
+			return server;
+		}
+	}
+		
+	if (!info.prefix.empty()) {
+		server = info.prefix + _T("://") + server;
 	}
 
 	return server;
@@ -665,7 +712,7 @@ void CServer::Initialize()
 	m_bypassProxy = false;
 }
 
-bool CServer::SetEncodingType(enum CharsetEncoding type, const wxString& encoding)
+bool CServer::SetEncodingType(CharsetEncoding type, const wxString& encoding)
 {
 	if (type == ENCODING_CUSTOM && encoding.empty())
 		return false;
@@ -687,7 +734,7 @@ bool CServer::SetCustomEncoding(const wxString& encoding)
 	return true;
 }
 
-enum CharsetEncoding CServer::GetEncodingType() const
+CharsetEncoding CServer::GetEncodingType() const
 {
 	return m_encodingType;
 }
@@ -697,14 +744,14 @@ wxString CServer::GetCustomEncoding() const
 	return m_customEncoding;
 }
 
-unsigned int CServer::GetDefaultPort(enum ServerProtocol protocol)
+unsigned int CServer::GetDefaultPort(ServerProtocol protocol)
 {
 	const t_protocolInfo& info = GetProtocolInfo(protocol);
 
 	return info.defaultPort;
 }
 
-enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port, bool defaultOnly /*=false*/)
+ServerProtocol CServer::GetProtocolFromPort(unsigned int port, bool defaultOnly /*=false*/)
 {
 	for (unsigned int i = 0; protocolInfos[i].protocol != UNKNOWN; ++i)
 	{
@@ -719,7 +766,7 @@ enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port, bool default
 	return FTP;
 }
 
-wxString CServer::GetProtocolName(enum ServerProtocol protocol)
+wxString CServer::GetProtocolName(ServerProtocol protocol)
 {
 	const t_protocolInfo *protocolInfo = protocolInfos;
 	while (protocolInfo->protocol != UNKNOWN)
@@ -739,18 +786,15 @@ wxString CServer::GetProtocolName(enum ServerProtocol protocol)
 	return wxString();
 }
 
-enum ServerProtocol CServer::GetProtocolFromName(const wxString& name)
+ServerProtocol CServer::GetProtocolFromName(const wxString& name)
 {
 	const t_protocolInfo *protocolInfo = protocolInfos;
-	while (protocolInfo->protocol != UNKNOWN)
-	{
-		if (protocolInfo->translateable)
-		{
+	while (protocolInfo->protocol != UNKNOWN) {
+		if (protocolInfo->translateable) {
 			if (wxGetTranslation(protocolInfo->name) == name)
 				return protocolInfo->protocol;
 		}
-		else
-		{
+		else {
 			if (protocolInfo->name == name)
 				return protocolInfo->protocol;
 		}
@@ -762,9 +806,8 @@ enum ServerProtocol CServer::GetProtocolFromName(const wxString& name)
 
 bool CServer::SetPostLoginCommands(const std::vector<wxString>& postLoginCommands)
 {
-	if (m_protocol != FTP && m_protocol != FTPS && m_protocol != FTPES)
-	{
-		// Currently, only regular FTP supports it
+	if (!SupportsPostLoginCommands(m_protocol)) {
+		m_postLoginCommands.clear();
 		return false;
 	}
 
@@ -772,10 +815,14 @@ bool CServer::SetPostLoginCommands(const std::vector<wxString>& postLoginCommand
 	return true;
 }
 
-enum ServerProtocol CServer::GetProtocolFromPrefix(const wxString& prefix)
+bool CServer::SupportsPostLoginCommands(ServerProtocol const protocol)
 {
-	for (unsigned int i = 0; protocolInfos[i].protocol != UNKNOWN; ++i)
-	{
+	return protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP;
+}
+
+ServerProtocol CServer::GetProtocolFromPrefix(const wxString& prefix)
+{
+	for (unsigned int i = 0; protocolInfos[i].protocol != UNKNOWN; ++i) {
 		if (!protocolInfos[i].prefix.CmpNoCase(prefix))
 			return protocolInfos[i].protocol;
 	}
@@ -783,7 +830,7 @@ enum ServerProtocol CServer::GetProtocolFromPrefix(const wxString& prefix)
 	return UNKNOWN;
 }
 
-wxString CServer::GetPrefixFromProtocol(const enum ServerProtocol protocol)
+wxString CServer::GetPrefixFromProtocol(const ServerProtocol protocol)
 {
 	const t_protocolInfo& info = GetProtocolInfo(protocol);
 
@@ -800,25 +847,24 @@ bool CServer::GetBypassProxy() const
   return m_bypassProxy;
 }
 
-bool CServer::ProtocolHasDataTypeConcept(const enum ServerProtocol protocol)
+bool CServer::ProtocolHasDataTypeConcept(const ServerProtocol protocol)
 {
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
+	if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP)
 		return true;
 
 	return false;
 }
 
-wxString CServer::GetNameFromServerType(enum ServerType type)
+wxString CServer::GetNameFromServerType(ServerType type)
 {
 	wxASSERT(type != SERVERTYPE_MAX);
 	return wxGetTranslation(typeNames[type]);
 }
 
-enum ServerType CServer::GetServerTypeFromName(const wxString& name)
+ServerType CServer::GetServerTypeFromName(const wxString& name)
 {
-	for (int i = 0; i < SERVERTYPE_MAX; ++i)
-	{
-		enum ServerType type = (enum ServerType)i;
+	for (int i = 0; i < SERVERTYPE_MAX; ++i) {
+		ServerType type = static_cast<ServerType>(i);
 		if (name == CServer::GetNameFromServerType(type))
 			return type;
 	}
@@ -826,12 +872,14 @@ enum ServerType CServer::GetServerTypeFromName(const wxString& name)
 	return DEFAULT;
 }
 
-enum LogonType CServer::GetLogonTypeFromName(const wxString& name)
+LogonType CServer::GetLogonTypeFromName(const wxString& name)
 {
 	if (name == _("Normal"))
 		return NORMAL;
 	else if (name == _("Ask for password"))
 		return ASK;
+	else if (name == _("Key file"))
+		return KEY;
 	else if (name == _("Interactive"))
 		return INTERACTIVE;
 	else if (name == _("Account"))
@@ -840,7 +888,7 @@ enum LogonType CServer::GetLogonTypeFromName(const wxString& name)
 		return ANONYMOUS;
 }
 
-wxString CServer::GetNameFromLogonType(enum LogonType type)
+wxString CServer::GetNameFromLogonType(LogonType type)
 {
 	wxASSERT(type != LOGONTYPE_MAX);
 
@@ -850,6 +898,8 @@ wxString CServer::GetNameFromLogonType(enum LogonType type)
 		return _("Normal");
 	case ASK:
 		return _("Ask for password");
+	case KEY:
+		return _("Key file");
 	case INTERACTIVE:
 		return _("Interactive");
 	case ACCOUNT:
