@@ -20,13 +20,14 @@ static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Failsafe
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
 	{ _T("."),   false,  '[',  ']',    false, 0, '^', false, false },
-	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false },
+	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false }, // DOS with backslashes
 	{ _T("."),   false, '\'', '\'',     true, 1, 0,   false, false },
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Same as Unix
 	{ _T("."),   false,    0,    0,    false, 0, 0,   false, false },
 	{ _T("\\"),  true,     0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  }  // Cygwin is like Unix but has optional prefix of form "//server"
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  },  // Cygwin is like Unix but has optional prefix of form "//server"
+	{ _T("/\\"), false,    0,    0,    false, 0, 0,   true,  false } // DOS with forwardslashes
 };
 
 bool CServerPathData::operator==(const CServerPathData& cmp) const
@@ -49,11 +50,13 @@ CServerPath::CServerPath(CServerPath const& path, wxString subdir)
 	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
-	if (subdir.empty())
+	if (subdir.empty()) {
 		return;
+	}
 
-	if (!ChangePath(subdir))
+	if (!ChangePath(subdir)) {
 		clear();
+	}
 }
 
 CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
@@ -77,8 +80,9 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	wxString path = newPath;
 	wxString file;
 
-	if (path.empty())
+	if (path.empty()) {
 		return false;
+	}
 
 	if (m_type == DEFAULT) {
 		int pos1 = path.Find(_T(":["));
@@ -95,8 +99,7 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 				m_type = DOS;
 		else if (path.c_str()[0] == FTP_MVS_DOUBLE_QUOTE && path.Last() == FTP_MVS_DOUBLE_QUOTE)
 			m_type = MVS;
-		else if (path[0] == ':' && (pos1 = path.Mid(1).Find(':')) > 0)
-		{
+		else if (path[0] == ':' && (pos1 = path.Mid(1).Find(':')) > 0) {
 			int slash = path.Find('/');
 			if (slash == -1 || slash > pos1)
 				m_type = VXWORKS;
@@ -133,9 +136,9 @@ wxString CServerPath::GetPath() const
 	if (m_data->m_segments.empty() && (!traits[m_type].has_root || !m_data->m_prefix || traits[m_type].separator_after_prefix))
 		path += traits[m_type].separators[0];
 
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter) {
-		const wxString& segment = *iter;
-		if (iter != m_data->m_segments.begin())
+	for (auto iter = m_data->m_segments.cbegin(); iter != m_data->m_segments.cend(); ++iter) {
+		std::wstring const& segment = *iter;
+		if (iter != m_data->m_segments.cbegin())
 			path += traits[m_type].separators[0];
 		else if (traits[m_type].has_root) {
 			if (!m_data->m_prefix || traits[m_type].separator_after_prefix)
@@ -159,7 +162,7 @@ wxString CServerPath::GetPath() const
 
 	// DOS is strange.
 	// C: is current working dir on drive C, C:\ the drive root.
-	if (m_type == DOS && m_data->m_segments.size() == 1)
+	if ((m_type == DOS || m_type == DOS_FWD_SLASHES) && m_data->m_segments.size() == 1)
 		path += traits[m_type].separators[0];
 
 	return path;
@@ -182,12 +185,13 @@ CServerPath CServerPath::GetParent() const
 		return CServerPath();
 
 	CServerPath parent(*this);
-	CServerPathData& parent_data = parent.m_data.Get();
+	CServerPathData& parent_data = parent.m_data.get();
 
 	parent_data.m_segments.pop_back();
 
-	if (m_type == MVS)
-		parent_data.m_prefix = CSparseOptional<wxString>(_T("."));
+	if (m_type == MVS) {
+		parent_data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
+	}
 
 	return parent;
 }
@@ -281,7 +285,7 @@ bool CServerPath::SetSafePath(const wxString& path)
 
 bool CServerPath::DoSetSafePath(const wxString& path)
 {
-	CServerPathData& data = m_data.Get();
+	CServerPathData& data = m_data.get();
 	data.m_prefix.clear();
 	data.m_segments.clear();
 
@@ -289,34 +293,33 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 	// Before the optimization this function was responsible for
 	// most CPU cycles used during loading of transfer queues
 	// from file
-	const int len = (int)path.Len();
-	wxCharTypeBuffer<wxChar> buf(len + 1);
-	wxChar* begin = buf.data();
-
-	memcpy(begin, (const wxChar*)path.c_str(), (len + 1) * sizeof(wxChar));
-	wxChar* p = begin;
+	wxChar const* const begin = path.c_str();
+	wxChar const* const end = begin + path.size();
+	
+	wxChar const* p = begin;
 
 	int type = 0;
-	do
-	{
-		if (*p < '0' || *p > '9')
+	do {
+		if (*p < '0' || *p > '9') {
 			return false;
+		}
 		type *= 10;
 		type += *p - '0';
 
-		if (type >= SERVERTYPE_MAX)
+		if (type >= SERVERTYPE_MAX) {
 			return false;
+		}
 		++p;
 	} while (*p != ' ');
 
-	m_type = (ServerType)type;
+	m_type = static_cast<ServerType>(type);
 	++p;
 
 	int prefix_len = 0;
-	do
-	{
-		if (*p < '0' || *p > '9')
+	do {
+		if (*p < '0' || *p > '9') {
 			return false;
+		}
 		prefix_len *= 10;
 		prefix_len += *p - '0';
 
@@ -327,8 +330,9 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 	while (*p && *p != ' ');
 
 	if (!*p) {
-		if (prefix_len != 0)
+		if (prefix_len != 0) {
 			return false;
+		}
 		else {
 			// Is root directory, like / on unix like systems.
 			return true;
@@ -337,43 +341,39 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 
 	++p;
 
-	if (len - (p - begin) < prefix_len)
+	if (prefix_len > end - p) {
 		return false;
-	if (prefix_len)
-	{
-		*(p + prefix_len) = 0;
-		if( *p ) {
-			data.m_prefix = CSparseOptional<wxString>(p);
-		}
-
+	}
+	if (prefix_len) {
+		data.m_prefix = fz::sparse_optional<std::wstring>(new std::wstring(p, p + prefix_len));
 		p += prefix_len + 1;
 	}
 
-	while (len > (p - begin))
-	{
+	while (p < end) {
 		int segment_len = 0;
-		do
-		{
-			if (*p < '0' || *p > '9')
+		do {
+			if (*p < '0' || *p > '9') {
 				return false;
+			}
 			segment_len *= 10;
 			segment_len += *p - '0';
 
-			if (segment_len > 32767) // Should be sane enough
+			if (segment_len > 32767) { // Should be sane enough
 				return false;
+			}
 			++p;
 		}
 		while (*p != ' ');
 
-		if (!segment_len)
+		if (!segment_len) {
 			return false;
+		}
 		++p;
 
-		if (len - (p - begin) < segment_len)
+		if (segment_len > end - p) {
 			return false;
-		*(p + segment_len) = 0;
-		wxString s(p);
-		data.m_segments.push_back(s);
+		}
+		data.m_segments.emplace_back(wxString(p, p + segment_len));
 
 		p += segment_len + 1;
 	}
@@ -383,8 +383,9 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 
 bool CServerPath::SetType(ServerType type)
 {
-	if (!empty() && m_type != DEFAULT)
+	if (!empty() && m_type != DEFAULT) {
 		return false;
+	}
 
 	m_type = type;
 
@@ -415,7 +416,7 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 			else if( !m_data->m_prefix && path.m_data->m_prefix ) {
 				return false;
 			}
-			else if( m_data->m_prefix && path.m_data->m_prefix && m_data->m_prefix->CmpNoCase(*path.m_data->m_prefix) ) {
+			else if( m_data->m_prefix && path.m_data->m_prefix && fz::stricmp(*m_data->m_prefix, *path.m_data->m_prefix) ) {
 				return false;
 			}
 		}
@@ -431,14 +432,17 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 	tConstSegmentIter iter1 = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
 	while (iter1 != m_data->m_segments.end()) {
-		if (iter2 == path.m_data->m_segments.end())
+		if (iter2 == path.m_data->m_segments.end()) {
 			return true;
-		if (cmpNoCase) {
-			if (iter1->CmpNoCase(*iter2))
-				return false;
 		}
-		else if (*iter1 != *iter2)
+		if (cmpNoCase) {
+			if (fz::stricmp(*iter1, *iter2)) {
+				return false;
+			}
+		}
+		else if (*iter1 != *iter2) {
 			return false;
+		}
 
 		++iter1;
 		++iter2;
@@ -452,7 +456,7 @@ bool CServerPath::IsParentOf(const CServerPath &path, bool cmpNoCase) const
 	return path.IsSubdirOf(*this, cmpNoCase);
 }
 
-bool CServerPath::ChangePath(wxString subdir)
+bool CServerPath::ChangePath(wxString const& subdir)
 {
 	wxString subdir2 = subdir;
 	return ChangePath(subdir2, false);
@@ -481,7 +485,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 	}
 
 	bool const was_empty = empty();
-	CServerPathData& data = m_data.Get();
+	CServerPathData& data = m_data.get();
 
 	switch (m_type)
 	{
@@ -518,19 +522,20 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 				dir = dir.Left(pos2);
 
 				if (pos1)
-					data.m_prefix = CSparseOptional<wxString>(dir.Left(pos1));
+					data.m_prefix = fz::sparse_optional<std::wstring>(dir.Left(pos1).ToStdWstring());
 				dir = dir.Mid(pos1 + 1);
 
 				data.m_segments.clear();
 			}
 
-			if (!Segmentize(dir, data.m_segments))
+			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 				return false;
 			if (data.m_segments.empty() && was_empty)
 				return false;
 		}
 		break;
 	case DOS:
+	case DOS_FWD_SLASHES:
 		{
 			bool is_relative = false;
 			int sep = dir.find_first_of(traits[m_type].separators);
@@ -546,7 +551,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			{
 				if (data.m_segments.empty())
 					return false;
-				wxString first = data.m_segments.front();
+				std::wstring first = data.m_segments.front();
 				data.m_segments.clear();
 				data.m_segments.push_back(first);
 				dir = dir.Mid(1);
@@ -555,7 +560,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			if (isFile && !ExtractFile(dir, file))
 				return false;
 
-			if (!Segmentize(dir, data.m_segments))
+			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 				return false;
 			if (data.m_segments.empty() && was_empty)
 				return false;
@@ -622,15 +627,15 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			if (isFile) {
 				if (!ExtractFile(dir, file))
 					return false;
-				data.m_prefix = CSparseOptional<wxString>(_T("."));
+				data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
 			}
 			else if (!dir.empty() && dir.Last() == '.')
-				data.m_prefix = CSparseOptional<wxString>(_T("."));
+				data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
 			else
 				data.m_prefix.clear();
 		}
 
-		if (!Segmentize(dir, data.m_segments))
+		if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 			return false;
 		break;
 	case HPNONSTOP:
@@ -640,7 +645,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 		if (isFile && !ExtractFile(dir, file))
 			return false;
 
-		if (!Segmentize(dir, data.m_segments))
+		if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 			return false;
 		if (data.m_segments.empty() && was_empty)
 			return false;
@@ -658,7 +663,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 				int colon2;
 				if ((colon2 = dir.Mid(1).Find(':')) < 1)
 					return false;
-				data.m_prefix = CSparseOptional<wxString>(dir.Left(colon2 + 2));
+				data.m_prefix = fz::sparse_optional<std::wstring>(dir.Left(colon2 + 2).ToStdWstring());
 				dir = dir.Mid(colon2 + 2);
 
 				data.m_segments.clear();
@@ -667,29 +672,27 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			if (isFile && !ExtractFile(dir, file))
 				return false;
 
-			if (!Segmentize(dir, data.m_segments))
+			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 				return false;
 		}
 		break;
 	case CYGWIN:
 		{
-			if (wxString(traits[m_type].separators).Contains(dir[0]))
-			{
+			if (wxString(traits[m_type].separators).Contains(dir[0])) {
 				data.m_segments.clear();
 				data.m_prefix.clear();
 			}
 			else if (was_empty)
 				return false;
-			if (dir.Left(2) == _T("//"))
-			{
-				data.m_prefix = CSparseOptional<wxString>(traits[m_type].separators[0]);
+			if (dir.Left(2) == _T("//")) {
+				data.m_prefix = fz::sparse_optional<std::wstring>(std::wstring(1, traits[m_type].separators[0]));
 				dir = dir.Mid(1);
 			}
 
 			if (isFile && !ExtractFile(dir, file))
 				return false;
 
-			if (!Segmentize(dir, data.m_segments))
+			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 				return false;
 		}
 		break;
@@ -703,7 +706,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			if (isFile && !ExtractFile(dir, file))
 				return false;
 
-			if (!Segmentize(dir, data.m_segments))
+			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
 				return false;
 		}
 		break;
@@ -767,14 +770,17 @@ bool CServerPath::operator<(const CServerPath &op) const
 
 	tConstSegmentIter iter1, iter2;
 	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2) {
-		if (iter2 == op.m_data->m_segments.end())
+		if (iter2 == op.m_data->m_segments.end()) {
 			return false;
+		}
 
-		const int cmp = iter1->Cmp(*iter2);
-		if (cmp < 0)
+		const int cmp = std::wcscmp(iter1->c_str(), iter2->c_str());
+		if (cmp < 0) {
 			return true;
-		if (cmp > 0)
+		}
+		if (cmp > 0) {
 			return false;
+		}
 	}
 
 	return iter2 != op.m_data->m_segments.end();
@@ -838,9 +844,10 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = op.m_data->m_segments.begin();
 	while (iter != m_data->m_segments.end()) {
-		int res = iter++->CmpNoCase(*iter2++);
-		if (res)
+		int res = fz::stricmp(*(iter++), *(iter2++));
+		if (res) {
 			return res;
+		}
 	}
 
 	return 0;
@@ -852,7 +859,7 @@ bool CServerPath::AddSegment(const wxString& segment)
 		return false;
 
 	// TODO: Check for invalid characters
-	m_data.Get().m_segments.push_back(segment);
+	m_data.get().m_segments.push_back(segment.ToStdWstring());
 
 	return true;
 }
@@ -889,7 +896,7 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	CServerPath parent;
 	parent.m_type = m_type;
 
-	CServerPathData& parentData = parent.m_data.Get();
+	CServerPathData& parentData = parent.m_data.get();
 
 	tConstSegmentIter last = m_data->m_segments.end();
 	tConstSegmentIter last2 = path.m_data->m_segments.end();
@@ -933,7 +940,7 @@ wxString CServerPath::FormatSubdir(const wxString &subdir) const
 	return res;
 }
 
-bool CServerPath::SegmentizeAddSegment(wxString & segment, tSegmentList& segments, bool& append)
+bool CServerPath::SegmentizeAddSegment(std::wstring & segment, tSegmentList& segments, bool& append)
 {
 	if (traits[m_type].has_dots) {
 		if (segment == _T("."))
@@ -949,7 +956,7 @@ bool CServerPath::SegmentizeAddSegment(wxString & segment, tSegmentList& segment
 	}
 
 	bool append_next = false;
-	if (!segment.empty() && traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape) {
+	if (!segment.empty() && traits[m_type].separatorEscape && segment.back() == traits[m_type].separatorEscape) {
 		append_next = true;
 		segment[segment.size() - 1] = traits[m_type].separators[0];
 	}
@@ -964,7 +971,7 @@ bool CServerPath::SegmentizeAddSegment(wxString & segment, tSegmentList& segment
 	return true;
 }
 
-bool CServerPath::Segmentize(wxString const& str, tSegmentList& segments)
+bool CServerPath::Segmentize(std::wstring const& str, tSegmentList& segments)
 {
 	bool append = false;
 	size_t start = 0;
@@ -980,7 +987,7 @@ bool CServerPath::Segmentize(wxString const& str, tSegmentList& segments)
 			continue;
 		}
 
-		wxString segment = str.substr(start, pos - start);
+		std::wstring segment = str.substr(start, pos - start);
 		start = pos + 1;
 
 		if (!SegmentizeAddSegment(segment, segments, append)) {
@@ -989,7 +996,7 @@ bool CServerPath::Segmentize(wxString const& str, tSegmentList& segments)
 	}
 
 	if (start < str.size()) {
-		wxString segment = str.substr(start);
+		std::wstring segment = str.substr(start);
 		if (!SegmentizeAddSegment(segment, segments, append)) {
 			return false;
 		}
