@@ -25,7 +25,7 @@ public:
 	CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode);
 	virtual ~CTransferSocket();
 
-	wxString SetupActiveTransfer(const wxString& ip);
+	wxString SetupActiveTransfer(std::string const& ip);
 	bool SetupPassiveTransfer(wxString host, int port);
 
 	void SetActive();
@@ -56,6 +56,7 @@ protected:
 	void OnReceive();
 	void OnSend();
 	void OnClose(int error);
+	void OnTimer(fz::timer_id);
 
 	// Create a socket server
 	CSocket* CreateSocketServer();
