@@ -1,6 +1,8 @@
 #ifndef __LOCAL_PATH_H__
 #define __LOCAL_PATH_H__
 
+#include <libfilezilla/shared.hpp>
+
 // This class encapsulates local paths.
 // On Windows it uses the C:\foo\bar\ syntax and also supports
 // UNC paths.
@@ -9,24 +11,27 @@
 class CLocalPath final
 {
 public:
-	CLocalPath() {}
-	CLocalPath(const CLocalPath &path);
+	CLocalPath() = default;
+	CLocalPath(CLocalPath const& path) = default;
+	CLocalPath(CLocalPath && path) noexcept = default;
+	CLocalPath& operator=(CLocalPath const& path) = default;
+	CLocalPath& operator=(CLocalPath && path) noexcept = default;
 
 	// Creates path. If the path is not syntactically
 	// correct, empty() will return true.
 	// If file is given and path not terminated by a separator,
 	// the filename portion is returned in file.
 	explicit CLocalPath(const wxString& path, wxString* file = 0);
-	bool SetPath(const wxString& path, wxString* file = 0);
+	bool SetPath(std::wstring const& path, wxString* file = 0);
 
 	// Always terminated by a separator
-	const wxString& GetPath() const { return m_path; }
+	std::wstring const& GetPath() const { return *m_path; }
 
 	bool empty() const;
 	void clear();
 
 	// On failure the path is undefined
-	bool ChangePath(const wxString& path);
+	bool ChangePath(const wxString& new_path);
 
 	// Do not call with separators in the segment
 	void AddSegment(const wxString& segment);
@@ -58,15 +63,14 @@ public:
 	// Checks if the directory exists.
 	bool Exists(wxString *error = 0) const;
 
-	static const wxChar path_separator;
+	static wchar_t const path_separator;
 
 	bool operator==(const CLocalPath& op) const;
 	bool operator!=(const CLocalPath& op) const;
 
-	void Coalesce();
+	bool operator<(CLocalPath const& op) const;
 protected:
-
-	wxString m_path;
+	fz::shared_value<std::wstring> m_path;
 };
 
 #endif //__LOCAL_PATH_H__
