@@ -22,13 +22,14 @@ enum ServerType
 	DEFAULT,
 	UNIX,
 	VMS,
-	DOS,
+	DOS, // Backslashes as preferred separator
 	MVS,
 	VXWORKS,
 	ZVM,
 	HPNONSTOP,
 	DOS_VIRTUAL,
 	CYGWIN,
+	DOS_FWD_SLASHES, // Forwardslashes as preferred separator
 
 	SERVERTYPE_MAX
 };
@@ -52,6 +53,15 @@ enum PasvMode
 	MODE_PASSIVE
 };
 
+enum class ServerFormat
+{
+	host_only,
+	with_optional_port,
+	with_user_and_optional_port,
+	url,
+	url_with_password
+};
+
 enum CharsetEncoding
 {
 	ENCODING_AUTO,
@@ -110,8 +120,7 @@ public:
 	void SetPasvMode(PasvMode pasvMode);
 	void MaximumMultipleConnections(int maximum);
 
-	wxString FormatHost(bool always_omit_port = false) const;
-	wxString FormatServer(const bool always_include_prefix = false) const;
+	wxString Format(ServerFormat formatType) const;
 
 	bool SetEncodingType(CharsetEncoding type, const wxString& encoding = wxString());
 	bool SetCustomEncoding(const wxString& encoding);
@@ -140,8 +149,8 @@ public:
 
 	// Abstract server name.
 	// Not compared in ==, < and related operators
-	void SetName(const wxString& name) { m_name = name; }
-	wxString GetName() const { return m_name; }
+	void SetName(std::wstring const& name) { m_name = name; }
+	std::wstring GetName() const { return m_name; }
 
 	static wxString GetNameFromServerType(ServerType type);
 	static ServerType GetServerTypeFromName(const wxString& name);
@@ -149,6 +158,8 @@ public:
 	static wxString GetNameFromLogonType(LogonType type);
 	static LogonType GetLogonTypeFromName(const wxString& name);
 
+	explicit operator bool() const { return !m_host.empty(); }
+
 protected:
 	void Initialize();
 
@@ -166,7 +177,7 @@ protected:
 	int m_maximumMultipleConnections;
 	CharsetEncoding m_encodingType;
 	wxString m_customEncoding;
-	wxString m_name;
+	std::wstring m_name;
 
 	std::vector<wxString> m_postLoginCommands;
 	bool m_bypassProxy;
