@@ -1,16 +1,16 @@
 #ifndef __SERVERPATH_H__
 #define __SERVERPATH_H__
 
-#include "optional.h"
-#include "refcount.h"
+#include <libfilezilla/optional.hpp>
+#include <libfilezilla/shared.hpp>
 
 #include <deque>
 
 class CServerPathData final
 {
 public:
-	std::deque<wxString> m_segments;
-	CSparseOptional<wxString> m_prefix;
+	std::deque<std::wstring> m_segments;
+	fz::sparse_optional<std::wstring> m_prefix;
 
 	bool operator==(const CServerPathData& cmp) const;
 };
@@ -36,7 +36,7 @@ public:
 
 	// If ChangePath returns false, the object will be left
 	// empty.
-	bool ChangePath(wxString subdir);
+	bool ChangePath(wxString const& subdir);
 	bool ChangePath(wxString &subdir, bool isFile);
 
 	wxString GetPath() const;
@@ -64,7 +64,7 @@ public:
 	// always use absolute filenames including the full path
 	wxString FormatFilename(const wxString &filename, bool omitPath = false) const;
 
-	// Returns identity on all but VMS. On VMS it esscapes dots
+	// Returns identity on all but VMS. On VMS it escapes dots
 	wxString FormatSubdir(const wxString &subdir) const;
 
 	bool AddSegment(const wxString& segment);
@@ -76,17 +76,17 @@ private:
 
 	ServerType m_type;
 
-	typedef std::deque<wxString> tSegmentList;
+	typedef std::deque<std::wstring> tSegmentList;
 	typedef tSegmentList::iterator tSegmentIter;
 	typedef tSegmentList::const_iterator tConstSegmentIter;
 
-	bool Segmentize(wxString const& str, tSegmentList& segments);
-	bool SegmentizeAddSegment(wxString & segment, tSegmentList& segments, bool& append);
+	bool Segmentize(std::wstring const& str, tSegmentList& segments);
+	bool SegmentizeAddSegment(std::wstring & segment, tSegmentList& segments, bool& append);
 	bool ExtractFile(wxString& dir, wxString& file);
 
 	static void EscapeSeparators(ServerType type, wxString& subdir);
 
-	CRefcountObject_Uninitialized<CServerPathData> m_data;
+	fz::shared_optional<CServerPathData> m_data;
 };
 
 #endif
