@@ -31,13 +31,7 @@ struct socket_event_type;
 typedef fz::simple_event<socket_event_type, CSocketEventSource*, SocketEventType, int> CSocketEvent;
 
 struct hostaddress_event_type;
-typedef fz::simple_event<hostaddress_event_type, CSocketEventSource*, wxString> CHostAddressEvent;
-
-class CCallback
-{
-public:
-	virtual void cb() {}
-};
+typedef fz::simple_event<hostaddress_event_type, CSocketEventSource*, std::string> CHostAddressEvent;
 
 void RemoveSocketEvents(fz::event_handler * handler, CSocketEventSource const* const source);
 void ChangeSocketEventHandler(fz::event_handler * oldHandler, fz::event_handler * newHandler, CSocketEventSource const* const source);
@@ -86,7 +80,7 @@ public:
 	// If host is a name that can be resolved, a hostaddress socket event gets sent.
 	// Once connections got established, a connection event gets sent. If
 	// connection could not be established, a close event gets sent.
-	int Connect(wxString const& host, unsigned int port, address_family family = unspec, wxString const& bind = wxString());
+	int Connect(fz::native_string const& host, unsigned int port, address_family family = unspec, std::string const& bind = std::string());
 
 	// After receiving a send or receive event, you can call these functions
 	// as long as their return value is positive.
@@ -97,11 +91,11 @@ public:
 	int Close();
 
 	// Returns empty string on error
-	wxString GetLocalIP(bool strip_zone_index = false) const;
-	wxString GetPeerIP(bool strip_zone_index = false) const;
+	std::string GetLocalIP(bool strip_zone_index = false) const;
+	std::string GetPeerIP(bool strip_zone_index = false) const;
 
 	// Returns the hostname passed to Connect()
-	wxString GetPeerHost() const;
+	fz::native_string GetPeerHost() const;
 
 	// -1 on error
 	int GetLocalPort(int& error);
@@ -110,17 +104,17 @@ public:
 	// If connected, either ipv4 or ipv6, unspec otherwise
 	address_family GetAddressFamily() const;
 
-	static wxString GetErrorString(int error);
-	static wxString GetErrorDescription(int error);
+	static std::string GetErrorString(int error);
+	static fz::native_string GetErrorDescription(int error);
 
-	// Can only be called if the state is none
+	// Due to asynchronicity it is possible that the old handler receives one last 
+	// socket event when changing handlers.
 	void SetEventHandler(fz::event_handler* pEvtHandler);
-	fz::event_handler* GetEventHandler() { return m_pEvtHandler; }
 
 	static void Cleanup(bool force);
 
-	static wxString AddressToString(const struct sockaddr* addr, int addr_len, bool with_port = true, bool strip_zone_index = false);
-	static wxString AddressToString(char const* buf, int buf_len);
+	static std::string AddressToString(const struct sockaddr* addr, int addr_len, bool with_port = true, bool strip_zone_index = false);
+	static std::string AddressToString(char const* buf, int buf_len);
 
 	int Listen(address_family family, int port = 0);
 	CSocket* Accept(int& error);
@@ -136,34 +130,42 @@ public:
 
 	// If called on listen socket, sizes will be inherited by
 	// accepted sockets
-	int SetBufferSizes(int size_read, int size_write);
+	int SetBufferSizes(int size_receive, int size_send);
 
-	void SetSynchronousReadCallback(CCallback* cb);
+	// Duration must not be smaller than 5 minutes.
+	// Default interval if 2 hours.
+	void SetKeepaliveInterval(fz::duration const& d);
+
+	// On a connected socket, gets the ideal send buffer size or
+	// -1 if it cannot be determined.
+	//
+	// Currently only implemented for Windows.
+	int GetIdealSendBufferSize();
 
 protected:
-	static int DoSetFlags(int fd, int flags, int flags_mask);
+	static int DoSetFlags(int fd, int flags, int flags_mask, fz::duration const&);
 	static int DoSetBufferSizes(int fd, int size_read, int size_write);
 	static int SetNonblocking(int fd);
 
-	void DetachThread();
+	// Note: Unlocks the lock.
+	void DetachThread(fz::scoped_lock & l);
 
 	fz::event_handler* m_pEvtHandler;
 
-	int m_fd;
+	int m_fd{-1};
 
-	SocketState m_state;
+	SocketState m_state{none};
 
-	CSocketThread* m_pSocketThread;
+	CSocketThread* m_pSocketThread{};
 
-	wxString m_host;
-	unsigned int m_port;
+	fz::native_string m_host;
+	unsigned int m_port{};
 	int m_family;
 
-	int m_flags;
+	int m_flags{};
+	fz::duration m_keepalive_interval;
 
 	int m_buffer_sizes[2];
-
-	CCallback* m_synchronous_read_cb{};
 };
 
 #ifdef FZ_WINDOWS
