@@ -14,31 +14,8 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
-#include <wx/xrc/xh_animatctrl.h>
-#include <wx/xrc/xh_bmpbt.h>
-#include <wx/xrc/xh_bttn.h>
-#include <wx/xrc/xh_chckb.h>
-#include <wx/xrc/xh_chckl.h>
-#include <wx/xrc/xh_choic.h>
-#include <wx/xrc/xh_dlg.h>
-#include <wx/xrc/xh_gauge.h>
-#include <wx/xrc/xh_listb.h>
-#include <wx/xrc/xh_listc.h>
-#include <wx/xrc/xh_menu.h>
-#include <wx/xrc/xh_notbk.h>
-#include <wx/xrc/xh_panel.h>
-#include <wx/xrc/xh_radbt.h>
-#include <wx/xrc/xh_scwin.h>
-#include <wx/xrc/xh_sizer.h>
-#include <wx/xrc/xh_spin.h>
-#include <wx/xrc/xh_stbmp.h>
-#include <wx/xrc/xh_stbox.h>
-#include <wx/xrc/xh_stlin.h>
-#include <wx/xrc/xh_sttxt.h>
-#include "xh_text_ex.h"
-#include <wx/xrc/xh_tree.h>
-#include <wx/xrc/xh_hyperlink.h>
-#include "xh_toolb_ex.h"
+#include "wxfilesystem_blob_handler.h"
+#include "xrc_helper.h"
 #ifdef __WXMSW__
 #include <wx/dynlib.h>
 #endif
@@ -65,7 +42,7 @@ IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 
 CFileZillaApp::CFileZillaApp()
 {
-	m_profilingActive = true;
+	m_profile_start = fz::monotonic_clock::now();
 	AddStartupProfileRecord(_T("CFileZillaApp::CFileZillaApp()"));
 }
 
@@ -438,36 +415,11 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
 #endif
 
-	pResource->AddHandler(new wxMenuXmlHandler);
-	pResource->AddHandler(new wxMenuBarXmlHandler);
-	pResource->AddHandler(new wxDialogXmlHandler);
-	pResource->AddHandler(new wxPanelXmlHandler);
-	pResource->AddHandler(new wxSizerXmlHandler);
-	pResource->AddHandler(new wxButtonXmlHandler);
-	pResource->AddHandler(new wxBitmapButtonXmlHandler);
-	pResource->AddHandler(new wxStaticTextXmlHandler);
-	pResource->AddHandler(new wxStaticBoxXmlHandler);
-	pResource->AddHandler(new wxStaticBitmapXmlHandler);
-	pResource->AddHandler(new wxTreeCtrlXmlHandler);
-	pResource->AddHandler(new wxListCtrlXmlHandler);
-	pResource->AddHandler(new wxCheckListBoxXmlHandler);
-	pResource->AddHandler(new wxChoiceXmlHandler);
-	pResource->AddHandler(new wxGaugeXmlHandler);
-	pResource->AddHandler(new wxCheckBoxXmlHandler);
-	pResource->AddHandler(new wxSpinCtrlXmlHandler);
-	pResource->AddHandler(new wxRadioButtonXmlHandler);
-	pResource->AddHandler(new wxNotebookXmlHandler);
-	pResource->AddHandler(new wxTextCtrlXmlHandlerEx);
-	pResource->AddHandler(new wxListBoxXmlHandler);
-	pResource->AddHandler(new wxToolBarXmlHandlerEx);
-	pResource->AddHandler(new wxStaticLineXmlHandler);
-	pResource->AddHandler(new wxScrolledWindowXmlHandler);
-	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
-	pResource->AddHandler(new wxAnimationCtrlXmlHandler);
-	pResource->AddHandler(new wxStdDialogButtonSizerXmlHandler);
+	InitHandlers(*pResource);
 
 	if (fz::local_filesys::get_file_type(fz::to_native(m_resourceDir.GetPath() + _T("xrc/resources.xrc"))) == fz::local_filesys::file) {
-		pResource->LoadFile(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
+		wxFileName fn(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
+		pResource->LoadFile(fn);
 	}
 	else {
 		fz::local_filesys fs;
@@ -483,6 +435,9 @@ bool CFileZillaApp::LoadResourceFiles()
 		}
 	}
 
+	// Useful for XRC files with embedded image data.
+	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
+
 	return true;
 }
 
@@ -525,7 +480,7 @@ bool CFileZillaApp::LoadLocales()
 		}
 	}
 #else
-	m_localesDir.SetPath(wxStandardPaths::Get().GetDataDir() + _T("/locales"));
+	m_localesDir.SetPath(wxStandardPaths::Get().GetDataDir().ToStdWstring() + _T("/locales"));
 #endif
 
 	if (!m_localesDir.empty()) {
@@ -543,20 +498,30 @@ bool CFileZillaApp::SetLocale(int language)
 	auto pLocale = std::make_unique<wxLocale>();
 	wxLogNull log;
 	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
+	if (!pLocale->IsOk()) {
+		return false;
+	}
+	
+	// Load catalog. Only fail if a non-default language has been selected
+	if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
 		return false;
 	}
 
-	// Now unload old locale
-	// We unload new locale as well, else the internal locale chain in wxWidgets get's broken.
-	pLocale.reset();
-	m_pLocale.reset();
+	if (m_pLocale) {
+		// Now unload old locale
+		// We unload new locale as well, else the internal locale chain in wxWidgets get's broken.
+		pLocale.reset();
+		m_pLocale.reset();
 
-	// Finally load new one
-	pLocale = std::make_unique<wxLocale>();
-	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
-		return false;
+		// Finally load new one
+		pLocale = std::make_unique<wxLocale>();
+		pLocale->Init(language);
+		if (!pLocale->IsOk()) {
+			return false;
+		}
+		else if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
+			return false;
+		}
 	}
 	m_pLocale = std::move(pLocale);
 
@@ -750,31 +715,30 @@ int CFileZillaApp::ProcessCommandLine()
 
 void CFileZillaApp::AddStartupProfileRecord(const wxString& msg)
 {
-	if (!m_profilingActive)
+	if (!m_profile_start) {
 		return;
+	}
 
-	m_startupProfile.push_back(std::make_pair(wxDateTime::UNow(), msg));
+	m_startupProfile.emplace_back(fz::monotonic_clock::now(), msg);
 }
 
 void CFileZillaApp::ShowStartupProfile()
 {
-	m_profilingActive = false;
+	if (m_profile_start && m_pCommandLine && m_pCommandLine->HasSwitch(CCommandLine::debug_startup)) {
+		wxString msg = _T("Profile:\n");
+		for (auto const& p : m_startupProfile) {
+			auto const diff = p.first - m_profile_start;
 
-	std::list<std::pair<wxDateTime, wxString> > profile;
-	profile.swap(m_startupProfile);
-
-	if (m_pCommandLine && !m_pCommandLine->HasSwitch(CCommandLine::debug_startup))
-		return;
-
-	wxString msg = _T("Profile:\n");
-	for (auto const& p : profile) {
-		msg += p.first.Format(_T("%Y-%m-%d %H:%M:%S %l"));
-		msg += _T(" ");
-		msg += p.second;
-		msg += _T("\n");
+			msg += std::to_wstring(diff.get_milliseconds());
+			msg += _T(" ");
+			msg += p.second;
+			msg += _T("\n");
+		}
+		wxMessageBoxEx(msg);
 	}
 
-	wxMessageBoxEx(msg);
+	m_profile_start = fz::monotonic_clock();
+	m_startupProfile.clear();
 }
 
 wxString CFileZillaApp::GetSettingsFile(wxString const& name) const
