@@ -2,25 +2,28 @@
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
-#include "RemoteListView.h"
-#include "commandqueue.h"
-#include "queue.h"
-#include "filezillaapp.h"
-#include "inputdialog.h"
 #include "chmoddialog.h"
-#include "filter.h"
-#include <algorithm>
-#include <wx/dcclient.h>
-#include <wx/dnd.h>
+#include "commandqueue.h"
 #include "dndobjects.h"
-#include "Options.h"
-#include "recursive_operation.h"
-#include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include <wx/clipbrd.h>
+#include "edithandler.h"
+#include "filezillaapp.h"
+#include "filter.h"
+#include "graphics.h"
+#include "inputdialog.h"
+#include "Options.h"
+#include "queue.h"
+#include "RemoteListView.h"
+#include "remote_recursive_operation.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
+
+#include <wx/clipbrd.h>
+#include <wx/dcclient.h>
+
+#include <algorithm>
+
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
@@ -91,7 +94,7 @@ public:
 				}
 			}
 
-			m_pRemoteListView->m_pState->UploadDroppedFiles(m_pFileDataObject, subdir, false);
+			m_pRemoteListView->m_state.UploadDroppedFiles(m_pFileDataObject, subdir, false);
 			return wxDragCopy;
 		}
 
@@ -103,7 +106,7 @@ public:
 			return wxDragNone;
 		}
 
-		if (!m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteListView->m_pState->GetServer()))
+		if (!m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteListView->m_state.GetServer()))
 		{
 			wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 			return wxDragNone;
@@ -163,13 +166,13 @@ public:
 		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 		{
 			const CRemoteDataObject::t_fileInfo& info = *iter;
-			m_pRemoteListView->m_pState->m_pCommandQueue->ProcessCommand(
+			m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 				new CRenameCommand(m_pRemoteDataObject->GetServerPath(), info.name, target, info.name)
 				);
 		}
 
 		// Refresh remote listing
-		m_pRemoteListView->m_pState->m_pCommandQueue->ProcessCommand(
+		m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 			new CListCommand()
 			);
 
@@ -253,7 +256,7 @@ public:
 			return wxDragNone;
 		}
 
-		const CServer* const pServer = m_pRemoteListView->m_pState->GetServer();
+		const CServer* const pServer = m_pRemoteListView->m_state.GetServer();
 		wxASSERT(pServer);
 
 		int hit = DoDisplayDropHighlight(wxPoint(x, y));
@@ -290,28 +293,40 @@ protected:
 class CInfoText : public wxWindow
 {
 public:
-	CInfoText(wxWindow* parent, const wxString& text)
-		: wxWindow(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize),
-		m_text(text)
+	CInfoText(wxWindow* parent)
+		: m_tinter(*this)
 	{
+		Hide();
+		Create(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize);
+
 		SetForegroundColour(parent->GetForegroundColour());
 		SetBackgroundColour(parent->GetBackgroundColour());
 		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
+
+#ifdef __WXMSW__
+		if (GetLayoutDirection() != wxLayout_RightToLeft)
+			SetDoubleBuffered(true);
+#endif
 	}
 
-	void SetText(const wxString &text)
+	void SetText(wxString const& text)
 	{
-		if (text == m_text)
+		if (text == m_text) {
 			return;
+		}
 
 		m_text = text;
-
 		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
 	}
 
 	wxSize GetTextSize() const { return m_textSize; }
 
 	bool AcceptsFocus() const { return false; }
+
+	void SetBackgroundTint(wxColour const& colour) {
+		m_tinter.SetBackgroundTint(colour);
+	}
+
 protected:
 	wxString m_text;
 
@@ -327,6 +342,8 @@ protected:
 
 	wxSize m_textSize;
 
+	CWindowTinter m_tinter;
+
 	DECLARE_EVENT_TABLE()
 };
 
@@ -352,20 +369,23 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_EDIT"), CRemoteListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CRemoteListView::OnMenuEnter)
 	EVT_MENU(XRCID("ID_GETURL"), CRemoteListView::OnMenuGeturl)
+	EVT_MENU(XRCID("ID_GETURL_PASSWORD"), CRemoteListView::OnMenuGeturl)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
-	: CFileListCtrl<CGenericFileData>(pParent, pState, pQueue),
-	CStateEventHandler(pState)
+CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue)
+	: CFileListCtrl<CGenericFileData>(pParent, pQueue),
+	CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
-	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	m_dropTarget = -1;
 
-	m_pInfoText = 0;
+	m_pInfoText = new CInfoText(this);
+
 	m_pDirectoryListing = 0;
 
 	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
@@ -378,18 +398,20 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	AddColumn(_("Owner/Group"), wxLIST_FORMAT_LEFT, widths[5]);
 	LoadColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
-	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
-
 	m_dirIcon = GetIconIndex(iconType::dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
 	InitHeaderSortImageList();
 
+	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
+
 	SetDirectoryListing(0);
 
 	SetDropTarget(new CRemoteListViewDropTarget(this));
 
 	EnablePrefixSearch(true);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
 }
 
 CRemoteListView::~CRemoteListView()
@@ -446,14 +468,20 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 
 	m_indexMapping[0] = pDirectoryListing->GetCount();
 
-	std::list<unsigned int> added;
-
 	CFilterManager filter;
 	const wxString path = m_pDirectoryListing->path.GetPath();
 
 	CGenericFileData last = m_fileData.back();
 	m_fileData.pop_back();
 
+	bool const has_selections = GetSelectedItemCount() != 0;
+
+	std::vector<int> added_indexes;
+	if (has_selections) {
+		added_indexes.reserve(to_add);
+	}
+
+	CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = GetSortComparisonObject();
 	for (unsigned int i = pDirectoryListing->GetCount() - to_add; i < pDirectoryListing->GetCount(); ++i) {
 		const CDirentry& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
@@ -480,43 +508,26 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 		if (m_hasParent)
 			++start;
-		CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = GetSortComparisonObject();
 		std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), i, compare);
-		compare.Destroy();
 
-		const int item = insertPos - m_indexMapping.begin();
+		const int added_index = insertPos - m_indexMapping.begin();
 		m_indexMapping.insert(insertPos, i);
 
-		for (auto iter = added.begin(); iter != added.end(); ++iter) {
-			unsigned int &pos = *iter;
-			if (pos >= (unsigned int)item)
-				++pos;
+		// Remember inserted index
+		if (has_selections) {
+			auto const added_indexes_insert_pos = std::lower_bound(added_indexes.begin(), added_indexes.end(), added_index);
+			for (auto index = added_indexes_insert_pos; index != added_indexes.end(); ++index) {
+				++(*index);
+			}
+			added_indexes.insert(added_indexes_insert_pos, added_index);
 		}
-		added.push_back(item);
 	}
+	compare.Destroy();
 
 	m_fileData.push_back(last);
 
-	std::list<bool> selected;
-	unsigned int start;
-	added.push_back(m_indexMapping.size());
-	start = added.front();
-
 	SetItemCount(m_indexMapping.size());
-
-	for (unsigned int i = start; i < m_indexMapping.size(); ++i) {
-		if (i == added.front()) {
-			selected.push_front(false);
-			added.pop_front();
-		}
-		bool is_selected = GetItemState(i, wxLIST_STATE_SELECTED) != 0;
-		selected.push_back(is_selected);
-
-		bool should_selected = selected.front();
-		selected.pop_front();
-		if (is_selected != should_selected)
-			SetSelection(i, should_selected);
-	}
+	UpdateSelections_ItemsAdded(added_indexes);
 
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->GetCount() + 1 - m_indexMapping.size());
@@ -660,10 +671,8 @@ bool CRemoteListView::UpdateDirectoryListing(std::shared_ptr<CDirectoryListing>
 	if (unsure & CDirectoryListing::unsure_invalid)
 		return false;
 
-	if (!(unsure & ~(CDirectoryListing::unsure_dir_changed | CDirectoryListing::unsure_file_changed)))
-	{
-		if (m_sortColumn && m_sortColumn != 2)
-		{
+	if (!(unsure & ~(CDirectoryListing::unsure_dir_changed | CDirectoryListing::unsure_file_changed))) {
+		if (m_sortColumn != 0 && m_sortColumn != 2) {
 			// If not sorted by file or type, changing file attributes can influence
 			// sort order.
 			return false;
@@ -729,12 +738,13 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 	if (reset) {
 		ResetSearchPrefix();
 
-		if (IsComparing() && m_pDirectoryListing)
+		if (IsComparing() && m_pDirectoryListing) {
 			ExitComparisonMode();
+		}
 
 		ClearSelection();
 
-		prevFocused = m_pState->GetPreviouslyVisitedRemoteSubdir();
+		prevFocused = m_state.GetPreviouslyVisitedRemoteSubdir();
 		ensureVisible = !prevFocused.empty();
 	}
 	else {
@@ -878,7 +888,7 @@ wxString StripVMSRevision(const wxString& name)
 void CRemoteListView::OnItemActivated(wxListEvent &event)
 {
 	int const action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
-	if (!m_pState->IsRemoteIdle(action ? false : true)) {
+	if (!m_state.IsRemoteIdle(action ? false : true)) {
 		wxBell();
 		return;
 	}
@@ -928,7 +938,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		const CServer* pServer = m_pState->GetServer();
+		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
 			wxBell();
 			return;
@@ -946,10 +956,10 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 					m_pLinkResolveState.reset(new t_linkResolveState);
 					m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 					m_pLinkResolveState->link = name;
-					m_pLinkResolveState->local_path = m_pState->GetLocalDir();
+					m_pLinkResolveState->local_path = m_state.GetLocalDir();
 					m_pLinkResolveState->server = *pServer;
 				}
-				m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
+				m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 			}
 			else {
 				wxCommandEvent evt(0, action == 1 ? XRCID("ID_DOWNLOAD") : XRCID("ID_ADDTOQUEUE"));
@@ -973,7 +983,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 			const bool queue_only = action == 1;
 
-			const CLocalPath local_path = m_pState->GetLocalDir();
+			const CLocalPath local_path = m_state.GetLocalDir();
 			if (!local_path.IsWriteable()) {
 				wxBell();
 				return;
@@ -989,13 +999,13 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 		}
 	}
 	else {
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
 void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 {
-	if (!m_pState->IsRemoteIdle(true)) {
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
@@ -1021,7 +1031,7 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		const CServer* pServer = m_pState->GetServer();
+		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
 			wxBell();
 			return;
@@ -1036,13 +1046,13 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 			m_pLinkResolveState.reset(new t_linkResolveState);
 			m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 			m_pLinkResolveState->link = name;
-			m_pLinkResolveState->local_path = m_pState->GetLocalDir();
+			m_pLinkResolveState->local_path = m_state.GetLocalDir();
 			m_pLinkResolveState->server = *pServer;
 		}
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 	}
 	else {
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
@@ -1057,9 +1067,9 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-	bool const idle = m_pState->IsRemoteIdle();
-	bool const userIdle = m_pState->IsRemoteIdle(true);
-	if (!m_pState->IsRemoteConnected() || !idle) {
+	bool const idle = m_state.IsRemoteIdle();
+	bool const userIdle = m_state.IsRemoteIdle(true);
+	if (!m_state.IsRemoteConnected() || !idle) {
 		bool canEnter = false;
 		if (userIdle) {
 			int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -1070,7 +1080,6 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 						canEnter = true;
 					}
 				}
-				canEnter = true;
 			}
 		}
 		else {
@@ -1152,13 +1161,15 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 				pMenu->Enable(XRCID("ID_RENAME"), false);
 			}
 
-			if (!m_pState->GetLocalDir().IsWriteable()) {
+			if (!m_state.GetLocalDir().IsWriteable()) {
 				pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 				pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 			}
 		}
 	}
 
+	pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
+
 	PopupMenu(pMenu);
 	delete pMenu;
 }
@@ -1166,9 +1177,9 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 {
 	// Make sure selection is valid
-	bool idle = m_pState->IsRemoteIdle();
+	bool idle = m_state.IsRemoteIdle();
 
-	const CLocalPath localDir = m_pState->GetLocalDir();
+	const CLocalPath localDir = m_state.GetLocalDir();
 	if (!localDir.IsWriteable()) {
 		wxBell();
 		return;
@@ -1197,16 +1208,16 @@ void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 	TransferSelectedFiles(localDir, event.GetId() == XRCID("ID_ADDTOQUEUE"));
 }
 
-void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly)
+void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool queue_only)
 {
-	bool idle = m_pState->IsRemoteIdle();
+	bool idle = m_state.IsRemoteIdle();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
 
 	wxASSERT(local_parent.IsWriteable());
 
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
@@ -1246,22 +1257,19 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 			wxString localFile = CQueueView::ReplaceInvalidCharacters(name);
 			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 				localFile = StripVMSRevision(localFile);
-			m_pQueue->QueueFile(queueOnly, true,
+			m_pQueue->QueueFile(queue_only, true,
 				name, (name == localFile) ? wxString() : localFile,
 				local_parent, m_pDirectoryListing->path, *pServer, entry.size);
 			added = true;
 		}
 	}
 	if (added)
-		m_pQueue->QueueFile_Finish(!queueOnly);
+		m_pQueue->QueueFile_Finish(!queue_only);
 
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
-		if (IsComparing())
-			ExitComparisonMode();
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download,
-													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), m_pDirectoryListing->path, !queue_only);
 	}
 }
 
@@ -1276,7 +1284,7 @@ void CRemoteListView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
 	CServerPath newdir = MenuMkdir();
 	if (!newdir.empty()) {
-		m_pState->ChangeRemoteDir(newdir, wxString(), 0, true);
+		m_state.ChangeRemoteDir(newdir, wxString(), 0, true);
 	}
 }
 
@@ -1284,7 +1292,7 @@ void CRemoteListView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 CServerPath CRemoteListView::MenuMkdir()
 {
-	if (!m_pDirectoryListing || !m_pState->IsRemoteIdle()) {
+	if (!m_pDirectoryListing || !m_state.IsRemoteIdle()) {
 		wxBell();
 		return CServerPath();
 	}
@@ -1315,7 +1323,7 @@ CServerPath CRemoteListView::MenuMkdir()
 		return CServerPath();
 
 	if (!m_pDirectoryListing || oldPath != m_pDirectoryListing->path ||
-		!m_pState->IsRemoteIdle())
+		!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return CServerPath();
@@ -1327,7 +1335,7 @@ CServerPath CRemoteListView::MenuMkdir()
 		return CServerPath();
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CMkdirCommand(path));
+	m_state.m_pCommandQueue->ProcessCommand(new CMkdirCommand(path));
 
 	// Return name of the New Directory
 	return path;
@@ -1335,7 +1343,7 @@ CServerPath CRemoteListView::MenuMkdir()
 
 void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1347,85 +1355,94 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (!item)
+		if (!item) {
 			continue;
-		if (item == -1)
+		}
+		if (item == -1) {
 			break;
+		}
 
-		if (!IsItemValid(item))
-		{
+		if (!IsItemValid(item)) {
 			wxBell();
 			return;
 		}
 
 		int index = GetItemIndex(item);
-		if (index == -1)
+		if (index == -1) {
 			continue;
-		if (m_fileData[index].comparison_flags == fill)
+		}
+		if (m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.is_dir())
-		{
+		if (entry.is_dir()) {
 			count_dirs++;
-			if (entry.is_link())
+			if (entry.is_link()) {
 				selected_link = true;
+			}
 		}
-		else
+		else {
 			count_files++;
+		}
 	}
 
 	wxString question;
-	if (!count_dirs)
+	if (!count_dirs) {
 		question.Printf(wxPLURAL("Really delete %d file from the server?", "Really delete %d files from the server?", count_files), count_files);
-	else if (!count_files)
+	}
+	else if (!count_files) {
 		question.Printf(wxPLURAL("Really delete %d directory with its contents from the server?", "Really delete %d directories with their contents from the server?", count_dirs), count_dirs);
-	else
-	{
+	}
+	else {
 		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
 		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
 		question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
 	}
 
-	if (wxMessageBoxEx(question, _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+	if (wxMessageBoxEx(question, _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES) {
 		return;
+	}
 
 	bool follow_symlink = false;
-	if (selected_link)
-	{
+	if (selected_link) {
 		wxDialogEx dlg;
-		if (!dlg.Load(this, _T("ID_DELETE_SYMLINK")))
-		{
+		if (!dlg.Load(this, _T("ID_DELETE_SYMLINK"))) {
 			wxBell();
 			return;
 		}
-		if (dlg.ShowModal() != wxID_OK)
+		if (dlg.ShowModal() != wxID_OK) {
 			return;
+		}
 
 		follow_symlink = XRCCTRL(dlg, "ID_RECURSE", wxRadioButton)->GetValue();
 	}
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
 
-	std::deque<wxString> filesToDelete;
+	std::deque<std::wstring> filesToDelete;
 
 	recursion_root root(m_pDirectoryListing->path, false);
 	for (item = -1; ;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (!item)
+		if (!item) {
 			continue;
-		if (item == -1)
+		}
+		if (item == -1) {
 			break;
+		}
 
 		int index = GetItemIndex(item);
-		if (index == -1)
+		if (index == -1) {
 			continue;
-		if (m_fileData[index].comparison_flags == fill)
+		}
+		if (m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		const wxString& name = entry.name;
+		std::wstring const& name = entry.name;
 
 		if (entry.is_dir() && (follow_symlink || !entry.is_link())) {
 			CServerPath remotePath = m_pDirectoryListing->path;
@@ -1433,21 +1450,21 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);;
 			}
 		}
-		else
+		else {
 			filesToDelete.push_back(name);
+		}
 	}
 
-	if (!filesToDelete.empty())
-		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_pDirectoryListing->path, std::move(filesToDelete)));
+	if (!filesToDelete.empty()) {
+		m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_pDirectoryListing->path, std::move(filesToDelete)));
+	}
 
 	if (!root.empty()) {
-		if (IsComparing())
-			ExitComparisonMode();
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete,
-													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
+													 filter.GetActiveFilters(), m_pDirectoryListing->path);
 	}
 }
 
@@ -1458,7 +1475,7 @@ void CRemoteListView::OnMenuRename(wxCommandEvent&)
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1524,7 +1541,7 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 
 bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return false;
@@ -1549,7 +1566,7 @@ bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 
 bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return false;
@@ -1601,14 +1618,14 @@ bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 		}
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(m_pDirectoryListing->path, entry.name, newPath, newFile));
+	m_state.m_pCommandQueue->ProcessCommand(new CRenameCommand(m_pDirectoryListing->path, entry.name, newPath, newFile));
 
 	return true;
 }
 
 void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1673,7 +1690,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 	}
 
 	// State may have changed while chmod dialog was shown
-	if (!m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		pChmodDlg->Destroy();
 		wxBell();
 		return;
@@ -1681,7 +1698,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 
 	const int applyType = pChmodDlg->GetApplyType();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
 	recursion_root root(m_pDirectoryListing->path, false);
 
@@ -1716,7 +1733,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 			bool res = pChmodDlg->ConvertPermissions(*entry.permissions, newPermissions);
 			wxString newPerms = pChmodDlg->GetPermissions(res ? newPermissions : 0, entry.is_dir());
 
-			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
+			m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
 		}
 
 		if (pChmodDlg->Recursive() && entry.is_dir())
@@ -1724,23 +1741,20 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 	}
 
 	if (pChmodDlg->Recursive()) {
-		if (IsComparing())
-			ExitComparisonMode();
-
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod,
-													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
+													 filter.GetActiveFilters(), m_pDirectoryListing->path);
 
 		// Refresh listing. This gets done implicitely by the recursive operation, so
 		// only it if not recursing.
 		if (pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_chmod)
-			m_pState->ChangeRemoteDir(m_pDirectoryListing->path);
+			m_state.ChangeRemoteDir(m_pDirectoryListing->path);
 	}
 	else {
 		pChmodDlg->Destroy();
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(""), 0, true);
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(""), 0, true);
 	}
 
 }
@@ -1749,8 +1763,9 @@ void CRemoteListView::ApplyCurrentFilter()
 {
 	CFilterManager filter;
 
-	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing())
+	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
+	}
 
 	if (m_fileData.size() <= 1)
 		return;
@@ -1956,17 +1971,17 @@ void CRemoteListView::OnSize(wxSizeEvent& event)
 
 void CRemoteListView::RepositionInfoText()
 {
-	if (!m_pInfoText)
+	if (!m_pInfoText) {
 		return;
+	}
 
 	wxRect rect = GetClientRect();
-
 	wxSize size = m_pInfoText->GetTextSize();
 
-	if (m_indexMapping.empty())
+	if (!GetItemCount()) {
 		rect.y = 60;
-	else
-	{
+	}
+	else {
 		wxRect itemRect;
 		GetItemRect(0, itemRect);
 		rect.y = wxMax(60, itemRect.GetBottom() + 1);
@@ -1977,8 +1992,7 @@ void CRemoteListView::RepositionInfoText()
 
 	m_pInfoText->SetSize(rect);
 #ifdef __WXMSW__
-	if (GetLayoutDirection() != wxLayout_RightToLeft)
-	{
+	if (GetLayoutDirection() != wxLayout_RightToLeft) {
 		m_pInfoText->Refresh(true);
 		m_pInfoText->Update();
 	}
@@ -1988,15 +2002,18 @@ void CRemoteListView::RepositionInfoText()
 
 }
 
-void CRemoteListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CRemoteListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2)
 {
-	wxASSERT(pState);
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(pState->GetRemoteDir());
+		SetDirectoryListing(m_state.GetRemoteDir());
 	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR) {
 		wxASSERT(data2);
 		LinkIsNotDir(*(CServerPath*)data2, data);
 	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		ApplyCurrentFilter();
@@ -2015,30 +2032,24 @@ void CRemoteListView::SetInfoText()
 			text = _("Empty directory listing");
 	}
 
+
 	if (text.empty()) {
-		delete m_pInfoText;
-		m_pInfoText = 0;
-		return;
+		m_pInfoText->Hide();
 	}
-
-	if (!m_pInfoText) {
-		m_pInfoText = new CInfoText(this, text);
-#ifdef __WXMSW__
-		if (GetLayoutDirection() != wxLayout_RightToLeft)
-			m_pInfoText->SetDoubleBuffered(true);
-#endif
-
+	else {
+		m_pInfoText->SetText(text);
 		RepositionInfoText();
-		return;
+		m_pInfoText->Show();
 	}
-
-	m_pInfoText->SetText(text);
-	RepositionInfoText();
 }
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -2048,7 +2059,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 		return;
 	}
 
-	bool idle = m_pState->m_pCommandQueue->Idle();
+	bool idle = m_state.m_pCommandQueue->Idle();
 
 	long item = -1;
 	int count = 0;
@@ -2080,7 +2091,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 
 	wxDataObjectComposite object;
 
-	CServer const* pServer = m_pState->GetServer();
+	CServer const* pServer = m_state.GetServer();
 	if (!pServer)
 		return;
 	CServer const server = *pServer;
@@ -2142,8 +2153,8 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			pServer = m_pState->GetServer();
-			if (!m_pState->IsRemoteIdle() ||
+			pServer = m_state.GetServer();
+			if (!m_state.IsRemoteIdle() ||
 				!pServer || *pServer != server ||
 				!m_pDirectoryListing || m_pDirectoryListing->path != path)
 			{
@@ -2153,7 +2164,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 			}
 
 			// Same checks as before
-			idle = m_pState->m_pCommandQueue->Idle();
+			idle = m_state.m_pCommandQueue->Idle();
 
 			item = -1;
 			for (;;) {
@@ -2192,7 +2203,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 
 void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteConnected() || !m_pDirectoryListing) {
+	if (!m_state.IsRemoteConnected() || !m_pDirectoryListing) {
 		wxBell();
 		return;
 	}
@@ -2226,7 +2237,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 	}
 
 	const CServerPath path = m_pDirectoryListing->path;
-	const CServer server = *m_pState->GetServer();
+	const CServer server = *m_state.GetServer();
 	pEditHandler->Edit(CEditHandler::remote, selected_items, path, server, this);
 }
 
@@ -2269,8 +2280,7 @@ bool CRemoteListView::CanStartComparison()
 
 void CRemoteListView::StartComparison()
 {
-	if (m_sortDirection || m_sortColumn)
-	{
+	if (m_sortDirection || m_sortColumn != 0) {
 		wxASSERT(m_originalIndexMapping.empty());
 		SortList(0, 0);
 	}
@@ -2395,9 +2405,9 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		{
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
 			if (m_pDirectoryListing->path.GetType() == VMS)
-				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
+				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir()).ToStdWstring();
 			else
-				data.fileType = GetType(entry.name, entry.is_dir());
+				data.fileType = GetType(entry.name, entry.is_dir()).ToStdWstring();
 		}
 
 		return data.fileType;
@@ -2420,8 +2430,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 	CFileListCtrlSort<CDirectoryListing>::NameSortMode nameSortMode = GetNameSortMode();
 
 	CDirectoryListing const& directoryListing = *m_pDirectoryListing;
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -2435,8 +2444,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 		else
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortName<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -2483,22 +2491,22 @@ void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link
 	m_pLinkResolveState.reset();
 }
 
-void CRemoteListView::OnMenuGeturl(wxCommandEvent&)
+void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 {
-	if (!m_pDirectoryListing)
+	if (!m_pDirectoryListing) {
 		return;
+	}
 
-	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
+	const CServer* pServer = m_state.GetServer();
+	if (!pServer) {
 		return;
+	}
 
 	long item = -1;
 
 	std::list<CDirentry> selected_item_list;
-	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
-		if (!item)
-		{
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
+		if (!item) {
 			wxBell();
 			return;
 		}
@@ -2511,49 +2519,47 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent&)
 
 		selected_item_list.push_back(entry);
 	}
-	if (selected_item_list.empty())
-	{
+	if (selected_item_list.empty()) {
 		wxBell();
 		return;
 	}
 
-	if (!wxTheClipboard->Open())
-	{
+	if (!wxTheClipboard->Open()) {
 		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
 	const CServerPath& path = m_pDirectoryListing->path;
-	const wxString server = pServer->FormatServer(true);
-	if (selected_item_list.size() == 1)
-	{
-		wxString url = server;
-		url += path.FormatFilename(selected_item_list.front().name, false);
 
-		// Poor mans URLencode
-		url.Replace(_T(" "), _T("%20"));
+	wxString const server = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-		wxTheClipboard->SetData(new wxURLDataObject(url));
+	auto getUrl = [](wxString const& serverPart, CServerPath const& path, wxString const& name) {
+		wxString url = serverPart;
+
+		auto const pathPart = url_encode(path.FormatFilename(name, false).ToStdWstring(), true);
+		if (!pathPart.empty() && pathPart[0] != '/') {
+			url += '/';
+		}
+		url += pathPart;
+
+		return url;
+	};
+
+	wxString urls;
+	if (selected_item_list.size() == 1) {
+		urls = getUrl(server, path, selected_item_list.front().name);
 	}
-	else
-	{
-		wxString urls;
-		for (std::list<CDirentry>::const_iterator iter = selected_item_list.begin(); iter != selected_item_list.end(); ++iter)
-		{
-			urls += server;
-			urls += path.FormatFilename(iter->name, false);
+	else {
+		for (auto const& entry : selected_item_list) {
+			urls += getUrl(server, path, entry.name);
 #ifdef __WXMSW__
 			urls += _T("\r\n");
 #else
 			urls += _T("\n");
 #endif
 		}
-
-		// Poor mans URLencode
-		urls.Replace(_T(" "), _T("%20"));
-
-		wxTheClipboard->SetData(new wxTextDataObject(urls));
 	}
+	wxTheClipboard->SetData(new wxURLDataObject(urls));
 
 	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
@@ -2577,14 +2583,13 @@ int CRemoteListView::GetOverlayIndex(int item)
 
 void CRemoteListView::OnMenuRefresh(wxCommandEvent&)
 {
-	if (m_pState)
-		m_pState->RefreshRemote();
+	m_state.RefreshRemote();
 }
 
 void CRemoteListView::OnNavigationEvent(bool forward)
 {
 	if (!forward) {
-		if (!m_pState->IsRemoteIdle(true)) {
+		if (!m_state.IsRemoteIdle(true)) {
 			wxBell();
 			return;
 		}
@@ -2594,13 +2599,13 @@ void CRemoteListView::OnNavigationEvent(bool forward)
 			return;
 		}
 
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
 void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle() || !m_pDirectoryListing) {
+	if (!m_state.IsRemoteIdle() || !m_pDirectoryListing) {
 		wxBell();
 		return;
 	}
@@ -2639,12 +2644,12 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		file.Create(emptyfile);
 	}
 
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
 	}
 
 	CFileTransferCommand *cmd = new CFileTransferCommand(emptyfile, m_pDirectoryListing->path, newFileName, false, CFileTransferCommand::t_transferSettings());
-	m_pState->m_pCommandQueue->ProcessCommand(cmd);
+	m_state.m_pCommandQueue->ProcessCommand(cmd);
 }
