@@ -1,18 +1,20 @@
 #include <filezilla.h>
-#include "RemoteTreeView.h"
-#include "commandqueue.h"
-#include <wx/dnd.h>
-#include "dndobjects.h"
+
 #include "chmoddialog.h"
-#include "recursive_operation.h"
-#include "inputdialog.h"
+#include "commandqueue.h"
+#include "dndobjects.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include <wx/clipbrd.h>
+#include "graphics.h"
+#include "inputdialog.h"
+#include "Options.h"
 #include "queue.h"
 #include "QueueView.h"
+#include "RemoteTreeView.h"
+#include "remote_recursive_operation.h"
 #include "themeprovider.h"
-#include "Options.h"
+
+#include <wx/clipbrd.h>
 
 #include <algorithm>
 
@@ -81,7 +83,7 @@ public:
 			pDragDropManager->pDropTarget = m_pRemoteTreeView;
 
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pRemoteTreeView->m_pState->UploadDroppedFiles(m_pFileDataObject, path, false);
+			m_pRemoteTreeView->m_state.UploadDroppedFiles(m_pFileDataObject, path, false);
 		else
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
@@ -90,7 +92,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_pState->GetServer()))
+			if (!m_pRemoteTreeView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_state.GetServer()))
 			{
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
@@ -124,7 +126,7 @@ public:
 			for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 			{
 				const CRemoteDataObject::t_fileInfo& info = *iter;
-				m_pRemoteTreeView->m_pState->m_pCommandQueue->ProcessCommand(
+				m_pRemoteTreeView->m_state.m_pCommandQueue->ProcessCommand(
 					new CRenameCommand(m_pRemoteDataObject->GetServerPath(), info.name, path, info.name)
 					);
 			}
@@ -240,17 +242,18 @@ EVT_CHAR(CRemoteTreeView::OnChar)
 EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
 
-CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
+CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView* pQueue)
 	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
 	CSystemImageList(16),
-	CStateEventHandler(pState)
+	CStateEventHandler(state)
 {
 #ifdef __WXMAC__
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
 
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
-	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	CreateImageList();
 
@@ -265,6 +268,8 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 	SetDropTarget(new CRemoteTreeViewDropTarget(this));
 
 	Enable(false);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*this);
 }
 
 CRemoteTreeView::~CRemoteTreeView()
@@ -273,12 +278,17 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* v)
+void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
 {
-	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(pState->GetRemoteDir(), v ? *reinterpret_cast<bool const*>(v) : false);
-	else if (notification == STATECHANGE_APPLYFILTER)
+	if (notification == STATECHANGE_REMOTE_DIR) {
+		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : false);
+	}
+	else if (notification == STATECHANGE_APPLYFILTER) {
 		ApplyFilters(false);
+	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 }
 
 void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
@@ -400,7 +410,7 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		if (!child) {
 			CDirectoryListing listing;
 
-			if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
+			if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
 				child = AppendItem(parent, *iter, 0, 2, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, false);
 			}
@@ -532,7 +542,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 		CDirectoryListing subListing;
 
-		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
+		if (m_state.m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
@@ -585,7 +595,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				if (!GetLastChild(child) && HasSubdirs(subListing, filter))
 					AppendItem(child, _T(""), -1, -1);
 				SetItemImages(child, false);
@@ -612,7 +622,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
 				if (childItem) {
 					SetItemImages(childItem, false);
@@ -648,7 +658,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		childPath.AddSegment(*iter);
 
 		CDirectoryListing subListing;
-		if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+		if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
 			if (childItem) {
 				SetItemImages(childItem, false);
@@ -721,7 +731,7 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	if (m_busy)
 		return;
 
-	if (!m_pState->IsRemoteIdle(true)) {
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
@@ -735,7 +745,7 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	if (path.empty())
 		return;
 
-	m_pState->ChangeRemoteDir(path);
+	m_state.ChangeRemoteDir(path);
 }
 
 void CRemoteTreeView::OnItemActivated(wxTreeEvent& event)
@@ -772,8 +782,12 @@ CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 
 void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	// Drag could result in recursive operation, don't allow at this point
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -793,7 +807,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	wxDataObjectComposite object;
 
-	CServer const* pServer = m_pState->GetServer();
+	CServer const* pServer = m_state.GetServer();
 	if (!pServer)
 		return;
 	CServer const server = *pServer;
@@ -839,8 +853,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			pServer = m_pState->GetServer();
-			if (!pServer || !m_pState->IsRemoteIdle() || *pServer != server) {
+			pServer = m_state.GetServer();
+			if (!pServer || !m_state.IsRemoteIdle() || *pServer != server) {
 				wxBell();
 				return;
 			}
@@ -852,7 +866,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 				return;
 			}
 
-			m_pState->DownloadDroppedFiles(pRemoteDataObject, target);
+			m_state.DownloadDroppedFiles(pRemoteDataObject, target);
 		}
 	}
 #endif
@@ -866,7 +880,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
-	if (!m_pState->IsRemoteIdle() || path.empty()) {
+	if (!m_state.IsRemoteIdle() || path.empty()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
@@ -879,18 +893,20 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	else if (!path.HasParent())
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 
-	if (!m_pState->GetLocalDir().IsWriteable()) {
+	if (!m_state.GetLocalDir().IsWriteable()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
 
+	pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
+
 	PopupMenu(pMenu);
 	delete pMenu;
 }
 
 void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 		return;
 
 	if (!m_contextMenuItem)
@@ -915,7 +931,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		CDirectoryListing listing;
 
 		// ... and it needs to be cached
-		cached = m_pState->m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
+		cached = m_state.m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
 		if (cached) {
 			for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 				if (listing[i].name != name)
@@ -939,7 +955,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 	}
 
 	// State may have changed while chmod dialog was shown
-	if (!m_contextMenuItem || !m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle()) {
+	if (!m_contextMenuItem || !m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
@@ -953,7 +969,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		if (!applyType || applyType == 2) {
 			wxString newPerms = pChmodDlg->GetPermissions(permissions, true);
 
-			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
+			m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
 		}
 
 		if (pChmodDlg->Recursive())
@@ -968,7 +984,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 	}
 
 	if (!cached || pChmodDlg->Recursive()) {
-		CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+		CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
 
@@ -977,27 +993,27 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		if (selected)
 			currentPath = GetPathFromItem(selected);
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters(false), currentPath);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters(), currentPath);
 	}
 	else {
 		pChmodDlg->Destroy();
 		const wxTreeItemId selected = GetSelection();
 		if (selected) {
 			CServerPath currentPath = GetPathFromItem(selected);
-			m_pState->ChangeRemoteDir(currentPath);
+			m_state.ChangeRemoteDir(currentPath);
 		}
 	}
 }
 
 void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 {
-	CLocalPath localDir = m_pState->GetLocalDir();
+	CLocalPath localDir = m_state.GetLocalDir();
 	if (!localDir.IsWriteable()) {
 		wxBell();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 		return;
 
 	if (!m_contextMenuItem)
@@ -1013,7 +1029,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	recursion_root root(path, true);
 	root.add_dir_to_visit(path, _T(""), localDir);
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	pRecursiveOperation->AddRecursionRoot(std::move(root));
 
 	CServerPath currentPath;
@@ -1023,12 +1039,12 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const bool addOnly = event.GetId() == XRCID("ID_ADDTOQUEUE");
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, filter.GetActiveFilters(false), currentPath);
+	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), currentPath, !addOnly);
 }
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 		return;
 
 	if (!m_contextMenuItem)
@@ -1043,7 +1059,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 
 	const bool hasParent = pathToDelete.HasParent();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 
 	recursion_root root;
 	CServerPath startDir;
@@ -1066,16 +1082,16 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 		currentPath = GetPathFromItem(selected);
 	if (!currentPath.empty() && (pathToDelete == currentPath || pathToDelete.IsParentOf(currentPath, false))) {
 		currentPath = startDir;
-		m_pState->ChangeRemoteDir(startDir);
+		m_state.ChangeRemoteDir(startDir);
 	}
 
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(false), currentPath);
+	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(), currentPath);
 }
 
 void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 		return;
 
 	if (!m_contextMenuItem)
@@ -1093,7 +1109,7 @@ void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 
 void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
@@ -1117,7 +1133,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
@@ -1138,8 +1154,8 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(parent, oldName, parent, newName));
-	m_pState->ChangeRemoteDir(parent);
+	m_state.m_pCommandQueue->ProcessCommand(new CRenameCommand(parent, oldName, parent, newName));
+	m_state.ChangeRemoteDir(parent);
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
@@ -1164,10 +1180,10 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		currentPath.AddSegment(newName);
 		for (std::list<wxString>::const_iterator iter = subdirs.begin(); iter != subdirs.end(); ++iter)
 			currentPath.AddSegment(*iter);
-		m_pState->ChangeRemoteDir(currentPath);
+		m_state.ChangeRemoteDir(currentPath);
 	}
 	else if (currentPath != parent)
-		m_pState->ChangeRemoteDir(currentPath);
+		m_state.ChangeRemoteDir(currentPath);
 }
 
 
@@ -1180,7 +1196,7 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent&)
 	if (newpath.HasParent())
 	{
 		listed = newpath.GetParent();
-		m_pState->ChangeRemoteDir(listed);
+		m_state.ChangeRemoteDir(listed);
 	}
 
 	CServerPath currentPath;
@@ -1188,7 +1204,7 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent&)
 	if (selected)
 		currentPath = GetPathFromItem(selected);
 	if (!currentPath.empty() && currentPath != listed)
-		m_pState->ChangeRemoteDir(currentPath);
+		m_state.ChangeRemoteDir(currentPath);
 
 }
 
@@ -1197,7 +1213,7 @@ void CRemoteTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
 	CServerPath newpath = MenuMkdir();
 	if (!newpath.empty()) {
-		m_pState->ChangeRemoteDir(newpath);
+		m_state.ChangeRemoteDir(newpath);
 	}
 }
 
@@ -1205,7 +1221,7 @@ void CRemoteTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 CServerPath CRemoteTreeView::MenuMkdir()
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 		return CServerPath();
 
 	if (!m_contextMenuItem)
@@ -1244,7 +1260,7 @@ CServerPath CRemoteTreeView::MenuMkdir()
 		return CServerPath();
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CMkdirCommand(newPath));
+	m_state.m_pCommandQueue->ProcessCommand(new CMkdirCommand(newPath));
 
 	return newPath;
 }
@@ -1257,7 +1273,7 @@ bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 		return false;
 
 	CDirectoryListing listing;
-	if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
+	if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
 		RefreshItem(item, listing, false);
 	else
 	{
@@ -1292,7 +1308,7 @@ struct _parents
 
 void CRemoteTreeView::ApplyFilters(bool resort)
 {
-	std::list<struct _parents> parents;
+	std::list<_parents> parents;
 
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemIdValue cookie;
@@ -1301,7 +1317,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 		if (path.empty())
 			continue;
 
-		struct _parents dir;
+		_parents dir;
 		dir.item = child;
 		dir.path = path;
 		parents.push_back(dir);
@@ -1309,7 +1325,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 
 	CFilterManager filter;
 	while (!parents.empty()) {
-		struct _parents parent = parents.back();
+		_parents parent = parents.back();
 		parents.pop_back();
 
 		if (resort) {
@@ -1317,7 +1333,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 		}
 
 		CDirectoryListing listing;
-		if (m_pState->m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
+		if (m_state.m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
 			RefreshItem(parent.item, listing, false);
 		else if (filter.HasActiveFilters()) {
 			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child)) {
@@ -1325,7 +1341,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 				if (path.empty())
 					continue;
 
-				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, fz::datetime())) {
+				if (filter.FilenameFiltered(GetItemText(child).ToStdWstring(), path.GetPath(), true, -1, false, 0, fz::datetime())) {
 					wxTreeItemId sel = GetSelection();
 					while (sel && sel != child)
 						sel = GetItemParent(sel);
@@ -1335,7 +1351,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 					}
 				}
 
-				struct _parents dir;
+				_parents dir;
 				dir.item = child;
 				dir.path = path;
 				parents.push_back(dir);
@@ -1349,7 +1365,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 			if (path.empty())
 				continue;
 
-			struct _parents dir;
+			_parents dir;
 			dir.item = child;
 			dir.path = path;
 			parents.push_back(dir);
@@ -1357,36 +1373,35 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 	}
 }
 
-void CRemoteTreeView::OnMenuGeturl(wxCommandEvent&)
+void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 {
 	if (!m_contextMenuItem)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.empty())
-	{
+	if (path.empty()) {
 		wxBell();
 		return;
 	}
 
-	const CServer *pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	const CServer *pServer = m_state.GetServer();
+	if (!pServer) {
 		wxBell();
 		return;
 	}
 
-	if (!wxTheClipboard->Open())
-	{
+	if (!wxTheClipboard->Open()) {
 		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString url = pServer->FormatServer(true);
-	url += path.GetPath();
+	wxString url = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-	// Poor mans URLencode
-	url.Replace(_T(" "), _T("%20"));
+	wxString const pathPart = url_encode(path.GetPath().ToStdWstring(), true);
+	if (!pathPart.empty() && pathPart[0] != '/') {
+		url += '/';
+	}
+	url += pathPart;
 
 	wxTheClipboard->SetData(new wxURLDataObject(url));
 
