@@ -232,7 +232,7 @@ void CEditHandler::Release()
 	delete this;
 }
 
-enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName) const
+CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
@@ -241,7 +241,7 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 	return iter->state;
 }
 
-enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
+CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
@@ -250,7 +250,7 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 	return iter->state;
 }
 
-int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandler::fileState state, const CServer* pServer /*=0*/) const
+int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, const CServer* pServer /*=0*/) const
 {
 	int count = 0;
 	if (state == unknown) {
@@ -283,7 +283,7 @@ int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandl
 	return count;
 }
 
-bool CEditHandler::AddFile(enum CEditHandler::fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server)
+bool CEditHandler::AddFile(CEditHandler::fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
 	wxASSERT(type != none);
 
@@ -395,7 +395,7 @@ bool CEditHandler::RemoveAll(bool force)
 	return m_fileDataList[local].empty() && m_fileDataList[remote].empty();
 }
 
-bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/)
+bool CEditHandler::RemoveAll(fileState state, const CServer* pServer /*=0*/)
 {
 	// Others not implemented
 	wxASSERT(state == upload_and_remove_failed);
@@ -580,7 +580,7 @@ bool CEditHandler::StartEditing(const wxString& file, const CServerPath& remoteP
 	return StartEditing(remote, *iter);
 }
 
-bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& data)
+bool CEditHandler::StartEditing(CEditHandler::fileType type, t_fileData& data)
 {
 	wxASSERT(type != none);
 	wxASSERT(data.state == edit);
@@ -628,10 +628,10 @@ checkmodifications_loopbegin:
 				goto checkmodifications_loopbegin;
 			}
 
-			if (!mtime.empty())
+			if (mtime.empty())
 				continue;
 
-			if (iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
+			if (!iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
@@ -716,7 +716,8 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 		else
 			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
 	}
-	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
+
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.Format(ServerFormat::with_user_and_optional_port));
 	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
@@ -740,7 +741,7 @@ bool CEditHandler::UploadFile(const wxString& file, bool unedit)
 	return UploadFile(local, iter, unedit);
 }
 
-bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterator iter, bool unedit)
+bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
 	wxCHECK(type != none, false);
 
@@ -762,7 +763,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 		return false;
 	}
 
-	if (!mtime.empty())
+	if (mtime.empty())
 		mtime = fz::datetime::now();
 
 	iter->modificationTime = mtime;
@@ -808,7 +809,7 @@ void CEditHandler::SetTimerState()
 		m_timer.Start(15000);
 }
 
-wxString CEditHandler::CanOpen(enum CEditHandler::fileType type, const wxString& fileName, bool &dangerous, bool &program_exists)
+wxString CEditHandler::CanOpen(CEditHandler::fileType type, const wxString& fileName, bool &dangerous, bool &program_exists)
 {
 	wxASSERT(type != none);
 
@@ -1117,7 +1118,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.FormatServer() + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1145,7 +1146,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.FormatServer() + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1184,7 +1185,7 @@ void CEditHandlerStatusDialog::SetCtrlState()
 	int item = -1;
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 		if (pData->state == CEditHandler::edit)
 			selectedEdited = true;
@@ -1218,7 +1219,7 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 	int item = -1;
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
@@ -1231,7 +1232,7 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		if (type == CEditHandler::local) {
@@ -1265,7 +1266,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
 		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
@@ -1278,7 +1279,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		bool unedit = event.GetId() == XRCID("ID_UPLOADANDUNEDIT") || pData->state == CEditHandler::upload_and_remove_failed;
@@ -1312,7 +1313,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
 		if (pData->state != CEditHandler::edit) {
@@ -1325,7 +1326,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		if (type == CEditHandler::local) {
@@ -1442,7 +1443,7 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 void CNewAssociationDialog::SetCtrlState()
 {
-	wxRadioButton* pCustom = wxDynamicCast(FindWindow(XRCID("ID_USE_CUSTOM")), wxRadioButton);
+	wxRadioButton* pCustom = dynamic_cast<wxRadioButton*>(FindWindow(XRCID("ID_USE_CUSTOM")));
 	if (!pCustom) {
 		// Return since it can get called before dialog got fully loaded
 		return;
