@@ -10,7 +10,6 @@
 
 class CQueueView;
 class CFileListCtrl_SortComparisonObject;
-class CState;
 class CFilelistStatusBar;
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
 class CGtkEventCallbackProxyBase;
@@ -19,7 +18,7 @@ class CGtkEventCallbackProxyBase;
 class CGenericFileData
 {
 public:
-	wxString fileType;
+	std::wstring fileType;
 	int icon{-2};
 
 	// t_fileEntryFlags is defined in listingcomparison.h as it will be used for
@@ -186,7 +185,7 @@ public:
 	typedef Listing List;
 	typedef typename Listing::value_type value_type;
 
-	CFileListCtrlSort(Listing const& listing, enum DirSortMode dirSortMode, enum NameSortMode nameSortMode)
+	CFileListCtrlSort(Listing const& listing, DirSortMode dirSortMode, NameSortMode nameSortMode)
 		: m_listing(listing), m_dirSortMode(dirSortMode), m_nameSortMode(nameSortMode)
 	{
 	}
@@ -346,9 +345,9 @@ public:
 		DataEntry &type1 = m_fileData[a];
 		DataEntry &type2 = m_fileData[b];
 		if (type1.fileType.empty())
-			type1.fileType = m_pListView->GetType(data1.name, data1.is_dir());
+			type1.fileType = m_pListView->GetType(data1.name, data1.is_dir()).ToStdWstring();
 		if (type2.fileType.empty())
-			type2.fileType = m_pListView->GetType(data2.name, data2.is_dir());
+			type2.fileType = m_pListView->GetType(data2.name, data2.is_dir()).ToStdWstring();
 
 		CMP(CmpStringNoCase, type1.fileType, type2.fileType);
 
@@ -441,12 +440,12 @@ public:
 		typename Listing::value_type const& data1 = this->m_listing[a];
 		typename Listing::value_type const& data2 = this->m_listing[b];
 
-		if (this->m_listing[a].path < m_fileData[b].path)
+		if (data1.path < data2.path)
 			return true;
-		if (this->m_listing[a].path != m_fileData[b].path)
+		if (data1.path != data2.path)
 			return false;
 
-		CMP_LESS(CmpName, static_cast<CDirentry const&>(data1), static_cast<CDirentry const&>(data2));
+		CMP_LESS(CmpName, data1, data2);
 	}
 	std::vector<DataEntry>& m_fileData;
 };
@@ -469,9 +468,9 @@ public:
 		CMP(CmpDir, data1, data2);
 		CMP(CmpName, data1, data2);
 
-		if (this->m_listing[a].path < m_fileData[b].path)
+		if (data1.path < data2.path)
 			return true;
-		if (this->m_listing[a].path != m_fileData[b].path)
+		if (data1.path != data2.path)
 			return false;
 
 		CMP_LESS(CmpName, data1, data2);
@@ -490,7 +489,7 @@ template<class CFileData> class CFileListCtrl : public wxListCtrlEx, public CCom
 {
 	template<typename Listing, typename DataEntry> friend class CFileListCtrlSortType;
 public:
-	CFileListCtrl(wxWindow* pParent, CState *pState, CQueueView *pQueue, bool border = false);
+	CFileListCtrl(wxWindow* pParent, CQueueView *pQueue, bool border = false);
 	virtual ~CFileListCtrl();
 
 	class CSortComparisonObject : public std::binary_function<int,int,bool>
@@ -521,6 +520,8 @@ public:
 
 	virtual void OnNavigationEvent(bool) {}
 
+	std::vector<unsigned int> const& indexMapping() const { return m_indexMapping; }
+
 protected:
 	CQueueView *m_pQueue;
 
@@ -536,13 +537,13 @@ protected:
 	// The .. item
 	bool m_hasParent;
 
-	int m_sortColumn;
-	int m_sortDirection;
+	int m_sortColumn{-1};
+	int m_sortDirection{};
 
 	void InitSort(int optionID); // Has to be called after initializing columns
 	void SortList(int column = -1, int direction = -1, bool updateSelections = true);
-	enum CFileListCtrlSortBase::DirSortMode GetDirSortMode();
-	enum CFileListCtrlSortBase::NameSortMode GetNameSortMode();
+	CFileListCtrlSortBase::DirSortMode GetDirSortMode();
+	CFileListCtrlSortBase::NameSortMode GetNameSortMode();
 	virtual CSortComparisonObject GetSortComparisonObject() = 0;
 
 	// An empty path denotes a virtual file
@@ -564,6 +565,9 @@ protected:
 
 	CFilelistStatusBar* m_pFilelistStatusBar;
 
+	// Indexes of the items added, sorted ascending.
+	void UpdateSelections_ItemsAdded(std::vector<int> const& added_indexes);
+
 #ifndef __WXMSW__
 	// Generic wxListCtrl does not support wxLIST_STATE_DROPHILITED, emulate it
 	wxListItemAttr m_dropHighlightAttribute;
@@ -580,6 +584,8 @@ protected:
 #endif
 
 private:
+	void UpdateSelections(int min, int max);
+
 	void SortList_UpdateSelections(bool* selections, int focused_item, unsigned int focused_index);
 
 	// If this is set to true, don't process selection changed events
