@@ -11,8 +11,6 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
-#include <wx/regex.h>
-
 bool CFilterManager::m_loaded = false;
 std::vector<CFilter> CFilterManager::m_globalFilters;
 std::vector<CFilterSet> CFilterManager::m_globalFilterSets;
@@ -44,21 +42,7 @@ CFilterCondition::CFilterCondition()
 	value = 0;
 }
 
-CFilter::CFilter()
-{
-	matchType = all;
-	filterDirs = true;
-	filterFiles = true;
-
-	// Filenames on Windows ignore case
-#ifdef __WXMSW__
-	matchCase = false;
-#else
-	matchCase = true;
-#endif
-}
-
-bool CFilter::HasConditionOfType(enum t_filterType type) const
+bool CFilter::HasConditionOfType(t_filterType type) const
 {
 	for (std::vector<CFilterCondition>::const_iterator iter = filters.begin(); iter != filters.end(); ++iter)
 	{
@@ -552,58 +536,47 @@ bool CFilterManager::HasActiveFilters(bool ignore_disabled /*=false*/)
 
 bool CFilterManager::HasSameLocalAndRemoteFilters() const
 {
-	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
-	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-	{
-		if (set.local[i])
-		{
-			if (!set.remote[i])
-				return false;
-		}
-		else if (set.remote[i])
-			return false;
-	}
+	CFilterSet const& set = m_globalFilterSets[m_globalCurrentFilterSet];
+	return set.local == set.remote;
 
 	return true;
 }
 
-bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
+bool CFilterManager::FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
 {
-	if (m_filters_disabled)
+	if (m_filters_disabled) {
 		return false;
+	}
 
 	wxASSERT(m_globalCurrentFilterSet < m_globalFilterSets.size());
 
-	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
+	CFilterSet const& set = m_globalFilterSets[m_globalCurrentFilterSet];
+	auto const& active = local ? set.local : set.remote;
 
 	// Check active filters
 	for (unsigned int i = 0; i < m_globalFilters.size(); ++i) {
-		if (local) {
-			if (set.local[i])
-				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes, date))
-					return true;
-		}
-		else {
-			if (set.remote[i])
-				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes, date))
-					return true;
+		if (active[i]) {
+			if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes, date)) {
+				return true;
+			}
 		}
 	}
 
 	return false;
 }
 
-bool CFilterManager::FilenameFiltered(std::vector<CFilter> const& filters, const wxString& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const
+bool CFilterManager::FilenameFiltered(std::vector<CFilter> const& filters, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const
 {
 	for (auto const& filter : filters) {
-		if (FilenameFilteredByFilter(filter, name, path, dir, size, attributes, date))
+		if (FilenameFilteredByFilter(filter, name, path, dir, size, attributes, date)) {
 			return true;
+		}
 	}
 
 	return false;
 }
 
-static bool StringMatch(const wxString& subject, const wxString& filter, int condition, bool matchCase, std::shared_ptr<const wxRegEx> const& pRegEx)
+static bool StringMatch(const wxString& subject, const wxString& filter, int condition, bool matchCase, std::shared_ptr<const std::wregex> const& pRegEx)
 {
 	bool match = false;
 
@@ -657,8 +630,9 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 		break;
 	case 4:
 		wxASSERT(pRegEx);
-		if (pRegEx && pRegEx->Matches(subject))
+		if (pRegEx && std::regex_search(subject.ToStdWstring(), *pRegEx)) {
 			match = true;
+		}
 		break;
 	case 5:
 		if (matchCase) {
@@ -675,17 +649,17 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 	return match;
 }
 
-bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date)
+bool CFilterManager::FilenameFilteredByFilter(CFilter const& filter, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date)
 {
-	if (dir && !filter.filterDirs)
+	if (dir && !filter.filterDirs) {
 		return false;
-	else if (!dir && !filter.filterFiles)
+	}
+	else if (!dir && !filter.filterFiles) {
 		return false;
+	}
 
-	for (std::vector<CFilterCondition>::const_iterator iter = filter.filters.begin(); iter != filter.filters.end(); ++iter)
-	{
+	for (auto const& condition : filter.filters) {
 		bool match = false;
-		const CFilterCondition& condition = *iter;
 
 		switch (condition.type)
 		{
@@ -696,8 +670,9 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 			match = StringMatch(path, condition.strValue, condition.condition, filter.matchCase, condition.pRegEx);
 			break;
 		case filter_size:
-			if (size == -1)
+			if (size == -1) {
 				continue;
+			}
 			switch (condition.condition)
 			{
 			case 0:
@@ -802,7 +777,7 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 #endif //__WXMSW__
 			break;
 		case filter_date:
-			if (date.empty()) {
+			if (!date.empty()) {
 				int cmp = date.compare( condition.date );
 				switch (condition.condition)
 				{
@@ -826,40 +801,55 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 			break;
 		}
 		if (match) {
-			if (filter.matchType == CFilter::any)
+			if (filter.matchType == CFilter::any) {
 				return true;
-			else if (filter.matchType == CFilter::none)
+			}
+			else if (filter.matchType == CFilter::none) {
 				return false;
+			}
 		}
 		else {
-			if (filter.matchType == CFilter::all)
+			if (filter.matchType == CFilter::all) {
 				return false;
+			}
 		}
 	}
 
-	if (filter.matchType != CFilter::any || filter.filters.empty())
+	if (filter.matchType != CFilter::any || filter.filters.empty()) {
 		return true;
+	}
 
 	return false;
 }
 
+bool CFilterManager::CompileRegexes(std::vector<CFilter>& filters)
+{
+	bool ret = true;
+	for (auto & filter : filters) {
+		ret &= CompileRegexes(filter);
+	}
+	return ret;
+}
+
 bool CFilterManager::CompileRegexes(CFilter& filter)
 {
-	for (auto iter = filter.filters.begin(); iter != filter.filters.end(); ++iter)
-	{
-		CFilterCondition& condition = *iter;
+	bool ret = true;
+	for (auto & condition : filter.filters) {
 		if ((condition.type == filter_name || condition.type == filter_path) && condition.condition == 4) {
-			condition.pRegEx = std::make_shared<wxRegEx>(condition.strValue);
-			if (!condition.pRegEx->IsValid()) {
+			try {
+				condition.pRegEx = std::make_shared<std::wregex>(condition.strValue.ToStdWstring());
+			}
+			catch (std::regex_error const&) {
 				condition.pRegEx.reset();
-				return false;
+				ret = false;
 			}
 		}
-		else
+		else {
 			condition.pRegEx.reset();
+		}
 	}
 
-	return true;
+	return ret;
 }
 
 bool CFilterManager::CompileRegexes()
@@ -939,7 +929,7 @@ bool CFilterManager::LoadFilter(pugi::xml_node& element, CFilter& filter)
 		}
 		else if (condition.type == filter_date) {
 			condition.date = fz::datetime(condition.strValue.ToStdWstring(), fz::datetime::local);
-			if (!condition.date.empty()) {
+			if (condition.date.empty()) {
 				continue;
 			}
 		}
@@ -1034,24 +1024,23 @@ void CFilterManager::ToggleFilters()
 		m_filters_disabled = true;
 }
 
-std::vector<CFilter> CFilterManager::GetActiveFilters(bool local)
+ActiveFilters CFilterManager::GetActiveFilters()
 {
-	std::vector<CFilter> filters;
+	ActiveFilters filters;
 
-	if (m_filters_disabled)
+	if (m_filters_disabled) {
 		return filters;
+	}
 
 	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
 
 	// Check active filters
 	for (unsigned int i = 0; i < m_globalFilters.size(); ++i) {
-		if (local) {
-			if (set.local[i])
-				filters.push_back(m_globalFilters[i]);
+		if (set.local[i]) {
+			filters.first.push_back(m_globalFilters[i]);
 		}
-		else {
-			if (set.remote[i])
-				filters.push_back(m_globalFilters[i]);
+		if (set.remote[i]) {
+			filters.second.push_back(m_globalFilters[i]);
 		}
 	}
 
