@@ -568,7 +568,7 @@ CFilter CFilterConditionsDialog::GetFilter()
 	CFilter filter;
 	for (unsigned int i = 0; i < m_currentFilter.filters.size(); ++i) {
 		CFilterControls const& controls = m_filterControls[i];
-		if (!controls.pType || !controls.pCondition || !controls.pValue) {
+		if (!controls.pType || !controls.pCondition) {
 			continue;
 		}
 		CFilterCondition condition = m_currentFilter.filters[i];
@@ -580,14 +580,16 @@ CFilter CFilterConditionsDialog::GetFilter()
 		{
 		case filter_name:
 		case filter_path:
-			if (controls.pValue->GetValue().empty())
+			if (!controls.pValue || controls.pValue->GetValue().empty()) {
 				continue;
+			}
 			condition.strValue = controls.pValue->GetValue();
 			break;
 		case filter_size:
 			{
-				if (controls.pValue->GetValue().empty())
+				if (!controls.pValue || controls.pValue->GetValue().empty()) {
 					continue;
+				}
 				condition.strValue = controls.pValue->GetValue();
 				unsigned long long tmp;
 				condition.strValue.ToULongLong(&tmp);
@@ -596,7 +598,10 @@ CFilter CFilterConditionsDialog::GetFilter()
 			break;
 		case filter_attributes:
 		case filter_permissions:
-			if (controls.pSet->GetSelection()) {
+			if (!controls.pSet) {
+				continue;
+			}
+			else if (controls.pSet->GetSelection()) {
 				condition.strValue = _T("0");
 				condition.value = 0;
 			}
@@ -606,12 +611,13 @@ CFilter CFilterConditionsDialog::GetFilter()
 			}
 			break;
 		case filter_date:
-			if (controls.pValue->GetValue().empty())
+			if (!controls.pValue || controls.pValue->GetValue().empty()) {
 				continue;
+			}
 			else {
 				condition.strValue = controls.pValue->GetValue();
 				condition.date = fz::datetime(condition.strValue.ToStdWstring(), fz::datetime::local);
-				if (!condition.date.empty()) {
+				if (condition.date.empty()) {
 					continue;
 				}
 			}
@@ -672,25 +678,43 @@ bool CFilterConditionsDialog::ValidateFilter(wxString& error, bool allow_empty /
 
 	for (unsigned int i = 0; i < size; ++i) {
 		const CFilterControls& controls = m_filterControls[i];
-		enum t_filterType type = GetTypeFromTypeSelection(controls.pType->GetSelection());
+		t_filterType type = GetTypeFromTypeSelection(controls.pType->GetSelection());
+		int condition = controls.pCondition ? controls.pCondition->GetSelection() : 0;
 		if (!controls.pValue) {
 			continue;
 		}
 
-		if ((type == filter_name || type == filter_path) && (!controls.pValue || controls.pValue->GetValue().empty())) {
-			if (allow_empty)
-				continue;
+		if (type == filter_name || type == filter_path) {
+			if (!controls.pValue || controls.pValue->GetValue().empty()) {
+				if (allow_empty) {
+					continue;
+				}
 
-			m_pListCtrl->SelectLine(i);
-			controls.pValue->SetFocus();
-			error = _("At least one filter condition is incomplete");
-			SetFilterCtrlState(false);
-			return false;
+				m_pListCtrl->SelectLine(i);
+				controls.pValue->SetFocus();
+				error = _("At least one filter condition is incomplete");
+				SetFilterCtrlState(false);
+				return false;
+			}
+
+			if (condition == 4) {
+				try {
+					std::wregex(controls.pValue->GetValue().ToStdWstring());
+				}
+				catch (std::regex_error const&) {
+					m_pListCtrl->SelectLine(i);
+					controls.pValue->SetFocus();
+					error = _("Invalid regular expression");
+					SetFilterCtrlState(false);
+					return false;
+				}
+			}
 		}
 		else if (type == filter_size) {
 			const wxString v = controls.pValue->GetValue();
-			if (v.empty() && allow_empty)
+			if (v.empty() && allow_empty) {
 				continue;
+			}
 
 			long long number;
 			if (!v.ToLongLong(&number) || number < 0) {
@@ -702,12 +726,13 @@ bool CFilterConditionsDialog::ValidateFilter(wxString& error, bool allow_empty /
 			}
 		}
 		else if (type == filter_date) {
-			const wxString d = controls.pValue->GetValue();
-			if (d.empty() && allow_empty)
+			wxString const d = controls.pValue->GetValue();
+			if (d.empty() && allow_empty) {
 				continue;
+			}
 
-			wxDateTime date;
-			if (!date.ParseFormat(d, _T("%Y-%m-%d")) || !date.IsValid()) {
+			fz::datetime date(d.ToStdWstring(), fz::datetime::local);
+			if (date.empty()) {
 				m_pListCtrl->SelectLine(i);
 				controls.pValue->SetFocus();
 				error = _("Please enter a date of the form YYYY-MM-DD such as for example 2010-07-18.");
