@@ -52,6 +52,8 @@ bool CLoginManager::DisplayDialog(CServer &server, wxString const& name, wxStrin
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_CHALLENGE", wxTextCtrl), false, true);
 	}
 	else {
+		challenge.Trim(true);
+		challenge.Trim(false);
 #ifdef __WXMSW__
 		challenge.Replace(_T("\n"), _T("\r\n"));
 #endif
@@ -59,22 +61,24 @@ bool CLoginManager::DisplayDialog(CServer &server, wxString const& name, wxStrin
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox), canRemember, true);
 		XRCCTRL(pwdDlg, "ID_CHALLENGE", wxTextCtrl)->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 	}
-	XRCCTRL(pwdDlg, "ID_HOST", wxStaticText)->SetLabel(server.FormatHost());
+	XRCCTRL(pwdDlg, "ID_HOST", wxStaticText)->SetLabel(server.Format(ServerFormat::with_optional_port));
 
 	if (server.GetUser().empty()) {
-		pwdDlg.SetTitle(_("Enter username and password"));
 		XRCCTRL(pwdDlg, "ID_OLD_USER_LABEL", wxStaticText)->Hide();
 		XRCCTRL(pwdDlg, "ID_OLD_USER", wxStaticText)->Hide();
 
 		XRCCTRL(pwdDlg, "ID_HEADER_PASS", wxStaticText)->Hide();
 		if (server.GetLogonType() == INTERACTIVE) {
+			pwdDlg.SetTitle(_("Enter username"));
 			XRCCTRL(pwdDlg, "ID_PASSWORD_LABEL", wxStaticText)->Hide();
 			XRCCTRL(pwdDlg, "ID_PASSWORD", wxTextCtrl)->Hide();
 			XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox)->Hide();
 			XRCCTRL(pwdDlg, "ID_HEADER_BOTH", wxStaticText)->Hide();
 		}
-		else
+		else {
+			pwdDlg.SetTitle(_("Enter username and password"));
 			XRCCTRL(pwdDlg, "ID_HEADER_USER", wxStaticText)->Hide();
+		}
 
 		XRCCTRL(pwdDlg, "ID_NEW_USER", wxTextCtrl)->SetFocus();
 	}
