@@ -221,7 +221,7 @@ void CQueueStorage::Impl::ReadLocalPaths()
 			int64_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
 			wxString localPathRaw = GetColumnText(selectLocalPathQuery_, path_table_column_names::path);
 			CLocalPath localPath;
-			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw))
+			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw.ToStdWstring()))
 				reverseLocalPaths_[id] = localPath;
 		}
 	}
@@ -590,7 +590,7 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	Bind(insertServerQuery_, server_table_column_names::protocol, static_cast<int>(server.GetProtocol()));
 	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(server.GetType()));
 
-	enum LogonType logonType = server.GetLogonType();
+	LogonType logonType = server.GetLogonType();
 	if (server.GetLogonType() != ANONYMOUS) {
 		Bind(insertServerQuery_, server_table_column_names::user, server.GetUser());
 
@@ -852,13 +852,13 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 	if (type < 0 || type >= SERVERTYPE_MAX)
 		return INVALID_DATA;
 
-	server.SetType((enum ServerType)type);
+	server.SetType(static_cast<ServerType>(type));
 
 	int logonType = GetColumnInt(selectServersQuery_, server_table_column_names::logontype);
 	if (logonType < 0 || logonType >= LOGONTYPE_MAX)
 		return INVALID_DATA;
 
-	server.SetLogonType((enum LogonType)logonType);
+	server.SetLogonType(static_cast<LogonType>(logonType));
 
 	if (server.GetLogonType() != ANONYMOUS)
 	{
@@ -935,7 +935,7 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 
 
 	server.SetBypassProxy(GetColumnInt(selectServersQuery_, server_table_column_names::bypass_proxy) == 1 );
-	server.SetName( GetColumnText(selectServersQuery_, server_table_column_names::name) );
+	server.SetName(GetColumnText(selectServersQuery_, server_table_column_names::name).ToStdWstring());
 
 	return GetColumnInt64(selectServersQuery_, server_table_column_names::id);
 }
@@ -1180,9 +1180,14 @@ bool CQueueStorage::BeginTransaction()
 	return sqlite3_exec(d_->db_, "BEGIN TRANSACTION", 0, 0, 0) == SQLITE_OK;
 }
 
-bool CQueueStorage::EndTransaction()
+bool CQueueStorage::EndTransaction(bool rollback)
 {
-	return sqlite3_exec(d_->db_, "END TRANSACTION", 0, 0, 0) == SQLITE_OK;
+	if (rollback) {
+		return sqlite3_exec(d_->db_, "ROLLBACK", 0, 0, 0) == SQLITE_OK;
+	}
+	else {
+		return sqlite3_exec(d_->db_, "END TRANSACTION", 0, 0, 0) == SQLITE_OK;
+	}
 }
 
 bool CQueueStorage::Vacuum()
