@@ -39,7 +39,7 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 
 #ifdef __WXMAC__
 	// Under OS X default buttons are toplevel window wide, where under Windows / GTK they stop at the parent panel.
-	wxTopLevelWindow *tlw = wxDynamicCast(wxGetTopLevelParent(pParent), wxTopLevelWindow);
+	wxTopLevelWindow *tlw = dynamic_cast<wxTopLevelWindow*>(wxGetTopLevelParent(pParent));
 	if (tlw)
 		tlw->SetDefaultItem(0);
 #endif
@@ -77,8 +77,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	wxString error;
 
 	CServerPath path;
-	if (!server.ParseUrl(host, port, user, pass, error, path))
-	{
+	if (!server.ParseUrl(host, port, user, pass, error, path)) {
 		wxString msg = _("Could not parse server address:");
 		msg += _T("\n");
 		msg += error;
@@ -86,7 +85,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		return;
 	}
 
-	host = server.FormatHost(true);
+	host = server.Format(ServerFormat::host_only);
 	ServerProtocol protocol = server.GetProtocol();
 	switch (protocol)
 	{
@@ -133,8 +132,13 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		server.SetLogonType(ASK);
 		CLoginManager::Get().RememberPassword(server);
 	}
-	if (!m_pMainFrame->ConnectToServer(server, path))
+	Site site;
+	site.m_server = server;
+	Bookmark bm;
+	bm.m_remoteDir = path;
+	if (!m_pMainFrame->ConnectToSite(site, bm)) {
 		return;
+	}
 
 	CRecentServerList::SetMostRecentServer(server);
 }
@@ -159,7 +163,7 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 			iter != m_recentServers.end();
 			++iter, ++i)
 		{
-			wxString name(iter->FormatServer());
+			wxString name(iter->Format(ServerFormat::with_user_and_optional_port));
 			name.Replace(_T("&"), _T("&&"));
 			pMenu->Append(10 + i, name);
 		}
@@ -175,37 +179,31 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 {
 	const int id = event.GetId();
-	if (id == 1)
+	if (id == 1) {
 		OnQuickconnect(event);
-	else if (id == 2)
+	}
+	else if (id == 2) {
 		ClearFields();
-	else if (id == 3)
+	}
+	else if (id == 3) {
 		CRecentServerList::Clear();
+	}
 
-	if (id < 10)
+	if (id < 10) {
 		return;
+	}
 
 	unsigned int index = id - 10;
-	if (index >= m_recentServers.size())
+	if (index >= m_recentServers.size()) {
 		return;
+	}
 
 	std::list<CServer>::const_iterator iter = m_recentServers.begin();
 	std::advance(iter, index);
 
-	CServer server = *iter;
-	if (server.GetLogonType() == ASK) {
-		if (!CLoginManager::Get().GetPassword(server, false))
-			return;
-	}
-
-	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState || !pState->m_pEngine)
-	{
-		wxMessageBoxEx(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
-		return;
-	}
-
-	m_pMainFrame->ConnectToServer(server);
+	Site site;
+	site.m_server = *iter;
+	m_pMainFrame->ConnectToSite(site, Bookmark());
 }
 
 void CQuickconnectBar::ClearFields()
