@@ -1,4 +1,4 @@
-	#include <filezilla.h>
+#include <filezilla.h>
 #include "sitemanager.h"
 
 #include "filezillaapp.h"
@@ -6,7 +6,81 @@
 #include "Options.h"
 #include "xmlfunctions.h"
 
-std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> CSiteManager::m_idMap;
+namespace {
+struct background_color {
+	wxColour const color;
+	wchar_t const*const name;
+};
+
+background_color const background_colors[] = {
+	{ wxColour(), TRANSLATE_T("None") },
+	{ wxColour(255, 0, 0, 32), TRANSLATE_T("Red") },
+	{ wxColour(0, 255, 0, 32), TRANSLATE_T("Green") },
+	{ wxColour(0, 0, 255, 32), TRANSLATE_T("Blue") },
+	{ wxColour(255, 255, 0, 32), TRANSLATE_T("Yellow") },
+	{ wxColour(0, 255, 255, 32), TRANSLATE_T("Cyan") },
+	{ wxColour(255, 0, 255, 32), TRANSLATE_T("Magenta") },
+	{ wxColour(255, 128, 0, 32), TRANSLATE_T("Orange") },
+	{ wxColour(), 0 }
+};
+}
+
+bool Bookmark::operator==(Bookmark const& b) const
+{
+	if (m_localDir != b.m_localDir) {
+		return false;
+	}
+	
+	if (m_remoteDir != b.m_remoteDir) {
+		return false;
+	}
+
+	if (m_sync != b.m_sync) {
+		return false;
+	}
+
+	if (m_comparison != b.m_comparison) {
+		return false;
+	}
+
+	if (m_name != b.m_name) {
+		return false;
+	}
+
+	return true;
+}
+
+bool Site::operator==(Site const& s) const
+{
+	if (m_server != s.m_server) {
+		return false;
+	}
+
+	if (m_comments != s.m_comments) {
+		return false;
+	}
+
+	if (m_default_bookmark != s.m_default_bookmark) {
+		return false;
+	}
+
+	if (m_bookmarks != s.m_bookmarks) {
+		return false;
+	}
+
+	if (m_path != s.m_path) {
+		return false;
+	}
+
+	if (m_colour != s.m_colour) {
+		return false;
+	}
+
+	return true;
+}
+
+
+std::map<int, std::unique_ptr<Site>> CSiteManager::m_idMap;
 
 bool CSiteManager::Load(CSiteManagerXmlHandler& handler)
 {
@@ -31,47 +105,25 @@ bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
 
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
-			wxString name = GetTextElement_Trimmed(child);
-			if (name.empty())
+			std::wstring name = GetTextElement_Trimmed(child).ToStdWstring();
+			if (name.empty()) {
 				continue;
+			}
 
 			const bool expand = GetTextAttribute(child, "expanded") != _T("0");
-			if (!handler.AddFolder(name, expand))
+			if (!handler.AddFolder(name, expand)) {
 				return false;
+			}
 			Load(child, handler);
-			if (!handler.LevelUp())
+			if (!handler.LevelUp()) {
 				return false;
+			}
 		}
 		else if (!strcmp(child.name(), "Server")) {
-			std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(child);
+			std::unique_ptr<Site> data = ReadServerElement(child);
 
 			if (data) {
 				handler.AddSite(std::move(data));
-
-				// Bookmarks
-				for (auto bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-					wxString name = GetTextElement_Trimmed(bookmark, "Name");
-					if (name.empty())
-						continue;
-
-					auto bookmarkData = std::make_unique<CSiteManagerItemData>(CSiteManagerItemData::BOOKMARK);
-
-					bookmarkData->m_localDir = GetTextElement(bookmark, "LocalDir");
-					bookmarkData->m_remoteDir.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
-					if (bookmarkData->m_localDir.empty() && bookmarkData->m_remoteDir.empty()) {
-						continue;
-					}
-
-					if (!bookmarkData->m_localDir.empty() && !bookmarkData->m_remoteDir.empty())
-						bookmarkData->m_sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
-
-					bookmarkData->m_comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
-
-					handler.AddBookmark(name, std::move(bookmarkData));
-				}
-
-				if (!handler.LevelUp())
-					return false;
 			}
 		}
 	}
@@ -79,7 +131,25 @@ bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
 	return true;
 }
 
-std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::ReadServerElement(pugi::xml_node element)
+bool CSiteManager::ReadBookmarkElement(Bookmark & bookmark, pugi::xml_node element)
+{
+	bookmark.m_localDir = GetTextElement(element, "LocalDir");
+	bookmark.m_remoteDir.SetSafePath(GetTextElement(element, "RemoteDir"));
+
+	if (bookmark.m_localDir.empty() && bookmark.m_remoteDir.empty()) {
+		return false;
+	}
+
+	if (!bookmark.m_localDir.empty() && !bookmark.m_remoteDir.empty()) {
+		bookmark.m_sync = GetTextElementBool(element, "SyncBrowsing", false);
+	}
+
+	bookmark.m_comparison = GetTextElementBool(element, "DirectoryComparison", false);
+
+	return true;
+}
+
+std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
 	CServer server;
 	if (!::GetServer(element, server))
@@ -87,15 +157,27 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::ReadServerElement(pugi:
 	if (server.GetName().empty())
 		return 0;
 
-	auto data = std::make_unique<CSiteManagerItemData_Site>(server);
+	auto data = std::make_unique<Site>();
+	data->m_server = server;
+
 	data->m_comments = GetTextElement(element, "Comments");
-	data->m_localDir = GetTextElement(element, "LocalDir");
-	data->m_remoteDir.SetSafePath(GetTextElement(element, "RemoteDir"));
+	data->m_colour = GetColourFromIndex(GetTextElementInt(element, "Colour"));
+	
+	ReadBookmarkElement(data->m_default_bookmark, element);
 
-	if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
-		data->m_sync = GetTextElementBool(element, "SyncBrowsing", false);
+	// Bookmarks
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
+		wxString name = GetTextElement_Trimmed(bookmark, "Name");
+		if (name.empty()) {
+			continue;
+		}
 
-	data->m_comparison = GetTextElementBool(element, "DirectoryComparison", false);
+		Bookmark bookmarkData;
+		if (ReadBookmarkElement(bookmarkData, bookmark)) {
+			bookmarkData.m_name = name;
+			data->m_bookmarks.push_back(bookmarkData);
+		}
+	}
 
 	return data;
 }
@@ -103,40 +185,42 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::ReadServerElement(pugi:
 class CSiteManagerXmlHandler_Menu : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *idMap, bool predefined)
+	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, std::unique_ptr<Site>> *idMap, bool predefined)
 		: m_pMenu(pMenu), m_idMap(idMap)
 	{
-		m_added_site = false;
-
-		if (predefined)
+		if (predefined) {
 			path = _T("1");
-		else
+		}
+		else {
 			path = _T("0");
+		}
 	}
 
 	unsigned int GetInsertIndex(wxMenu* pMenu, const wxString& name)
 	{
 		unsigned int i;
-		for (i = 0; i < pMenu->GetMenuItemCount(); ++i)
-		{
+		for (i = 0; i < pMenu->GetMenuItemCount(); ++i) {
 			const wxMenuItem* const pItem = pMenu->FindItemByPosition(i);
-			if (!pItem)
+			if (!pItem) {
 				continue;
+			}
 
 			// Use same sorting as site tree in site manager
 #ifdef __WXMSW__
-			if (pItem->GetItemLabelText().CmpNoCase(name) > 0)
+			if (pItem->GetItemLabelText().CmpNoCase(name) > 0) {
 				break;
+			}
 #else
-			if (pItem->GetItemLabelText() > name)
+			if (pItem->GetItemLabelText() > name) {
 				break;
+			}
 #endif
 		}
 
 		return i;
 	}
 
-	virtual bool AddFolder(const wxString& name, bool)
+	virtual bool AddFolder(std::wstring const& name, bool)
 	{
 		m_parents.push_back(m_pMenu);
 		m_childNames.push_back(name);
@@ -148,7 +232,7 @@ public:
 		return true;
 	}
 
-	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site> data)
+	virtual bool AddSite(std::unique_ptr<Site> data)
 	{
 		wxString newName(data->m_server.GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
@@ -159,32 +243,19 @@ public:
 
 		(*m_idMap)[pItem->GetId()] = std::move(data);
 
-		m_added_site = true;
-
-		return true;
-	}
-
-	virtual bool AddBookmark(const wxString&, std::unique_ptr<CSiteManagerItemData>)
-	{
 		return true;
 	}
 
 	// Go up a level
 	virtual bool LevelUp()
 	{
-		if (m_added_site)
-		{
-			m_added_site = false;
-			return true;
-		}
-
-		if (m_parents.empty() || m_childNames.empty())
+		if (m_parents.empty() || m_childNames.empty()) {
 			return false;
+		}
 
 		wxMenu* pChild = m_pMenu;
 		m_pMenu = m_parents.back();
-		if (pChild->GetMenuItemCount())
-		{
+		if (pChild->GetMenuItemCount()) {
 			wxString name = m_childNames.back();
 			int i = GetInsertIndex(m_pMenu, name);
 			name.Replace(_T("&"), _T("&&"));
@@ -192,8 +263,9 @@ public:
 			wxMenuItem* pItem = new wxMenuItem(m_pMenu, wxID_ANY, name, _T(""), wxITEM_NORMAL, pChild);
 			m_pMenu->Insert(i, pItem);
 		}
-		else
+		else {
 			delete pChild;
+		}
 		m_childNames.pop_back();
 		m_parents.pop_back();
 
@@ -206,36 +278,31 @@ public:
 protected:
 	wxMenu* m_pMenu;
 
-	std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *m_idMap;
+	std::map<int, std::unique_ptr<Site>> *m_idMap;
 
-	std::list<wxMenu*> m_parents;
-	std::list<wxString> m_childNames;
+	std::deque<wxMenu*> m_parents;
+	std::deque<std::wstring> m_childNames;
 
-	bool m_added_site;
-
-	wxString path;
-	std::list<wxString> m_paths;
+	std::wstring path;
+	std::deque<std::wstring> m_paths;
 };
 
 
 class CSiteManagerXmlHandler_Stats : public CSiteManagerXmlHandler
 {
 public:
-	virtual bool AddFolder(const wxString&, bool)
+	virtual bool AddFolder(std::wstring const&, bool)
 	{
 		++directories_;
 		return true;
 	}
 
-	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site>)
+	virtual bool AddSite(std::unique_ptr<Site> site)
 	{
-		++sites_;
-		return true;
-	}
-
-	virtual bool AddBookmark(const wxString&, std::unique_ptr<CSiteManagerItemData>)
-	{
-		++bookmarks_;
+		if (site) {
+			++sites_;
+			bookmarks_ += site->m_bookmarks.size();
+		}
 		return true;
 	}
 
@@ -311,7 +378,7 @@ bool CSiteManager::LoadPredefined(CSiteManagerXmlHandler& handler)
 	return true;
 }
 
-std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu_Predefined(std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> &idMap)
+std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu_Predefined(std::map<int, std::unique_ptr<Site>> &idMap)
 {
 	auto pMenu = std::make_unique<wxMenu>();
 	CSiteManagerXmlHandler_Menu handler(pMenu.get(), &idMap, true);
@@ -327,11 +394,11 @@ std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu_Predefined(std::map<int, std:
 	return pMenu;
 }
 
-std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteById(int id)
+std::unique_ptr<Site> CSiteManager::GetSiteById(int id)
 {
 	auto iter = m_idMap.find(id);
 
-	std::unique_ptr<CSiteManagerItemData_Site> pData;
+	std::unique_ptr<Site> pData;
 	if (iter != m_idMap.end()) {
 		pData = std::move(iter->second);
 	}
@@ -340,79 +407,93 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteById(int id)
 	return pData;
 }
 
-bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
+bool CSiteManager::UnescapeSitePath(std::wstring path, std::vector<std::wstring>& result)
 {
 	result.clear();
 
-	wxString name;
-	wxChar const* p = path.c_str();
+	std::wstring name;
+	wchar_t const* p = path.c_str();
 
 	// Undo escapement
 	bool lastBackslash = false;
-	while (*p)
-	{
+	while (*p) {
 		const wxChar& c = *p;
-		if (c == '\\')
-		{
-			if (lastBackslash)
-			{
+		if (c == '\\') {
+			if (lastBackslash) {
 				name += _T("\\");
 				lastBackslash = false;
 			}
-			else
+			else {
 				lastBackslash = true;
+			}
 		}
-		else if (c == '/')
-		{
-			if (lastBackslash)
-			{
+		else if (c == '/') {
+			if (lastBackslash) {
 				name += _T("/");
 				lastBackslash = 0;
 			}
-			else
-			{
-				if (!name.empty())
+			else {
+				if (!name.empty()) {
 					result.push_back(name);
+				}
 				name.clear();
 			}
 		}
-		else
+		else {
 			name += *p;
+		}
 		++p;
 	}
-	if (lastBackslash)
+	if (lastBackslash) {
 		return false;
-	if (!name.empty())
+	}
+	if (!name.empty()) {
 		result.push_back(name);
+	}
 
 	return !result.empty();
 }
 
-wxString CSiteManager::EscapeSegment(wxString segment)
+std::wstring CSiteManager::EscapeSegment(std::wstring segment)
 {
-	segment.Replace(_T("\\"), _T("\\\\"));
-	segment.Replace(_T("/"), _T("\\/"));
+	fz::replace_substrings(segment, _T("\\"), _T("\\\\"));
+	fz::replace_substrings(segment, _T("/"), _T("\\/"));
 	return segment;
 }
 
-wxString CSiteManager::BuildPath(wxChar root, std::list<wxString> const& segments)
+std::wstring CSiteManager::BuildPath(wxChar root, std::vector<std::wstring> const& segments)
 {
-	wxString ret = root;
-	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
-		ret += _T("/") + EscapeSegment(*it);
+	std::wstring ret;
+	ret += root;
+	for (auto const& segment : segments) {
+		ret += _T("/") + EscapeSegment(segment);
+	}
 
 	return ret;
 }
 
-std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString sitePath)
+std::pair<std::unique_ptr<Site>, Bookmark> CSiteManager::GetSiteByPath(std::wstring const& sitePath, bool printErrors)
 {
-	wxChar c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0' && c != '1') {
-		wxMessageBoxEx(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
-		return 0;
+	wxString error;
+
+	auto ret = DoGetSiteByPath(sitePath, error);
+	if (!ret.first && printErrors) {
+		wxMessageBoxEx(_("Site does not exist."), error);
 	}
 
-	sitePath = sitePath.Mid(1);
+	return ret;
+}
+
+std::pair<std::unique_ptr<Site>, Bookmark> CSiteManager::DoGetSiteByPath(std::wstring sitePath, wxString& error)
+{
+	std::pair<std::unique_ptr<Site>, Bookmark> ret;
+	wxChar c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0' && c != '1') {
+		error = _("Site path has to begin with 0 or 1.");
+		return ret;
+	}
+
+	sitePath = sitePath.substr(1);
 
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -425,8 +506,8 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString
 	else {
 		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
 		if (defaultsDir.empty()) {
-			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
-			return 0;
+			error = _("Site does not exist.");
+			return ret;
 		}
 		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
@@ -434,25 +515,25 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString
 	auto document = file.Load();
 	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-		return 0;
+		return ret;
 	}
 
 	auto element = document.child("Servers");
 	if (!element) {
-		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
-		return 0;
+		error = _("Site does not exist.");
+		return ret;
 	}
 
-	std::list<wxString> segments;
+	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments) || segments.empty()) {
-		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
-		return 0;
+		error = _("Site path is malformed.");
+		return ret;
 	}
 
 	auto child = GetElementByPath(element, segments);
 	if (!child) {
-		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
-		return 0;
+		error = _("Site does not exist.");
+		return ret;
 	}
 
 	pugi::xml_node bookmark;
@@ -462,96 +543,25 @@ std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString
 		segments.pop_back();
 	}
 
-	std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(child);
-
-	if (!data) {
-		wxMessageBoxEx(_("Could not read server item."), _("Invalid site path"));
-		return 0;
-	}
-
-	if (bookmark) {
-		data->m_localDir = GetTextElement(bookmark, "LocalDir");
-		data->m_remoteDir.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
-		if (!data->m_localDir.empty() && !data->m_remoteDir.empty()) {
-			data->m_sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
-		}
-		else
-			data->m_sync = false;
-		data->m_comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
-	}
-
-	data->m_path = BuildPath( c, segments );
-
-	return data;
-}
-
-bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmarks)
-{
-	if (sitePath.empty())
-		return false;
-	wxChar c = sitePath[0];
-	if (c != '0' && c != '1')
-		return false;
-
-	sitePath = sitePath.Mid(1);
-
-	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
-	// to the same file or one is reading while the other one writes.
-	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
-
-	CXmlFile file;
-	if (c == '0') {
-		file.SetFileName(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	ret.first = ReadServerElement(child);
+	if (!ret.first) {
+		error = _("Could not read server item.");
 	}
 	else {
-		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
-		if (defaultsDir.empty()) {
-			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
-			return 0;
+		if (bookmark) {
+			Bookmark bm;
+			if (ReadBookmarkElement(bm, bookmark)) {
+				ret.second = bm;
+			}
 		}
-		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
+		else {
+			ret.second = ret.first->m_default_bookmark;
+		}
+
+		ret.first->m_path = BuildPath(c, segments);
 	}
 
-	auto document = file.Load();
-	if (!document) {
-		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-		return false;
-	}
-
-	auto element = document.child("Servers");
-	if (!element)
-		return false;
-
-	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments)) {
-		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
-		return 0;
-	}
-
-	auto child = GetElementByPath(element, segments);
-
-	if (child && !strcmp(child.name(), "Bookmark"))
-		child = child.parent();
-
-	if (!child || strcmp(child.name(), "Server"))
-		return 0;
-
-	// Bookmarks
-	for (auto bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString name = GetTextElement_Trimmed(bookmark, "Name");
-		if (name.empty())
-			continue;
-
-		wxString localPath = GetTextElement(bookmark, "LocalDir");
-		CServerPath remotePath;
-		remotePath.SetSafePath(GetTextElement(bookmark, "RemoteDir"));
-		if (localPath.empty() && remotePath.empty())
-			continue;
-
-		bookmarks.push_back(name);
-	}
-
-	return true;
+	return ret;
 }
 
 wxString CSiteManager::AddServer(CServer server)
@@ -582,7 +592,7 @@ wxString CSiteManager::AddServer(CServer server)
 		names.push_back(name);
 	}
 
-	wxString name = _("New site");
+	std::wstring name = _("New site").ToStdWstring();
 	int i = 1;
 
 	for (;;) {
@@ -615,11 +625,9 @@ wxString CSiteManager::AddServer(CServer server)
 	return _T("0/") + EscapeSegment(name);
 }
 
-pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::list<wxString> const& segments)
+pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<std::wstring> const& segments)
 {
-	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it) {
-		const wxString & segment = *it;
-
+	for (auto const& segment : segments) {
 		pugi::xml_node child;
 		for (child = node.first_child(); child; child = child.next_sibling()) {
 			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark"))
@@ -644,7 +652,7 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::list<wxS
 	return node;
 }
 
-bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
+bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
 	if (local_dir.empty() && remote_dir.empty())
 		return false;
@@ -653,7 +661,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	if (c != '0')
 		return false;
 
-	sitePath = sitePath.Mid(1);
+	sitePath = sitePath.substr(1);
 
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -672,7 +680,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	if (!element)
 		return false;
 
-	std::list<wxString> segments;
+	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
 		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
@@ -724,13 +732,13 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	return true;
 }
 
-bool CSiteManager::ClearBookmarks(wxString sitePath)
+bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 {
 	wxChar const c = sitePath.empty() ? 0 : sitePath[0];
 	if (c != '0')
 		return false;
 
-	sitePath = sitePath.Mid(1);
+	sitePath = sitePath.substr(1);
 
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -749,7 +757,7 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	if (!element)
 		return false;
 
-	std::list<wxString> segments;
+	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
 		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
@@ -785,3 +793,31 @@ bool CSiteManager::HasSites()
 
 	return handler.sites_ > 0;
 }
+
+wxColour CSiteManager::GetColourFromIndex(int i)
+{
+	if (i < 0 || static_cast<unsigned int>(i) + 1 >= (sizeof(background_colors) / sizeof(background_color) + 1)) {
+		return wxColour();
+	}
+	return background_colors[i].color;
+}
+
+int CSiteManager::GetColourIndex(wxColour const& c)
+{
+	for (int i = 0; background_colors[i].name; ++i) {
+		if (c == background_colors[i].color) {
+			return i;
+		}
+	}
+
+	return 0;
+}
+
+wxString CSiteManager::GetColourName(int i)
+{
+	if (i < 0 || static_cast<unsigned int>(i) + 1 >= (sizeof(background_colors) / sizeof(background_color))) {
+		return wxString();
+	}
+	return wxGetTranslation(background_colors[i].name);
+}
+
