@@ -2,7 +2,7 @@
 #include "sizeformatting.h"
 #include "Options.h"
 
-wxString CSizeFormat::Format(int64_t size, bool add_bytes_suffix, enum CSizeFormat::_format format, bool thousands_separator, int num_decimal_places)
+wxString CSizeFormat::Format(int64_t size, bool add_bytes_suffix, CSizeFormat::_format format, bool thousands_separator, int num_decimal_places)
 {
 	return CSizeFormatBase::Format(COptions::Get(), size, add_bytes_suffix, format, thousands_separator, num_decimal_places);
 }
