@@ -6,7 +6,8 @@
 #include "Mainfrm.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "recursive_operation.h"
+#include "local_recursive_operation.h"
+#include "remote_recursive_operation.h"
 #include "listingcomparison.h"
 #include "xrc_helper.h"
 
@@ -44,8 +45,7 @@ void CContextManager::DestroyState(CState* pState)
 		m_contexts.erase(m_contexts.begin() + i);
 		if ((int)i < m_current_context)
 			m_current_context--;
-		else if ((int)i == m_current_context)
-		{
+		else if ((int)i == m_current_context) {
 			if (i >= m_contexts.size())
 				m_current_context--;
 			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
@@ -63,8 +63,7 @@ void CContextManager::SetCurrentContext(CState* pState)
 	if (GetCurrentContext() == pState)
 		return;
 
-	for (unsigned int i = 0; i < m_contexts.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
 		if (m_contexts[i] != pState)
 			continue;
 
@@ -78,8 +77,7 @@ void CContextManager::DestroyAllStates()
 	m_current_context = -1;
 	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 
-	while (!m_contexts.empty())
-	{
+	while (!m_contexts.empty()) {
 		CState* pState = m_contexts.back();
 		m_contexts.pop_back();
 
@@ -88,7 +86,7 @@ void CContextManager::DestroyAllStates()
 	}
 }
 
-void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only)
+void CContextManager::RegisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification, bool current_only)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
@@ -106,7 +104,7 @@ void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_state
 	handlers.push_back(handler);
 }
 
-void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CContextManager::UnregisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
@@ -139,7 +137,7 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -160,13 +158,13 @@ CState* CContextManager::GetCurrentContext()
 	return m_contexts[m_current_context];
 }
 
-void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	for (unsigned int i = 0; i < m_contexts.size(); i++)
 		m_contexts[i]->NotifyHandlers(notification, data, data2);
 }
 
-void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
@@ -179,34 +177,29 @@ CState::CState(CMainFrame &mainFrame)
 {
 	m_title = _("Not connected");
 
-	m_pComparisonManager = new CComparisonManager(this);
+	m_pComparisonManager = new CComparisonManager(*this);
 
-	m_pRecursiveOperation = new CRecursiveOperation(this);
+	m_pLocalRecursiveOperation = new CLocalRecursiveOperation(*this);
+	m_pRemoteRecursiveOperation = new CRemoteRecursiveOperation(*this);
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.compare = false;
 
-	m_localDir.SetPath(CLocalPath::path_separator);
+	m_localDir.SetPath(std::wstring(1, CLocalPath::path_separator));
 }
 
 CState::~CState()
 {
-	delete m_pServer;
-
 	delete m_pComparisonManager;
 	delete m_pCommandQueue;
 	delete m_pEngine;
+	delete m_pLocalRecursiveOperation;
+	delete m_pRemoteRecursiveOperation;
 
 	// Unregister all handlers
-	for (int i = 0; i < STATECHANGE_MAX; i++)
-	{
-		for (auto iter = m_handlers[i].begin(); iter != m_handlers[i].end(); ++iter)
-		{
-			iter->pHandler->m_pState = 0;
-		}
+	for (int i = 0; i < STATECHANGE_MAX; ++i) {
+		wxASSERT(m_handlers[i].empty());
 	}
-
-	delete m_pRecursiveOperation;
 }
 
 CLocalPath CState::GetLocalDir() const
@@ -244,7 +237,7 @@ bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPr
 		return false;
 
 	if (!m_sync_browse.local_root.empty()) {
-		wxASSERT(m_pServer);
+		wxASSERT(m_site.m_server);
 
 		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
@@ -427,13 +420,13 @@ void CState::RefreshLocalFile(wxString file)
 
 void CState::LocalDirCreated(const CLocalPath& path)
 {
-	if (!path.IsSubdirOf(m_localDir))
+	if (!path.IsSubdirOf(m_localDir)) {
 		return;
+	}
 
-	wxString next_segment = path.GetPath().Mid(m_localDir.GetPath().Len());
+	wxString next_segment = path.GetPath().substr(m_localDir.GetPath().size());
 	int pos = next_segment.Find(CLocalPath::path_separator);
-	if (pos <= 0)
-	{
+	if (pos <= 0) {
 		// Shouldn't ever come true
 		return;
 	}
@@ -445,52 +438,48 @@ void CState::LocalDirCreated(const CLocalPath& path)
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
 }
 
-void CState::SetServer(const CServer* server, CServerPath const& path)
+void CState::SetSite(Site const& site, CServerPath const& path)
 {
-	if (m_pServer) {
-		if (server && *server == *m_pServer &&
-			server->GetName() == m_pServer->GetName() &&
-			server->MaximumMultipleConnections() == m_pServer->MaximumMultipleConnections())
+	if (m_site.m_server) {
+		if (site.m_server && site.m_server == m_site.m_server &&
+			site.m_server.GetName() == m_site.m_server.GetName() &&
+			site.m_server.MaximumMultipleConnections() == m_site.m_server.MaximumMultipleConnections())
 		{
 			// Nothing changes
 			return;
 		}
 
 		SetRemoteDir(0);
-		delete m_pServer;
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
-	if (server) {
+	if (site.m_server) {
 		if (!path.empty()) {
 			m_last_path = path;
 		}
-		else if (m_last_server != *server) {
+		else if (m_last_site.m_server != site.m_server) {
 			m_last_path.clear();
 		}
-		m_last_server = *server;
-
-		m_pServer = new CServer(*server);
-
-		const wxString& name = server->GetName();
-		if (!name.empty())
-			m_title = name + _T(" - ") + server->FormatServer();
-		else
-			m_title = server->FormatServer();
-	}
-	else {
-		m_pServer = 0;
-		m_title = _("Not connected");
+		m_last_site = site;
 	}
+	
+	m_site = site;
+
+	UpdateTitle();
 
 	m_successful_connect = false;
 
 	NotifyHandlers(STATECHANGE_SERVER);
 }
 
-const CServer* CState::GetServer() const
+Site const& CState::GetSite() const
 {
-	return m_pServer;
+	return m_site;
+}
+
+CServer const* CState::GetServer() const
+{
+	return m_site.m_server ? &m_site.m_server : 0;
 }
 
 wxString CState::GetTitle() const
@@ -498,19 +487,24 @@ wxString CState::GetTitle() const
 	return m_title;
 }
 
-bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPath()*/)
+bool CState::Connect(Site const& site, CServerPath const& path)
 {
-	if (!m_pEngine)
+	if (!site.m_server) {
 		return false;
-	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle())
+	}
+	if (!m_pEngine) {
+		return false;
+	}
+	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle()) {
 		m_pCommandQueue->Cancel();
-	m_pRecursiveOperation->StopRecursiveOperation();
+	}
+	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
 
-	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
+	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.m_server));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
-	SetServer(&server, path);
+	SetSite(site, path);
 
 	return true;
 }
@@ -526,7 +520,7 @@ bool CState::Disconnect()
 	if (!IsRemoteIdle())
 		return false;
 
-	SetServer(0);
+	SetSite(Site());
 	m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
 
 	return true;
@@ -538,10 +532,9 @@ bool CState::CreateEngine()
 	if (m_pEngine)
 		return true;
 
-	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext());
-	m_pEngine->Init(&m_mainFrame);
+	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext(), m_mainFrame);
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, this);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, *this);
 
 	return true;
 }
@@ -554,11 +547,11 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, CStateEventHandler* insertBefore)
+void CState::RegisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification, CStateEventHandler* insertBefore)
 {
 	wxASSERT(pHandler);
-	wxASSERT(pHandler->m_pState == this);
-	if (pHandler->m_pState != this)
+	wxASSERT(&pHandler->m_state == this);
+	if (!pHandler || &pHandler->m_state != this)
 		return;
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 	wxASSERT(pHandler != insertBefore);
@@ -582,7 +575,7 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_no
 	handlers.insert(insertionPoint, handler);
 }
 
-void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
@@ -609,36 +602,38 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 }
 
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CState::NotifyHandlers(t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
-		handler.pHandler->OnStateChange(this, notification, data, data2);
+		handler.pHandler->OnStateChange(notification, data, data2);
 	}
 
 	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
-CStateEventHandler::CStateEventHandler(CState* pState)
-	: m_pState(pState)
+CStateEventHandler::CStateEventHandler(CState &state)
+	: m_state(state)
 {
 }
 
 CStateEventHandler::~CStateEventHandler()
 {
-	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
+	m_state.UnregisterHandler(this, STATECHANGE_NONE);
+}
 
-	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-		(*iter)->UnregisterHandler(this, STATECHANGE_NONE);
+CGlobalStateEventHandler::~CGlobalStateEventHandler()
+{
+	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
 {
-	if (!m_pServer || !m_pDirectoryListing)
+	if (!m_site.m_server || !m_pDirectoryListing) {
 		return;
+	}
 
 	CServerPath path = m_pDirectoryListing->path;
 	if (subdir == _T("..") && path.HasParent())
@@ -651,8 +646,19 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const w
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly)
 {
-	if (!m_pServer)
+	if (!m_site.m_server) {
 		return;
+	}
+
+	if (path.empty()) {
+		return;
+	}
+
+	auto recursiveOperation = GetLocalRecursiveOperation();
+	if (!recursiveOperation || recursiveOperation->IsActive()) {
+		wxBell();
+		return;
+	}
 
 	const wxArrayString& files = pFileDataObject->GetFilenames();
 
@@ -663,19 +669,27 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		if (type == fz::local_filesys::file) {
 			wxString localFile;
 			const CLocalPath localPath(files[i], &localFile);
-			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
+			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.m_server, size);
 			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
 			CLocalPath localPath(files[i]);
-			if (localPath.HasParent())
-			{
-				CServerPath target = path;
-				target.AddSegment(localPath.GetLastSegment());
-				m_mainFrame.GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
+			if (localPath.HasParent()) {
+				
+				CServerPath remotePath = path;
+				if (!remotePath.ChangePath(localPath.GetLastSegment())) {
+					continue;
+				}
+
+				local_recursion_root root;
+				root.add_dir_to_visit(localPath, remotePath);
+				recursiveOperation->AddRecursionRoot(std::move(root));
 			}
 		}
 	}
+
+	CFilterManager filter;
+	recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queueOnly);
 }
 
 void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
@@ -701,9 +715,9 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	}
 	*p = 0; // End of list
 
-	wxChar* to = new wxChar[path.GetPath().Len() + 2];
+	wxChar* to = new wxChar[path.GetPath().size() + 2];
 	wxStrcpy(to, path.GetPath());
-	to[path.GetPath().Len() + 1] = 0; // End of list
+	to[path.GetPath().size() + 1] = 0; // End of list
 
 	SHFILEOPSTRUCT op = {0};
 	op.pFrom = from;
@@ -849,14 +863,10 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	}
 
 	if (!root.empty()) {
-		if (m_pComparisonManager->IsComparing())
-			m_pComparisonManager->ExitComparisonMode();
-
-		m_pRecursiveOperation->AddRecursionRoot(std::move(root));
+		m_pRemoteRecursiveOperation->AddRecursionRoot(std::move(root));
 
 		CFilterManager filter;
-		m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download,
-			filter.GetActiveFilters(false), pRemoteDataObject->GetServerPath());
+		m_pRemoteRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), pRemoteDataObject->GetServerPath(), !queueOnly);
 	}
 
 	return true;
@@ -867,12 +877,12 @@ bool CState::IsRemoteConnected() const
 	if (!m_pEngine)
 		return false;
 
-	return m_pServer != 0;
+	return static_cast<bool>(m_site.m_server);
 }
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (!ignore_recursive && m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRemoteRecursiveOperation && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
 		return false;
 
 	if (!m_pCommandQueue)
@@ -881,6 +891,14 @@ bool CState::IsRemoteIdle(bool ignore_recursive) const
 	return m_pCommandQueue->Idle(ignore_recursive ? CCommandQueue::normal : CCommandQueue::any);
 }
 
+bool CState::IsLocalIdle(bool ignore_recursive) const
+{
+	if (!ignore_recursive && m_pLocalRecursiveOperation && m_pLocalRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+		return false;
+
+	return true;
+}
+
 void CState::ListingFailed(int)
 {
 	if (m_sync_browse.is_changing && !m_sync_browse.target_path.empty()) {
@@ -895,7 +913,6 @@ void CState::ListingFailed(int)
 		xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the local directory"));
 		dlg.GetSizer()->Fit(&dlg);
 		if (dlg.ShowModal() == wxID_OK) {
-
 			if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
 				m_pCommandQueue->ProcessCommand(new CMkdirCommand(m_sync_browse.target_path));
 				m_pCommandQueue->ProcessCommand(new CListCommand(m_sync_browse.target_path));
@@ -922,7 +939,7 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 
 bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=_T("")*/, int flags /*=0*/, bool ignore_busy /*=false*/)
 {
-	if (!m_pServer || !m_pCommandQueue)
+	if (!m_site.m_server || !m_pCommandQueue)
 		return false;
 
 	if (!m_sync_browse.local_root.empty()) {
@@ -1117,3 +1134,46 @@ void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 	m_pSftpEncryptionInfo = std::make_unique<CSftpEncryptionNotification>(info);
 	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
+
+void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
+{
+	if (newSite.m_path.empty() || !newSite.m_server) {
+		return;
+	}
+
+	bool changed = false;
+	if (m_site.m_server && m_site != newSite) {
+		if (m_site.m_path == oldPath && m_site.m_server.EqualsNoPass(newSite.m_server)) {
+			m_site = newSite;
+			changed = true;
+		}
+	}
+	if (m_last_site.m_server && m_last_site != newSite) {
+		if (m_last_site.m_path == oldPath && m_last_site.m_server.EqualsNoPass(newSite.m_server)) {
+			m_last_site = newSite;
+			if (!m_site.m_server) {
+				// Active site has precedence over historic data
+				changed = true;
+			}
+		}
+	}
+	if (changed) {
+		UpdateTitle();
+		NotifyHandlers(STATECHANGE_SERVER);
+	}
+}
+
+void CState::UpdateTitle()
+{
+	if (m_site.m_server) {
+		wxString const& name = m_site.m_server.GetName();
+		m_title.clear();
+		if (!name.empty()) {
+			m_title = name + _T(" - ");
+		}
+		m_title += m_site.m_server.Format(ServerFormat::with_user_and_optional_port);
+	}
+	else {
+		m_title = _("Not connected");
+	}
+}
