@@ -2,6 +2,7 @@
 #define __STATE_H__
 
 #include "local_path.h"
+#include "sitemanager.h"
 
 #include <memory>
 
@@ -28,7 +29,10 @@ enum t_statechange_notifications
 	STATECHANGE_SYNC_BROWSE,
 	STATECHANGE_COMPARISON,
 
-	STATECHANGE_RECURSION_STATUS,
+	STATECHANGE_REMOTE_RECURSION_STATUS,
+	STATECHANGE_LOCAL_RECURSION_STATUS,
+
+	STATECHANGE_LOCAL_RECURSION_LISTING,
 
 	/* Global notifications */
 	STATECHANGE_QUEUEPROCESSING,
@@ -42,10 +46,12 @@ enum t_statechange_notifications
 class CDirectoryListing;
 class CFileZillaEngine;
 class CCommandQueue;
+class CLocalRecursiveOperation;
 class CMainFrame;
+class CGlobalStateEventHandler;
 class CStateEventHandler;
 class CRemoteDataObject;
-class CRecursiveOperation;
+class CRemoteRecursiveOperation;
 class CComparisonManager;
 
 class CState;
@@ -55,10 +61,10 @@ class CContextManager final
 public:
 	// If current_only is set, only notifications from the current (at the time
 	// of notification emission) context is dispatched to the handler.
-	void RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only);
-	void UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification);
+	void RegisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification, bool current_only);
+	void UnregisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification);
 
-	size_t HandlerCount(enum t_statechange_notifications notification) const;
+	size_t HandlerCount(t_statechange_notifications notification) const;
 
 	CState* CreateState(CMainFrame &mainFrame);
 	void DestroyState(CState* pState);
@@ -69,8 +75,8 @@ public:
 
 	static CContextManager* Get();
 
-	void NotifyAllHandlers(enum t_statechange_notifications notification, const wxString& data = _T(""), const void* data2 = 0);
-	void NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data = _T(""), const void* data2 = 0);
+	void NotifyAllHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
+	void NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
 
 	void SetCurrentContext(CState* pState);
 
@@ -82,12 +88,12 @@ protected:
 
 	struct t_handler
 	{
-		CStateEventHandler* pHandler;
+		CGlobalStateEventHandler* pHandler;
 		bool current_only;
 	};
 	std::vector<t_handler> m_handlers[STATECHANGE_MAX];
 
-	void NotifyHandlers(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	void NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2);
 
 	static CContextManager m_the_context_manager;
 };
@@ -109,7 +115,7 @@ public:
 	bool SetLocalDir(CLocalPath const& dir, wxString *error = 0, bool rememberPreviousSubdir = true);
 	bool SetLocalDir(const wxString& dir, wxString *error = 0, bool rememberPreviousSubdir = true);
 
-	bool Connect(const CServer& server, const CServerPath& path = CServerPath());
+	bool Connect(Site const& site, const CServerPath& path = CServerPath());
 	bool Disconnect();
 
 	bool ChangeRemoteDir(const CServerPath& path, const wxString& subdir = _T(""), int flags = 0, bool ignore_busy = false);
@@ -117,7 +123,8 @@ public:
 	std::shared_ptr<CDirectoryListing> GetRemoteDir() const;
 	const CServerPath GetRemotePath() const;
 
-	const CServer* GetServer() const;
+	Site const& GetSite() const;
+	CServer const* GetServer() const;
 	wxString GetTitle() const;
 
 	void RefreshLocal();
@@ -126,8 +133,8 @@ public:
 
 	bool RefreshRemote();
 
-	void RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, CStateEventHandler* insertBefore = 0);
-	void UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification);
+	void RegisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification, CStateEventHandler* insertBefore = 0);
+	void UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification);
 
 	CFileZillaEngine* m_pEngine{};
 	CCommandQueue* m_pCommandQueue{};
@@ -142,10 +149,12 @@ public:
 
 	bool IsRemoteConnected() const;
 	bool IsRemoteIdle(bool ignore_recursive = false) const;
+	bool IsLocalIdle(bool ignore_recursive = false) const;
 
-	CRecursiveOperation* GetRecursiveOperationHandler() { return m_pRecursiveOperation; }
+	CLocalRecursiveOperation* GetLocalRecursiveOperation() { return m_pLocalRecursiveOperation; }
+	CRemoteRecursiveOperation* GetRemoteRecursiveOperation() { return m_pRemoteRecursiveOperation; }
 
-	void NotifyHandlers(enum t_statechange_notifications notification, wxString const& data = wxString(), const void* data2 = 0);
+	void NotifyHandlers(t_statechange_notifications notification, wxString const& data = wxString(), const void* data2 = 0);
 
 	bool SuccessfulConnect() const { return m_successful_connect; }
 	void SetSuccessfulConnect() { m_successful_connect = true; }
@@ -156,10 +165,10 @@ public:
 	bool SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root = CServerPath());
 	bool GetSyncBrowse() const { return !m_sync_browse.local_root.empty(); }
 
-	CServer GetLastServer() const { return m_last_server; }
+	Site GetLastSite() const { return m_last_site; }
 	CServerPath GetLastServerPath() const { return m_last_path; }
-	void SetLastServer(const CServer& server, const CServerPath& path)
-	{ m_last_server = server; m_last_path = path; }
+	void SetLastSite(Site const& server, CServerPath const& path)
+		{ m_last_site = server; m_last_path = path; }
 
 	bool GetSecurityInfo(CCertificateNotification *& pInfo);
 	bool GetSecurityInfo(CSftpEncryptionNotification *& pInfo);
@@ -173,22 +182,28 @@ public:
 	void ClearPreviouslyVisitedLocalSubdir() { m_previouslyVisitedLocalSubdir = _T(""); }
 	void ClearPreviouslyVisitedRemoteSubdir() { m_previouslyVisitedRemoteSubdir = _T(""); }
 
+	void UpdateSite(wxString const& oldPath, Site const& newSite);
+
 protected:
-	void SetServer(const CServer* server, CServerPath const& path = CServerPath());
+	void SetSite(Site const& site, CServerPath const& path = CServerPath());
+
+	void UpdateTitle();
 
 	CLocalPath m_localDir;
 	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
-	CServer* m_pServer{};
+	Site m_site;
+	
 	wxString m_title;
 	bool m_successful_connect{};
 
-	CServer m_last_server;
+	Site m_last_site;
 	CServerPath m_last_path;
 
 	CMainFrame& m_mainFrame;
 
-	CRecursiveOperation* m_pRecursiveOperation;
+	CLocalRecursiveOperation* m_pLocalRecursiveOperation;
+	CRemoteRecursiveOperation* m_pRemoteRecursiveOperation;
 
 	CComparisonManager* m_pComparisonManager;
 
@@ -219,15 +234,24 @@ protected:
 	wxString m_previouslyVisitedRemoteSubdir;
 };
 
+class CGlobalStateEventHandler
+{
+public:
+	CGlobalStateEventHandler() = default;
+	virtual ~CGlobalStateEventHandler();
+
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
+};
+
 class CStateEventHandler
 {
 public:
-	CStateEventHandler(CState* pState);
+	CStateEventHandler(CState& state);
 	virtual ~CStateEventHandler();
 
-	CState* m_pState;
+	CState& m_state;
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
 };
 
 #endif
