@@ -24,6 +24,8 @@ wxStatusBarEx::wxStatusBarEx(wxTopLevelWindow* pParent)
 	m_columnWidths = 0;
 	Create(pParent, wxID_ANY);
 
+	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+
 #ifdef __WXMSW__
 	m_parentWasMaximized = false;
 
@@ -291,10 +293,11 @@ END_EVENT_TABLE()
 BEGIN_EVENT_TABLE(CStatusBar, CWidgetsStatusBar)
 EVT_MENU(XRCID("ID_SPEEDLIMITCONTEXT_ENABLE"), CStatusBar::OnSpeedLimitsEnable)
 EVT_MENU(XRCID("ID_SPEEDLIMITCONTEXT_CONFIGURE"), CStatusBar::OnSpeedLimitsConfigure)
+EVT_TIMER(wxID_ANY, CStatusBar::OnTimer)
 END_EVENT_TABLE()
 
 CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
-	: CWidgetsStatusBar(pParent), CStateEventHandler(0)
+	: CWidgetsStatusBar(pParent)
 {
 	// Speedlimits
 	RegisterOption(OPTION_SPEEDLIMIT_ENABLE);
@@ -311,15 +314,9 @@ CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
 	// Reload icons
 	RegisterOption(OPTION_THEME);
 
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true, false);
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
-
-	m_pDataTypeIndicator = 0;
-	m_pEncryptionIndicator = 0;
-	m_pSpeedLimitsIndicator = 0;
-
-	m_size = 0;
-	m_hasUnknownFiles = false;
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_ENCRYPTION, true);
 
 	const int count = 3;
 	SetFieldsCount(count);
@@ -336,25 +333,38 @@ CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
 	UpdateSpeedLimitsIcon();
 	DisplayDataType();
 	DisplayEncrypted();
+
+	m_queue_size_timer.SetOwner(this);
 }
 
 CStatusBar::~CStatusBar()
 {
 }
 
-void CStatusBar::DisplayQueueSize(wxLongLong totalSize, bool hasUnknown)
+void CStatusBar::DisplayQueueSize(int64_t totalSize, bool hasUnknown)
 {
 	m_size = totalSize;
 	m_hasUnknownFiles = hasUnknown;
 
-	if (totalSize == 0 && !hasUnknown)
-	{
+	if (m_queue_size_timer.IsRunning()) {
+		m_queue_size_changed = true;
+	}
+	else {
+		DoDisplayQueueSize();
+		m_queue_size_timer.Start(200, true);
+	}
+}
+
+void CStatusBar::DoDisplayQueueSize()
+{
+	m_queue_size_changed = false;
+	if (m_size == 0 && !m_hasUnknownFiles) {
 		SetStatusText(_("Queue: empty"), FIELD_QUEUESIZE);
 		return;
 	}
 
-	wxString queueSize = wxString::Format(_("Queue: %s%s"), hasUnknown ? _T(">") : _T(""),
-		CSizeFormat::Format(totalSize, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces));
+	wxString queueSize = wxString::Format(_("Queue: %s%s"), m_hasUnknownFiles ? _T(">") : _T(""),
+		CSizeFormat::Format(m_size, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces));
 
 	SetStatusText(queueSize, FIELD_QUEUESIZE);
 }
@@ -366,40 +376,33 @@ void CStatusBar::DisplayDataType()
 	if (pState)
 		pServer = pState->GetServer();
 
-	if (!pServer || !CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol()))
-	{
-		if (m_pDataTypeIndicator)
-		{
+	if (!pServer || !CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol())) {
+		if (m_pDataTypeIndicator) {
 			RemoveField(widget_datatype);
 			m_pDataTypeIndicator->Destroy();
 			m_pDataTypeIndicator = 0;
 		}
 	}
-	else
-	{
+	else {
 		wxString name;
 		wxString desc;
 
 		const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
-		if (type == 1)
-		{
+		if (type == 1) {
 			name = _T("ART_ASCII");
 			desc = _("Current transfer type is set to ASCII.");
 		}
-		else if (type == 2)
-		{
+		else if (type == 2) {
 			name = _T("ART_BINARY");
 			desc = _("Current transfer type is set to binary.");
 		}
-		else
-		{
+		else {
 			name = _T("ART_AUTO");
 			desc = _("Current transfer type is set to automatic detection.");
 		}
 
 		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-		if (!m_pDataTypeIndicator)
-		{
+		if (!m_pDataTypeIndicator) {
 			m_pDataTypeIndicator = new CIndicator(this, bmp);
 			AddField(0, widget_datatype, m_pDataTypeIndicator);
 		}
@@ -431,24 +434,31 @@ void CStatusBar::MeasureQueueSizeWidth()
 void CStatusBar::DisplayEncrypted()
 {
 	const CServer* pServer = 0;
-	const CState* pState = CContextManager::Get()->GetCurrentContext();
+	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (pState)
 		pServer = pState->GetServer();
 
-	if (!pServer || (pServer->GetProtocol() != FTPS && pServer->GetProtocol() != FTPES && pServer->GetProtocol() != SFTP))
-	{
-		if (m_pEncryptionIndicator)
-		{
+	bool encrypted = false;
+	if (pServer) {
+		CCertificateNotification* info;
+		if (pServer->GetProtocol() == FTPS || pServer->GetProtocol() == FTPES || pServer->GetProtocol() == SFTP) {
+			encrypted = true;
+		}
+		else if (pServer->GetProtocol() == FTP && pState->GetSecurityInfo(info)) {
+			encrypted = true;
+		}
+	}
+
+	if (!encrypted) {
+		if (m_pEncryptionIndicator) {
 			RemoveField(widget_encryption);
 			m_pEncryptionIndicator->Destroy();
 			m_pEncryptionIndicator = 0;
 		}
 	}
-	else
-	{
+	else {
 		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER,  CThemeProvider::GetIconSize(iconSizeSmall));
-		if (!m_pEncryptionIndicator)
-		{
+		if (!m_pEncryptionIndicator) {
 			m_pEncryptionIndicator = new CIndicator(this, bmp);
 			AddField(0, widget_encryption, m_pEncryptionIndicator);
 			m_pEncryptionIndicator->SetToolTip(_("The connection is encrypted. Click icon for details."));
@@ -476,58 +486,36 @@ void CStatusBar::UpdateSizeFormat()
 
 void CStatusBar::OnHandleLeftClick(wxWindow* pWnd)
 {
-	if (pWnd == m_pEncryptionIndicator)
-	{
+	if (pWnd == m_pEncryptionIndicator) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
 		CCertificateNotification *pCertificateNotification = 0;
 		CSftpEncryptionNotification *pSftpEncryptionNotification = 0;
-		if (pState->GetSecurityInfo(pCertificateNotification))
-		{
+		if (pState->GetSecurityInfo(pCertificateNotification)) {
 			CVerifyCertDialog dlg;
-			dlg.ShowVerificationDialog(pCertificateNotification, true);
+			dlg.ShowVerificationDialog(*pCertificateNotification, true);
 		}
-		else if (pState->GetSecurityInfo(pSftpEncryptionNotification))
-		{
+		else if (pState->GetSecurityInfo(pSftpEncryptionNotification)) {
 			CSftpEncryptioInfoDialog dlg;
 			dlg.ShowDialog(pSftpEncryptionNotification);
 		}
 		else
 			wxMessageBoxEx(_("Certificate and session data are not available yet."), _("Security information"));
 	}
-	else if (pWnd == m_pSpeedLimitsIndicator)
-	{
+	else if (pWnd == m_pSpeedLimitsIndicator) {
 		CSpeedLimitsDialog dlg;
 		dlg.Run(m_pParent);
 	}
+	else if (pWnd == m_pDataTypeIndicator) {
+		ShowDataTypeMenu();
+	}
 }
 
 void CStatusBar::OnHandleRightClick(wxWindow* pWnd)
 {
-	if (pWnd == m_pDataTypeIndicator)
-	{
-		wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TRANSFER_TYPE_CONTEXT"));
-		if (!pMenu)
-			return;
-
-		const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
-		switch (type)
-		{
-		case 1:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), true);
-			break;
-		case 2:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
-			break;
-		default:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_AUTO"), true);
-			break;
-		}
-
-		PopupMenu(pMenu);
-		delete pMenu;
+	if (pWnd == m_pDataTypeIndicator) {
+		ShowDataTypeMenu();
 	}
-	else if (pWnd == m_pSpeedLimitsIndicator)
-	{
+	else if (pWnd == m_pSpeedLimitsIndicator) {
 		wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_SPEEDLIMITCONTEXT"));
 		if (!pMenu)
 			return;
@@ -544,24 +532,49 @@ void CStatusBar::OnHandleRightClick(wxWindow* pWnd)
 	}
 }
 
+void CStatusBar::ShowDataTypeMenu()
+{
+	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TRANSFER_TYPE_CONTEXT"));
+	if (!pMenu)
+		return;
+
+	const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
+	switch (type)
+	{
+	case 1:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), true);
+		break;
+	case 2:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
+		break;
+	default:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_AUTO"), true);
+		break;
+	}
+
+	PopupMenu(pMenu);
+	delete pMenu;
+}
+
 void CStatusBar::UpdateSpeedLimitsIcon()
 {
 	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
 
 	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+	if (!bmp.Ok()) {
+		return;
+	}
 	wxString tooltip;
 
 	int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
 	int uploadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
-	if (!enable || (!downloadLimit && !uploadLimit))
-	{
+	if (!enable || (!downloadLimit && !uploadLimit)) {
 		wxImage img = bmp.ConvertToImage();
 		img = img.ConvertToGreyscale();
 		bmp = wxBitmap(img);
 		tooltip = _("Speed limits are disabled, click to change.");
 	}
-	else
-	{
+	else {
 		tooltip = _("Speed limits are enabled, click to change.");
 		tooltip += _T("\n");
 		if (downloadLimit)
@@ -575,8 +588,7 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 			tooltip += _("Upload limit: none");
 	}
 
-	if (!m_pSpeedLimitsIndicator)
-	{
+	if (!m_pSpeedLimitsIndicator) {
 		m_pSpeedLimitsIndicator = new CIndicator(this, bmp);
 		AddField(0, widget_speedlimit, m_pSpeedLimitsIndicator);
 	}
@@ -585,40 +597,33 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 	m_pSpeedLimitsIndicator->SetToolTip(tooltip);
 }
 
-void CStatusBar::OnOptionChanged(int option)
+void CStatusBar::OnOptionsChanged(changed_options_t const& options)
 {
-	switch (option)
-	{
-	case OPTION_SPEEDLIMIT_ENABLE:
-	case OPTION_SPEEDLIMIT_INBOUND:
-	case OPTION_SPEEDLIMIT_OUTBOUND:
+	if (options.test(OPTION_SPEEDLIMIT_ENABLE) || options.test(OPTION_SPEEDLIMIT_INBOUND) || options.test(OPTION_SPEEDLIMIT_OUTBOUND)) {
 		UpdateSpeedLimitsIcon();
-		break;
-	case OPTION_SIZE_FORMAT:
-	case OPTION_SIZE_USETHOUSANDSEP:
-	case OPTION_SIZE_DECIMALPLACES:
+	}
+	if (options.test(OPTION_SIZE_FORMAT) || options.test(OPTION_SIZE_USETHOUSANDSEP) || options.test(OPTION_SIZE_DECIMALPLACES)) {
 		UpdateSizeFormat();
-		break;
-	case OPTION_ASCIIBINARY:
+	}
+	if (options.test(OPTION_ASCIIBINARY)) {
 		DisplayDataType();
-		break;
-	case OPTION_THEME:
+	}
+	if (options.test(OPTION_THEME)) {
 		DisplayDataType();
 		UpdateSpeedLimitsIcon();
 		DisplayEncrypted();
-		break;
-	default:
-		break;
 	}
 }
 
-void CStatusBar::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
+void CStatusBar::OnStateChange(CState*, t_statechange_notifications notification, const wxString&, const void*)
 {
-	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT)
-	{
+	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT) {
 		DisplayDataType();
 		DisplayEncrypted();
 	}
+	else if (notification == STATECHANGE_ENCRYPTION) {
+		DisplayEncrypted();
+	}
 }
 
 void CStatusBar::OnSpeedLimitsEnable(wxCommandEvent&)
@@ -645,3 +650,11 @@ void CStatusBar::OnSpeedLimitsConfigure(wxCommandEvent&)
 	CSpeedLimitsDialog dlg;
 	dlg.Run(m_pParent);
 }
+
+void CStatusBar::OnTimer(wxTimerEvent&)
+{
+	if (m_queue_size_changed && !m_queue_size_timer.IsRunning()) {
+		DoDisplayQueueSize();
+		m_queue_size_timer.Start(200, true);
+	}
+}
