@@ -4,6 +4,7 @@
 #include "state.h"
 
 class CComboBoxEx;
+class CWindowTinter;
 class CViewHeader : public wxNavigationEnabled<wxWindow>
 {
 	friend class CComboBoxEx;
@@ -37,19 +38,22 @@ protected:
 #endif //__WXMSW__
 
 	void AddRecentDirectory(const wxString &directory);
-	std::list<wxString> m_recentDirectories;
+	std::deque<wxString> m_recentDirectories; // ordered by age
+	std::list<wxString> m_sortedRecentDirectories; // ordered by sort order
 
 	int m_cbOffset{};
 	int m_labelHeight{};
+
+	std::unique_ptr<CWindowTinter> m_windowTinter;
 };
 
 class CLocalViewHeader : public CViewHeader, CStateEventHandler
 {
 public:
-	CLocalViewHeader(wxWindow* pParent, CState* pState);
+	CLocalViewHeader(wxWindow* pParent, CState& state);
 
 protected:
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	wxString m_oldValue;
 
@@ -67,10 +71,10 @@ protected:
 class CRemoteViewHeader : public CViewHeader, CStateEventHandler
 {
 public:
-	CRemoteViewHeader(wxWindow* pParent, CState* pState);
+	CRemoteViewHeader(wxWindow* pParent, CState& state);
 
 protected:
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	CServer m_lastServer;
 	CServerPath m_path;
