@@ -1,4 +1,4 @@
-#define FZSFTP_PROTOCOL_VERSION 4
+#define FZSFTP_PROTOCOL_VERSION 6
 
 typedef enum
 {
@@ -10,9 +10,11 @@ typedef enum
     sftpStatus,
     sftpRecv, /* socket */
     sftpSend, /* socket */
-    sftpClose,
-    sftpRequest,
     sftpListentry,
+    sftpAskHostkey,
+    sftpAskHostkeyChanged,
+    sftpAskHostkeyBetteralg,
+    sftpAskPassword,
     sftpTransfer, /* payload: when written to local file (download) or acknowledged by server (upload) */
     sftpRequestPreamble,
     sftpRequestInstruction,
@@ -20,7 +22,7 @@ typedef enum
     sftpUsedQuotaSend,
     sftpKexAlgorithm,
     sftpKexHash,
-	sftpKexCurve,
+    sftpKexCurve,
     sftpCipherClientToServer,
     sftpCipherServerToClient,
     sftpMacClientToServer,
@@ -28,16 +30,14 @@ typedef enum
     sftpHostkey
 } sftpEventTypes;
 
-enum sftpRequestTypes
-{
-    sftpReqPassword,
-    sftpReqHostkey,
-    sftpReqHostkeyChanged,
-    sftpReqUnknown
-};
-
 int fznotify(sftpEventTypes type);
+
+// Format the string. Each line of the string is prepended by type
 int fzprintf(sftpEventTypes type, const char* p, ...);
+
+// Format the string, then print the type and the string as-is. Caller is responsible to add trailing linebreak
 int fzprintf_raw(sftpEventTypes type, const char* p, ...);
+
+// Format the string, then print the type (if not sftpUnknown) and the string with linebreaks replaced by spaces.
 int fzprintf_raw_untrusted(sftpEventTypes type, const char* p, ...);
 int fznotify1(sftpEventTypes type, int data);
