@@ -493,7 +493,7 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
     xfer = xfer_download_init(fh, offset);
     while (!xfer_done(xfer)) {
 	void *vbuf;
-	int ret, len;
+	int len;
 	int wpos, wlen;
 
 	xfer_download_queue(xfer);
@@ -516,8 +516,10 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
 	    while (file && wpos < len) {
 		wlen = write_to_file(file, buf + wpos, len - wpos);
 		if (wlen <= 0) {
-		    fzprintf(shown_err ? sftpStatus : sftpError, "error while writing local file");
-		    shown_err = TRUE;
+		    if (!shown_err) {
+			fzprintf(sftpError, "error while writing local file");
+			shown_err = TRUE;
+		    }
 		    ret = 0;
 		    xfer_set_error(xfer);
 		    break;
@@ -1238,7 +1240,9 @@ int sftp_cmd_ls(struct sftp_command *cmd)
 	    if (ournames[i]->attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
 		mtime = ournames[i]->attrs.mtime;
 	    }
-	    fzprintf(sftpListentry, "%lu %s", mtime, ournames[i]->longname);
+	    fzprintf_raw_untrusted(sftpListentry, "%s", ournames[i]->longname);
+	    fzprintf_raw_untrusted(sftpUnknown, "%lu", mtime);
+	    fzprintf_raw_untrusted(sftpUnknown, "%s", ournames[i]->filename);
 	    fxp_free_name(ournames[i]);
 	}
 	sfree(ournames);
@@ -2970,6 +2974,10 @@ int sftp_senddata(char *buf, int len)
     back->send(backhandle, buf, len);
     return 1;
 }
+int sftp_sendbuffer(void)
+{
+    return back->sendbuffer(backhandle);
+}
 
 /*
  *  Short description of parameters.
@@ -3433,6 +3441,8 @@ int psftp_main(int argc, char *argv[])
     argv += i;
     back = NULL;
 
+    platform_psftp_post_option_setup();
+
     /*
      * If the loaded session provides a hostname, and a hostname has not
      * otherwise been specified, pop it in `userhost' so that
