@@ -1387,7 +1387,7 @@ struct fxp_xfer *xfer_upload_init(struct fxp_handle *fh, uint64 offset)
 
 int xfer_upload_ready(struct fxp_xfer *xfer)
 {
-    if (xfer->req_totalsize < xfer->req_maxsize)
+    if (sftp_sendbuffer() == 0)
 	return 1;
     else
 	return 0;
@@ -1490,6 +1490,7 @@ void xfer_cleanup(struct fxp_xfer *xfer)
     sfree(xfer);
 }
 
+int recv_peek(Socket sk, char* buf, int len);
 int pending_receive()
 {
     char tmp[5];
