@@ -12,6 +12,10 @@
 #include <termios.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/time.h>
+#ifndef HAVE_NO_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 
 #include "putty.h"
 #include "storage.h"
@@ -131,7 +135,6 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"If you do not trust this host, press Return to abandon the\n"
 	"connection.\n"
 	"Store key in cache? (y/n) ";
-*/
     static const char wrongmsg_batch[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
@@ -141,7 +144,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"The new %s key fingerprint is:\n"
 	"%s\n"
 	"Connection abandoned.\n";
-/*    static const char wrongmsg[] =
+    static const char wrongmsg[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
 	"cached. This means that either the server administrator\n"
@@ -172,20 +175,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	return 1;
 
 //FZ premsg(&cf);
-    if (ret == 2) {		       /* key was different */
-	if (console_batch_mode) {
-	    fprintf(stderr, wrongmsg_batch, keytype, fingerprint);
-	    return 0;
-	}
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkeyChanged, host, port, fingerprint);
-    }
-    if (ret == 1) {		       /* key was absent */
-	if (console_batch_mode) {
-	    fprintf(stderr, absentmsg_batch, keytype, fingerprint);
-	    return 0;
-	}
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkey, host, port, fingerprint);
-    }
+    fzprintf_raw((ret == 1) ? sftpAskHostkey : sftpAskHostkeyChanged, "%s\n%d\n%s\n", host, port, fingerprint);
 
     {
 	struct termios oldmode, newmode;
@@ -209,7 +199,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 //FZ	postmsg(&cf);
         return 1;
     } else {
-	fprintf(stderr, abandoned);
+	fzprintf(sftpError, abandoned);
 //FZ	postmsg(&cf);
         return 0;
     }
@@ -226,6 +216,61 @@ int askalg(void *frontend, const char *algtype, const char *algname,
     return 0;
 }
 
+int askhk(void *frontend, const char *algname, const char *betteralgs,
+          void (*callback)(void *ctx, int result), void *ctx)
+{
+    /*static const char msg[] =
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
+	"Continue with connection? (y/n) ";*/
+    static const char msg_batch[] =
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
+	"Connection abandoned.\n";
+    static const char abandoned[] = "Connection abandoned.";
+
+    char line[32];
+//FZ    struct termios cf;
+
+//FZ    premsg(&cf);
+    if (console_batch_mode) {
+	fprintf(stderr, msg_batch, algname, betteralgs);
+	return 0;
+    }
+
+    fzprintf_raw(sftpAskHostkeyBetteralg, "%d%s\n%s\n", algname, betteralgs);
+
+    {
+	struct termios oldmode, newmode;
+	tcgetattr(0, &oldmode);
+	newmode = oldmode;
+	newmode.c_lflag |= ISIG | ICANON;
+	tcsetattr(0, TCSANOW, &newmode);
+	line[0] = '\0';
+	int ret;
+	do
+	{
+	    ret = read(0, line, sizeof(line) - 1);
+	} while (ret == -1 && (errno == EINTR || errno == EAGAIN));
+
+	tcsetattr(0, TCSANOW, &oldmode);
+    }
+
+    if (line[0] == 'y' || line[0] == 'Y') {
+//FZ	postmsg(&cf);
+	return 1;
+    } else {
+	fzprintf(sftpError, abandoned);
+//FZ	postmsg(&cf);
+	return 0;
+    }
+}
 /*
  * Ask whether to wipe a session log file before writing to it.
  * Returns 2 for wipe, 1 for append, 0 for cancel (don't log).
@@ -384,13 +429,13 @@ int console_get_userpass_input(prompts_t *p, const unsigned char *in,
      */
     /* We only print the `name' caption if we have to... */
     if (p->name)
-	fzprintf_raw_untrusted(sftpRequestPreamble, p->name);
+	fzprintf_raw_untrusted(sftpRequestPreamble, "%s", p->name);
     else
 	fzprintf_raw_untrusted(sftpRequestPreamble, "");
 
     /* ...but we always print any `instruction'. */
     if (p->instruction)
-	fzprintf_raw_untrusted(sftpRequestInstruction, p->instruction);
+	fzprintf_raw_untrusted(sftpRequestInstruction, "%s", p->instruction);
     else
 	fzprintf_raw_untrusted(sftpRequestInstruction, "");
 
@@ -409,7 +454,7 @@ int console_get_userpass_input(prompts_t *p, const unsigned char *in,
 //	    newmode.c_lflag |= ECHO;
 	tcsetattr(infd, TCSANOW, &newmode);
 
-	fzprintf_raw_untrusted(sftpRequest, "%d%s\n", (int)sftpReqPassword, pr->prompt);
+	fzprintf_raw_untrusted(sftpAskPassword, "%s", pr->prompt);
 
         len = 0;
         while (1) {
